<?php
class Utils
{
    public static function isAuthenticated($post)
    {
        global $wpdb;

        if (!isset($post["apikey"]) || !isset($post["apisecret"])) {
            return false;
        } else {
            if ($post["apikey"] == null || $post["apisecret"] == null || !preg_match("/^[a-zA-Z0-9]+$/", $post["apikey"])) {
                return false;
            }

            $sql = "SELECT COUNT(id) as total FROM " . TABLE_ACCESS_KEYS . " WHERE ";
            $sql .= "api_key=%s AND api_secret=%s AND status='1';";
            $total = $wpdb->get_var($wpdb->prepare($sql, $post["apikey"], $post["apisecret"]));
            return is_numeric($total) ? ($total > 0) : false;
        }
        return false;
    }

    /**
     * This method takes data from an API request, validates any required params and if all
     * required parameters are present and valid, returns an object will all the data in it. 
     * Otherwise, it returns an MM_Response object with an error message
     * @param array $post API request data
     * @param stdClass $requiredParams key/value pairs of required fields and a regular expression to validate it's contents
     * @return stdClass|MM_Response returns an object with API request data or an MM_Response object if there was an error
     */
    public static function processApiRequestData($post, $requiredParams)
    {
        $data = new stdClass();
        foreach ($requiredParams as $key => $regex) {
            if (!isset($post[$key]) || (isset($post[$key]) && !preg_match("/" . $regex . "/", $post[$key]))) {
                return new MM_Response(RESPONSE_ERROR_MESSAGE_MISSING_PARAMS . " : " . $key, MM_Response::$ERROR);
            } else {
                $data->$key = stripslashes($post[$key]);
            }
        }

        foreach ($post as $k => $v) {
            $data->$k = $v;
        }
        return $data;
    }
}
