<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * Local Version
 */
class MM_License
{
    private $id;
    private $memberId;
    private $name;
    private $url;
    private $apiKey;
    private $authKey;
    private $apiSecret;
    private $majorVersion;
    private $minorVersion;
    private $permissionsProfileId;
    private $permissions;
    private $status;
    private $isArchivedFlag;
    private $isStaging;
    private $lastUpdated;
    private $dateAdded;
    private $isUnlicensed = false;

    protected static $license = null;

    public function __construct() {}

    public function initialize()
    {
        MM_MemberMouseService::getLicense($this);
    }

    /**
     * Fetch and return the current license
     * 
     * @return MM_License An MM_License representing the current active license
     */
    public static function getCurrentLicense()
    {
        if (self::$license == null) {
            self::$license = new MM_License();
            self::$license->initialize();
        }
        return self::$license;
    }

    public function setData($data)
    {
        $this->id = $data->id;
        $this->memberId = $data->memberId;
        $this->name = $data->name;
        $this->url = $data->url;
        $this->apiKey = $data->apiKey;
        $this->apiSecret = $data->apiSecret;
        $this->majorVersion = $data->majorVersion;
        $this->minorVersion = $data->minorVersion;
        $this->permissionsProfileId = $data->permissionsProfileId;
        $this->permissions = $data->permissions;
        $this->status = $data->status;
        $this->isArchivedFlag = $data->isArchivedFlag;
        $this->isStaging = $data->isStaging;
        $this->dateAdded = $data->dateAdded;
        $this->lastUpdated = $data->lastUpdated;
    }

    public function commitData() {}


    /** GETTERS / SETTERS **/

    public function setId($id)
    {
        $this->id = $id;
    }

    public function getId()
    {
        return $this->id;
    }

    public function getAuthKey()
    {
        return $this->authKey;
    }

    public function setAuthKey($authKey)
    {
        $this->authKey = $authKey;
    }

    public function setMemberId($str)
    {
        $this->memberId = $str;
    }

    public function getMemberId()
    {
        return $this->memberId;
    }

    public function setName($str)
    {
        $this->name = $str;
    }

    public function getName()
    {
        return $this->name;
    }

    public function setUrl($str)
    {
        $this->url = $str;
    }

    public function getUrl()
    {
        return $this->url;
    }

    public function setApiKey($str)
    {
        $this->apiKey = $str;
    }

    public function getApiKey()
    {
        return $this->apiKey;
    }

    public function setApiSecret($str)
    {
        $this->apiSecret = $str;
    }

    public function getApiSecret()
    {
        return $this->apiSecret;
    }

    public function setMajorVersion($str)
    {
        $this->majorVersion = $str;
    }

    public function getMajorVersion()
    {
        return $this->majorVersion;
    }

    public function setMinorVersion($str)
    {
        $this->minorVersion = $str;
    }

    public function getMinorVersion()
    {
        return $this->minorVersion;
    }

    public function setPermissionsProfileId($str)
    {
        $this->permissionsProfileId = $str;
    }

    public function getpermissionsProfileId()
    {
        return $this->permissionsProfileId;
    }

    /**
     * Returns data about a license.
     * 
     * @param  integer|string $profileId A permissions profile ID, or empty string for current license data.
     * @param  string $type The type of data to retrieve. Currently, either "name" or "slug".
     * @return string The requested license data.
     */
    public function getLicenseType($profileId = "", $type = "name"): string
    {
        if (empty($profileId) || !is_numeric($profileId)) {
            $profileId = $this->getpermissionsProfileId();
        }

        $licenseData = $this->getLicenseDataArray();
        return $licenseData[$profileId][$type] ?? $licenseData["unknown"][$type] ?? "Unknown";
    }

    /**
     * Returns an array of license data
     *
     * @return array License data.
     */
    private function getLicenseDataArray(): array
    {
        return [
            -1 => [
                "name" => "Custom",
                "slug" => "membermouse-custom",
            ],
            0 => [
                "name" => "Unlicensed",
                "slug" => "membermouse-unlicensed",
            ],
            1 => [
                "name" => "Free (Legacy)",
                "slug" => "membermouse-free",
            ],
            2  => [
                "name" => "Starter V1 (Legacy)",
                "slug" => "membermouse-starter-v1",
            ],
            3  => [
                "name" => "Growth V1 (Legacy)",
                "slug" => "membermouse-growth-v1",
            ],
            4  => [
                "name" => "Professional V1 (Legacy)",
                "slug" => "membermouse-professional-v1",
            ],
            5  => [
                "name" => "Starter (Legacy)",
                "slug" => "membermouse-starter",
            ],
            6  => [
                "name" => "Builder (Legacy)",
                "slug" => "membermouse-builder",
            ],
            7  => [
                "name" => "Growth (Legacy)",
                "slug" => "membermouse-growth",
            ],
            10 => [
                "name" => "Advanced (Legacy)",
                "slug" => "membermouse-advanced",
            ],
            8  => [
                "name" => "Premium (Legacy)",
                "slug" => "membermouse-premium",
            ],
            9  => [
                "name" => "Professional (Legacy)",
                "slug" => "membermouse-professional",
            ],
            11 => [
                "name" => "Advanced Lite (Legacy)",
                "slug" => "membermouse-advanced-lite",
            ],
            100 => [
                "name" => "Starter V3 (Legacy)",
                "slug" => "membermouse-unknown",
            ],
            101 => [
                "name" => "Growth V3 (Legacy)",
                "slug" => "membermouse-unknown",
            ],
            102 => [
                "name" => "Advanced V3 (Legacy)",
                "slug" => "membermouse-unknown",
            ],
            103 => [
                "name" => "Developer (Legacy)",
                "slug" => "membermouse-unknown",
            ],
            104 => [
                "name" => "Startup (Legacy)",
                "slug" => "membermouse-startup",
            ],
            105 => [
                "name" => "Basic",
                "slug" => "membermouse-basic",
            ],
            106 => [
                "name" => "Plus",
                "slug" => "membermouse-plus",
            ],
            107 => [
                "name" => "Pro",
                "slug" => "membermouse-pro",
            ],
            108 => [
                "name" => "Elite",
                "slug" => "membermouse-elite",
            ],
            "unknown" => [
                "name" => "Unknown",
                "slug" => "membermouse-unknown",
            ],
        ];
    }

    public function setPermissions($str)
    {
        $this->permissions = $str;
    }

    public function getPermissions()
    {
        return $this->permissions;
    }

    public function setStatus($str)
    {
        $this->status = $str;
    }

    public function getStatus()
    {
        return $this->status;
    }

    public function setLastUpdated($str)
    {
        $this->lastUpdated = $str;
    }

    public function getDateAdded()
    {
        return $this->dateAdded;
    }

    public function setDateAdded($str)
    {
        $this->dateAdded = $str;
    }

    public function getLastUpdated()
    {
        return $this->lastUpdated;
    }

    public function setAsStaging($str)
    {
        $this->isStaging = $str;
    }

    public function isStaging()
    {
        return ($this->isStaging == "1") ? true : false;
    }

    public function isArchived()
    {
        return ($this->isArchivedFlag == "1") ? true : false;
    }

    public function setAsUnlicensed($isUnlicensed = true)
    {
        $this->isUnlicensed = boolval($isUnlicensed);
    }

    public function isUnlicensed()
    {
        return ($this->isUnlicensed === true);
    }


    public function isValid()
    {
        //For compatibility

        //TODO: check status also?
        return !$this->isUnlicensed();
    }
}
