<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_Utils
{
    /*
 	 * Modified and taken from http://php.net/manual/en/function.get-browser.php#101125 
 	 */
    public static function getClientBrowsingInfo()
    {
        $u_agent = $_SERVER['HTTP_USER_AGENT'];
        $bname = 'Unknown';
        $platform = 'Unknown';
        $version = "";
        $ip = $_SERVER['REMOTE_ADDR'];

        //First get the platform?
        if (preg_match('/linux/i', $u_agent)) {
            $platform = 'linux';
        } elseif (preg_match('/macintosh|mac os x/i', $u_agent)) {
            $platform = 'mac';
        } elseif (preg_match('/windows|win32/i', $u_agent)) {
            $platform = 'windows';
        }

        // Next get the name of the useragent yes seperately and for good reason
        if (preg_match('/MSIE/i', $u_agent) && !preg_match('/Opera/i', $u_agent)) {
            $bname = 'Internet Explorer';
            $ub = "MSIE";
        } elseif (preg_match('/Firefox/i', $u_agent)) {
            $bname = 'Mozilla Firefox';
            $ub = "Firefox";
        } elseif (preg_match('/Chrome/i', $u_agent)) {
            $bname = 'Google Chrome';
            $ub = "Chrome";
        } elseif (preg_match('/Safari/i', $u_agent)) {
            $bname = 'Apple Safari';
            $ub = "Safari";
        } elseif (preg_match('/Opera/i', $u_agent)) {
            $bname = 'Opera';
            $ub = "Opera";
        } elseif (preg_match('/Netscape/i', $u_agent)) {
            $bname = 'Netscape';
            $ub = "Netscape";
        }

        // finally get the correct version number
        $known = array('Version', $ub, 'other');
        $pattern = '#(?<browser>' . join('|', $known) .
            ')[/ ]+(?<version>[0-9.|a-zA-Z.]*)#';
        if (!preg_match_all($pattern, $u_agent, $matches)) {
            // we have no matching number just continue
        }

        // see how many we have
        $i = count($matches['browser']);
        if ($i != 1) {
            //we will have two since we are not using 'other' argument yet
            //see if version is before or after the name
            if (strripos($u_agent, "Version") < strripos($u_agent, $ub)) {
                $version = $matches['version'][0];
            } else {
                $version = $matches['version'][1];
            }
        } else {
            $version = $matches['version'][0];
        }

        // check if we have a number
        if ($version == null || $version == "") {
            $version = "?";
        }

        return array(
            'userAgent' => $u_agent,
            'name'      => $bname,
            'version'   => $version,
            'platform'  => $platform,
            'pattern'   => $pattern,
            'ip'        => $ip
        );
    }

    /**
     * Generates HTML parameters appropriate for the current browser that will prevent autocomplete and autocorrect
     *
     * @param $isPasswordField boolean Indicates that the field is a password field
     * @param $includeSafariParams boolean Indicates that the Safari parameters to prevent autocorrect should be included
     * @return string HTML parameters and values
     */
    public static function preventAutocomplete($isPasswordField = false, $includeSafariParams = true)
    {
        $browserInfo = MM_Utils::getClientBrowsingInfo();
        $browserName = $browserInfo['name'];
        $param = "off";

        if ($isPasswordField) {
            $param = (strpos($browserName, "Firefox") !== false) ? "new-password" : "one-time-code";
        } else {
            $param = (strpos($browserName, "Chrome") !== false) ? "one-time-code" : "off";
        }

        $autocompleteString = 'autocomplete="' . $param . '"';

        if ($includeSafariParams && strpos($browserName, "Safari") !== false) {
            $autocompleteString .= ' autocorrect="off" autocapitalize="off" spellcheck="false"';
        }

        return $autocompleteString;
    }

    public static function releaseDBLock($id)
    {
        global $wpdb;

        if (!empty($id) && !is_null($id) && strlen($id) > 0) {
            $key = "{$wpdb->dbname}_order_submission_{$id}";
            return $wpdb->query("SELECT RELEASE_LOCK('{$key}')");
        }
        return false;
    }

    public static function acquireDBLock($id, $timeout = 60)
    {
        global $wpdb;
        $id = trim($id);

        if (!empty($id) && !is_null($id) && strlen($id) > 0) {
            $key = "{$wpdb->dbname}_order_submission_{$id}";
            return $wpdb->get_var($wpdb->prepare("SELECT COALESCE(GET_LOCK(%s,{$timeout}),0)", $key));
        }
        return false;
    }

    public static function isMemberMouseActive()
    {
        $plugins = get_option('active_plugins');
        $required_plugin = MM_PLUGIN_NAME . '/index.php';
        $mmActive = false;

        if (in_array($required_plugin, $plugins)) {
            $mmActive = true;
        }

        return $mmActive;
    }

    public static function isAndroid()
    {
        if (isset($_SERVER['HTTP_USER_AGENT'])) {
            $ua = strtolower($_SERVER['HTTP_USER_AGENT']);
            return strpos($ua, 'android') !== false;
        }
        return false;
    }

    public static function sendRequest($url, $params, $post = 1)
    {
        $ch = curl_init($url);

        if ($post == 1) {
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
            curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
            curl_setopt($ch, CURLOPT_POSTREDIR, 7);
        } else {
            curl_setopt($ch, CURLOPT_POST, $post);
        }
        curl_setopt($ch, CURLOPT_USERAGENT, MM_CURL_USER_AGENT);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_HEADER, 0);  // DO NOT RETURN HTTP HEADERS
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);  // RETURN THE CONTENTS OF THE CALL
        $contents = curl_exec($ch);
        curl_close($ch);
        return $contents;
    }

    public static function getJsFiles($directory, $recursive = false, $includeDirs = false, $pattern = '/.*/')
    {
        $items = array();

        if ($handle = opendir($directory)) {
            while (($file = readdir($handle)) !== false) {
                if ($file != '.' && $file != '..') {
                    $path = "$directory/$file";
                    $path = preg_replace('#//#si', '/', $path);
                    if (is_dir($path)) {
                        if ($includeDirs) {
                            $items[] = $path;
                        }
                        if ($recursive) {
                            $items = array_merge($items, self::getJsFiles($path, true, $includeDirs, $pattern));
                        }
                    } else {
                        if (preg_match($pattern, $file)) {
                            $items[] = $path;
                        }
                    }
                }
            }

            closedir($handle);
        }
        sort($items);

        return $items;
    }

    public static function startExecutionTimer()
    {
        $mtime = microtime();
        $mtime = explode(" ", $mtime);
        $mtime = $mtime[1] + $mtime[0];
        $starttime = $mtime;
        return $starttime;
    }

    public static function getTotalExecutionTime($starttime)
    {
        $mtime = microtime();
        $mtime = explode(" ", $mtime);
        $mtime = $mtime[1] + $mtime[0];
        $endtime = $mtime;
        $totaltime = ($endtime - $starttime);
        return $totaltime;
    }

    public static function getCustomPostTypes()
    {
        $args = array(
            'public'   => true,
            '_builtin' => false
        );

        return get_post_types($args, 'names', 'and');
    }

    public static function isCustomPostType($slug)
    {
        $postTypes = self::getCustomPostTypes();

        return in_array($slug, $postTypes);
    }

    /**
     * This funciton returns the current UTC/GMT time in a format appropriate for
     * storing in the database.
     * 
     * @return String|int returns a date-time string representing the current UTC/GMT time if format is 'mysql', integer timestamp if format is 'timestamp' or 'U'
     */
    public static function getCurrentTime($format = "mysql", $isUTC = true)
    {
        return current_time($format, $isUTC ? 1 : 0);
    }


    /**
     * This function uses the current WordPress timezone configuration to translate a UTC/GMT
     * date to the local timezone.
     * 
     * @param string|integer $utcDateStr the UTC/GMT date to translate to the local timezone. It can be a unix timestamp or a date-time string
     * @param string $format (optional) this is the format to use when formatting the date
     * 
     * @return string a formatted date in the local timezone configured in WordPress
     */
    public static function dateToLocal($utcDateStr, $format = "M j, Y g:i a")
    {
        // check if date string passed is already a unix timestamp
        if (is_numeric($utcDateStr) && ($utcDateStr <= PHP_INT_MAX) && ($utcDateStr >= ~PHP_INT_MAX)) {
            $localDate = new DateTime("@{$utcDateStr}"); //use @ to mark it as UTC
        } else {
            $localDate = new DateTime("{$utcDateStr} UTC"); //explicitly mark as UTC, will have no effect if the string already has a timezone component (which it shouldnt)

        }

        $localDate->setTimezone(wp_timezone());
        return $localDate->format($format);
    }


    /**
     * This function uses the current WordPress timezone configuration to translate a localized date to UTC/GMT
     * Note: It is assumed that the supplied date does not have a timezone component. Dates that already include timezone may convert unpredictably
     * 
     * @param string|integer $localDateStr this is the local date to translate to UTC. It can be a unix timestamp or a date-time string
     * @param string $format (optional) this is the format to use when formatting the date
     * @param string $fixedTime (optional) prodvide this parameter in order to fix the time of the localDateString passed prior to converting to UTC
     * 
     * @return string a formatted date in the local timezone configured in WordPress
     */
    public static function dateToUTC($localDateStr, $format = "M j, Y g:i a", $fixedTime = "")
    {
        if (!empty($fixedTime)) {
            $localDateStr = date("Y-m-d {$fixedTime}", strtotime($localDateStr));
        }

        $timezoneStr = wp_timezone_string();

        // check if date string passed is already a unix timestamp
        if (((string) (int) $localDateStr === $localDateStr) && ($localDateStr <= PHP_INT_MAX) && ($localDateStr >= ~PHP_INT_MAX)) {
            $utcDate = new DateTime("@{$localDateStr}");
        } else {
            if (!empty($timezoneStr)) {
                $localDateStr .= " {$timezoneStr}";
            }
            $utcDate = new DateTime($localDateStr);
        }

        $utcDate->setTimezone(new DateTimeZone('UTC'));

        return $utcDate->format($format);
    }

    public static function isSSL()
    {
        $plugins = get_option('active_plugins');
        $required_plugin = "wordpress-https/wordpress-https.php";
        $wpHttpsActive = false;

        if (in_array($required_plugin, $plugins)) {
            $wpHttpsActive = true;
        }

        if ($wpHttpsActive) {
            // use is_ssl function from WordPress HTTPS plugin if it's activated
            $wphttps = new WordPressHTTPS();
            return $wphttps->is_ssl();
        } else {
            // use WordPress is_ssl function
            return is_ssl();
        }
    }

    public static function translateText($str, $domain)
    {
        if (function_exists("__")) {
            return __($str, $domain);
        } else {
            return false;
        }
    }

    public static function isLoggedIn()
    {
        if (function_exists("is_user_logged_in")) {
            return is_user_logged_in();
        } else {
            return false;
        }
    }

    public static function abbrevString($str, $maxLength = 40)
    {
        $origStr = $str;

        if (strlen($str) >= $maxLength) {
            $str = substr($str, 0, $maxLength) . "...";
        }

        return "<span title='" . $origStr . "'>" . $str . "</span>";
    }

    public static function isGetParamAllowed($getParam)
    {
        global $reservedGetParams;

        $key = strtolower($getParam);

        if (isset($reservedGetParams[$key])) {
            return false;
        }

        return true;
    }

    /**
     * Pass in a comma delimited list (or whatever is specified by $delim) and get back whether all emails in list are valid.
     * This ONLY handles the case for multiple delimited emails for backward compatibility.  I do not know of a valid entry 
     * where we actually should accept multiple emails in a text box.
     * 
     * @param string $delim (optional) specify how to parse the string for multiple emails
     * @param string $emailStr the string with concatenated email addresses.
     * @return true if all (non-empty) emails (at least one) parsed are valid, otherwise false.
     */
    public static function hasValidEmail($emailStr, $delim = ",")
    {
        $foundValidEmail = false;
        if ($emailStr != null) {
            $emailArr = explode($delim, $emailStr);
            foreach ($emailArr as $email) {
                $email = trim($email);
                if (strlen($email) > 0) {
                    if (!is_email($email)) {
                        // hard stop, invalid email in list
                        return false;
                    }

                    // Found at least one good email in list.
                    $foundValidEmail = true;
                }
            }
        }
        return $foundValidEmail;
    }


    /*
 	 * Legacy function
 	 */
    public static function validateEmail($email)
    {
        $isValid = true;
        $atIndex = strrpos($email, "@");

        if (is_bool($atIndex) && !$atIndex) {
            $isValid = false;
        } else {
            $domain = substr($email, $atIndex + 1);
            $local = substr($email, 0, $atIndex);
            $localLen = strlen($local);
            $domainLen = strlen($domain);

            if ($localLen < 1 || $localLen > 64) {
                // local part length exceeded
                $isValid = false;
            } else if ($domainLen < 1 || $domainLen > 255) {
                // domain part length exceeded
                $isValid = false;
            } else if ($local[0] == '.' || $local[$localLen - 1] == '.') {
                // local part starts or ends with '.'
                $isValid = false;
            } else if (preg_match('/\\.\\./', $local)) {
                // local part has two consecutive dots
                $isValid = false;
            } else if (!preg_match('/^[A-Za-z0-9\\-\\.]+$/', $domain)) {
                // character not valid in domain part
                $isValid = false;
            } else if (preg_match('/\\.\\./', $domain)) {
                // domain part has two consecutive dots
                $isValid = false;
            } else if (!preg_match('/^(\\\\.|[A-Za-z0-9!#%&`_=\\/$\'*+?^{}|~.-])+$/', str_replace("\\\\", "", $local))) {
                // character not valid in local part unless
                // local part is quoted
                if (!preg_match(
                    '/^"(\\\\"|[^"])+"$/',
                    str_replace("\\\\", "", $local)
                )) {
                    $isValid = false;
                }
            }

            if ($isValid && function_exists("checkdnsrr") && !(checkdnsrr($domain, "MX") || checkdnsrr($domain, "A"))) {
                // domain not found in DNS
                $isValid = false;
            }
        }

        return $isValid;
    }

    private static function countDays($a, $b)
    {
        // First we need to break these dates into their constituent parts:
        $gd_a = getdate($a);
        $gd_b = getdate($b);

        // Now recreate these timestamps, based upon noon on each day
        // The specific time doesn't matter but it must be the same each day
        $a_new = mktime(12, 0, 0, $gd_a['mon'], $gd_a['mday'], $gd_a['year']);
        $b_new = mktime(12, 0, 0, $gd_b['mon'], $gd_b['mday'], $gd_b['year']);

        // Subtract these two numbers and divide by the number of seconds in a
        //  day. Round the result since crossing over a daylight savings time
        //  barrier will cause this time to be off by an hour or two.
        return round(abs($a_new - $b_new) / 86400);
    }

    public static function getNextPaymentDate($startDate, $productId)
    {
        $product = new MM_Product($productId);
        $start  = strtotime($startDate);

        if ($product->isRecurring(false)) {
            $period = $product->getRebillPeriod();
            $freq = $product->getRebillFrequency();
            $newdate = null;
            switch ($freq) {
                case "months":
                    $months = floor((time() - $start) / 2628000);
                    $newStart = strtotime($months . ' month', strtotime($start));
                    $newdate = strtotime($period . ' month', strtotime($newStart));
                    break;
                case "days":
                    $days = self::countDays(Date("Y-m-d h:i:s"), $startDate);
                    $diff = ($days > $period) ? intval($days / $period) : 1;
                    $newStart = strtotime($diff . ' day', strtotime($start));
                    $newdate = strtotime($period . ' day', strtotime($newStart));
                    break;
                case "weeks":
                    $days = self::countDays(Date("Y-m-d h:i:s"), $startDate);
                    $weeks = ($days > 7) ? intval($days / 7) : 1;
                    $diff = ($weeks > $period) ? intval($weeks / $period) : 1;

                    $newStart = strtotime($diff . ' week', strtotime($start));
                    $newdate = strtotime($period . ' week', strtotime($newStart));
                    break;
                case "years":
                    $days = self::countDays(Date("Y-m-d h:i:s"), $startDate);
                    $years = ($days > 365) ? intval($days / 365) : 1;
                    $diff = ($years > $period) ? intval($years / $period) : 1;
                    $newStart = strtotime($diff . ' year', strtotime($start));
                    $newdate = strtotime($period . ' year', strtotime($newStart));
                    break;
            }

            if (!is_null($newdate)) {
                return $newdate;
            }
        }
        return false;
    }

    public static function convertArrayToObject($arr)
    {
        if (is_array($arr)) {
            $info = new stdClass();
            foreach ($arr as $k => $v) {
                $info->$k = $v;
            }
            return $info;
        }
        return new stdClass();
    }

    public static function getFileContents($files)
    {
        $contents = "";
        if (is_array($files)) {
            foreach ($files as $file) {
                $contents .= self::loadFile($file);
            }
        }
        return $contents;
    }

    public static function loadFile($file)
    {
        if (file_exists($file)) {
            return file_get_contents($file);
        }
        return "";
    }

    public static function getReferrer()
    {
        if (isset($_SERVER["HTTP_REFERER"])) {
            return $_SERVER["HTTP_REFERER"];
        }
        return "";
    }

    public static function explode($needle, $haystack)
    {
        $arr = explode($needle, $haystack);
        if (is_array($arr)) {
            foreach ($arr as &$value) {
                $value = urldecode($value);
            }
        }
        return $arr;
    }

    public static function isURL($url = null)
    {
        if (is_null($url)) {
            return false;
        }

        $protocol = '(http://|https://)';
        $allowed = '([a-z0-9]([-a-z0-9]*[a-z0-9]+)?)';

        $regex = "^" . $protocol . // must include the protocol
            '(' . $allowed . '{1,63}\.)+' . // 1 or several sub domains with a max of 63 chars
            '[a-z]' . '{2,6}'; // followed by a TLD
        if (preg_match("/" . $regex . "/", $url) === true) {
            return true;
        } else {
            return false;
        }
    }

    public static function appendUrlParam($url, $paramKey, $paramVal, $urlencode = true)
    {
        if ($urlencode) {
            $paramVal = urlencode($paramVal);
        }

        if (preg_match("/(\?)/", $url)) {
            return $url . "&" . $paramKey . "=" . $paramVal;
        }
        return $url . "?" . $paramKey . "=" . $paramVal;
    }

    public static function chooseRandomAssocOption($options)
    {
        if (is_array($options)) {
            $key = array_rand($options, 1);
            return $options[$key];
        }
        return "";
    }

    public static function chooseRandomOption($options)
    {
        if (is_array($options)) {
            $index = rand(0, count($options) - 1);
            return $options[$index];
        }
        return "";
    }


    /**
     * Generates a random string according to the parameters passed in
     * 
     * Internally calls wp_rand(), which has the following behavior:
     * for Wordpress versions > 4.4, PHP version 7.0+  - uses the strong PHP 7 random_int() function 
     * for Wordpress versions > 4.4, PHP version 5.x - uses the random_int() polyfill from the bundled random_compat library 
     * for Wordpress versions < 4.4 - uses and internal psrng algorithm seeded by microtime(), mt_rand(), MySQL's random number generator, and optionally a user-configurable random seed 
     * 
     **/
    public static function createRandomString($length = 8, $onlyAlpha = false, $onlyDigits = false)
    {
        if ($length <= 0) {
            return "";
        }

        if ($onlyDigits) {
            $charset = "0123456789";
        } else if ($onlyAlpha) {
            $charset = "abcdefghijkmnopqrstuvwxyz";
        } else {
            $charset = "abcdefghijkmnopqrstuvwxyz0123456789";
        }

        $charsetLength = strlen($charset);
        $randomString = "";
        for ($i = 0; $i < $length; $i++) {
            $randomString .= substr($charset, wp_rand(0, $charsetLength - 1), 1);
        }

        return $randomString;
    }


    public static function calculateDaysDiff($startDate, $endDate)
    {
        $day = 86400;
        $start_time = strtotime($startDate);
        $end_time = strtotime($endDate);

        return (round($end_time - $start_time) / $day) + 1;
    }

    public static function getFilesFromDir($directory, $recursive = false, $includeDirs = false, $pattern = '/.*/')
    {
        $items = array();

        if ($handle = opendir($directory)) {
            while (($file = readdir($handle)) !== false) {
                if ($file != '.' && $file != '..') {
                    $path = "$directory/$file";
                    $path = preg_replace('#//#si', '/', $path);
                    if (is_dir($path)) {
                        if ($includeDirs) {
                            $items[] = $path;
                        }
                        if ($recursive) {
                            $items = array_merge($items, self::getFilesFromDir($path, true, $includeDirs, $pattern));
                        }
                    } else {
                        if (preg_match($pattern, $file)) {
                            $items[] = $path;
                        }
                    }
                }
            }

            closedir($handle);
        }

        sort($items);

        return $items;
    }


    /**
     * When we reference IP addresses within the plugin we link it to infosniper by opening a new page.   
     * In this manner, this provides a central point to split up ip addresses as this will support any number of ips that are 
     * separated by space or comma and generate html links for it.  
     * 
     * @param $ipAddress string List of IPs separated by comma or space
     * @param $delim (optional) how you want the generated html to be split apart.
     * @param $link (optional) if you want to redirect to a different link/IP service, provide it here with the %s option in place as needed.
     * 
     * @return string of valid HTML with links provided an IP address is found.  Otherwise empty string.
     */
    public static function generateIPLinks($ipAddress, $delim = "<br />", $link = "http://www.infosniper.net/index.php?ip_address=%s")
    {
        $ipAddressHtml = "";
        $ips = preg_split("/(\,)|[\s]+/", $ipAddress);
        if (count($ips) > 0) {
            foreach ($ips as $ip) {
                $url = sprintf($link, $ip);
                $ipAddressHtml .= "<a href='{$url}' target='_blank'>" . $ip . "</a>{$delim}";
            }
            if (strlen($delim) > 0) {
                $ipAddressHtml = substr($ipAddressHtml, 0, strlen($ipAddressHtml) - strlen($delim));
            }
        }
        return $ipAddressHtml;
    }

    public static function getClientIPAddress()
    {
        if (!empty($_SERVER['HTTP_CF_CONNECTING_IP'])) {
            $ip = $_SERVER['HTTP_CF_CONNECTING_IP'];
        } else if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : "127.0.0.1";
        }

        //don't use ipv6 for local installs
        if (isLocalInstall() && (strpos($ip, "::") !== false)) {
            $ip = "127.0.0.1";
        }
        return $ip;
    }

    public static function getIcon($iconName, $color = "", $fontSize = "", $offset = "", $title = "", $addlStyles = "")
    {
        $iconStr = "<i class=\"fa fa-{$iconName} mm-icon {$color}\" style=\"";

        if (!empty($fontSize)) {
            $iconStr .= " font-size:{$fontSize};";
        }

        if (!empty($offset)) {
            $iconStr .= " position:relative; top:{$offset};";
        }

        if (!empty($addlStyles)) {
            $iconStr .= " {$addlStyles}";
        }

        $iconStr .= "\"></i>";

        if (!empty($title)) {
            $iconStr = "<a title=\"{$title}\">{$iconStr}</a>";
        }

        return $iconStr;
    }

    public static function getInfoIcon($description = "", $addlStyles = 'margin-left:4px;', $onClickHandler = "")
    {
        if (empty($onClickHandler)) {
            return MM_Utils::getIcon('info-circle', 'blue', '1.3em', '2px', $description, $addlStyles);
        } else {
            return "<a onclick='{$onClickHandler}' style='cursor:pointer;' title='{$description}'>" . MM_Utils::getIcon('info-circle', 'blue', '1.3em', '2px', '', $addlStyles) . "</a>";
        }
    }

    public static function getEditIcon($description = "", $addlStyles = '', $actionString = "", $isDisabled = false)
    {
        return self::getGridActionIcon('yellow', 'pencil', $description, $addlStyles, $actionString, $isDisabled);
    }

    public static function getDeleteIcon($description = "", $addlStyles = '', $actionString = "", $isDisabled = false)
    {
        return self::getGridActionIcon('red', 'trash-o', $description, $addlStyles, $actionString, $isDisabled);
    }


    public static function getDuplicateIcon($description = "", $addlStyles = '', $actionString = "")
    {
        return self::getGridActionIcon('blue', 'clone', $description, $addlStyles, $actionString);
    }


    protected static function getGridActionIcon($activeColor, $icon, $description = "", $addlStyles = '', $actionString = "", $isDisabled = false)
    {
        $color = ($isDisabled) ? "grey" : $activeColor;

        if (empty($actionString) || $isDisabled) {
            if (empty($addlStyles) || (strpos($addlStyles, "cursor") === false)) {
                $addlStyles = "cursor:pointer;{$addlStyles}"; //change pointer to indicate that the action icons are interactive
            }
            return MM_Utils::getIcon($icon, $color, '1.3em', '2px', $description, $addlStyles);
        } else {
            return "<a {$actionString} style='cursor:pointer; {$addlStyles}' title='{$description}'>" . MM_Utils::getIcon($icon, $color, '1.3em', '2px', '', '') . "</a>";
        }
    }

    public static function getHideIcon($description = "", $addlStyles = '', $actionString = "", $isHidden = false)
    {
        $color = ($isHidden) ? "light-blue" : "salmon";
        $icon = ($isHidden) ? "eye" : "eye-slash";

        return "<a {$actionString} style='cursor:pointer; {$addlStyles}' title='{$description}'>" . MM_Utils::getIcon($icon, $color, '1.3em', '2px', '', '') . "</a>";
    }

    public static function getAccessIcon($accessType, $description = '', $addlStyles = '')
    {
        switch ($accessType) {
            case MM_OrderItemAccess::$ACCESS_TYPE_BUNDLE:
                $description = (empty($description)) ? "Bundle" : $description;
                return MM_Utils::getIcon('cube', 'yellow', '1.3em', '2px', $description, $addlStyles);
                break;

            case MM_OrderItemAccess::$ACCESS_TYPE_MEMBERSHIP:
                $description = (empty($description)) ? "Membership Level" : $description;
                return MM_Utils::getIcon('user', 'blue', '1.3em', '2px', $description, $addlStyles);
                break;
        }
    }

    public static function getDefaultFlag($description = "", $actionString = "", $isDefault = false, $addlStyles = "")
    {
        if ($isDefault) {
            return MM_Utils::getIcon('flag', 'orange', '1.3em', '2px', $description, $addlStyles);
        } else {
            return "<a {$actionString} style='cursor:pointer; {$addlStyles}' title='{$description}'>" . MM_Utils::getIcon('flag-o', 'grey', '1.3em', '2px', '', '') . "</a>";
        }
    }

    public static function getCalendarIcon()
    {
        return MM_Utils::getIcon('calendar', 'blue', '1.2em', '1px');
    }

    public static function getCheckIcon($description = "")
    {
        return MM_Utils::getIcon('check', 'green', '1.3em', '1px', $description);
    }

    public static function getCrossIcon($description = "")
    {
        return MM_Utils::getIcon('times', 'red', '1.3em', '1px', $description);
    }

    public static function getAffiliateIcon($description = "", $addlStyles = "")
    {
        return MM_Utils::getIcon('bullhorn', 'orange', '1.4em', '2px', $description, $addlStyles);
    }

    public static function getDiscountIcon($description = "", $addlStyles = "")
    {
        return MM_Utils::getIcon('ticket', 'purple', '1.4em', '2px', $description, $addlStyles);
    }

    public static function getImageUrl($imageName)
    {
        $imageUrl = MM_IMAGES_URL;

        if (MM_Utils::isSSL()) {
            $imageUrl = preg_replace("/(http\:)/", "https:", $imageUrl);
        }

        $imageType = "";
        if (strpos($imageName, "/") !== false) {
            $split = explode("/", $imageName);
            if (count($split) > 1) {
                $imageType = strtolower($split[0]);
            }
        }

        switch ($imageType) {
            case 'dashboard':
                $central = preg_replace("/^(http:|https:)/", "", MM_PRETTY_CENTRAL_SERVER_URL); //remove the scheme so the browser can adjust based on secure/non-secure (rfc 1808)
                $imageUrl = $central . "/images/{$imageName}.png";
                break;
            default:
                if (file_exists(MM_IMAGES_PATH . "/" . $imageName . ".png")) {
                    $imageUrl .= $imageName . ".png";
                } else if (file_exists(MM_IMAGES_PATH . "/" . $imageName . ".jpg")) {
                    $imageUrl .= $imageName . ".jpg";
                } else if (file_exists(MM_IMAGES_PATH . "/" . $imageName . ".gif")) {
                    $imageUrl .= $imageName . ".gif";
                } else if (file_exists(MM_IMAGES_PATH . "/" . $imageName . ".svg")) {
                    $imageUrl .= $imageName . ".svg";
                }
                break;
        }

        return $imageUrl;
    }

    public static function createOptionsArray($obj, $idLabel, $valueLabel)
    {
        $retArr = array();

        if (is_array($obj)) {
            foreach ($obj as $row) {
                if (isset($row->$idLabel) && isset($row->$valueLabel)) {
                    $retArr[$row->$idLabel] = $row->$valueLabel;
                }
            }
        }
        return $retArr;
    }

    public static function constructPageUrl()
    {
        static $pageURL = null; //make multiple invocations of this function less expensive

        if ($pageURL == null) {
            $pageURL = "http://";

            if ((MM_Utils::isSSL() == true)
                || (isset($_SERVER["HTTPS"]) && ($_SERVER["HTTPS"] == "on"))
                || (isset($_SERVER["SERVER_PORT"]) && ($_SERVER["SERVER_PORT"] == "443"))
                || (stripos(site_url(), 'https://') === 0)
            ) {
                $pageURL = "https://";
            }

            if (isset($_SERVER["SERVER_PORT"]) && ($_SERVER["SERVER_PORT"] != "80") && ($_SERVER["SERVER_PORT"] != "443")) {
                $pageURL .= $_SERVER["SERVER_NAME"] . ":" . $_SERVER["SERVER_PORT"] . $_SERVER["REQUEST_URI"];
            } else if (isset($_SERVER["SERVER_NAME"]) && isset($_SERVER["REQUEST_URI"])) {
                $pageURL .= $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"];
            } else {
                $siteUrl = MM_OptionUtils::getOption("siteurl");
                $pageURL = $siteUrl;
            }
            $pageURL = htmlspecialchars($pageURL, ENT_COMPAT, "UTF-8");
        }
        return $pageURL;
    }


    public static function getStatusImage($status)
    {
        if ($status == '1') {
            return MM_Utils::getIcon('circle', 'green', '1em', '2px', 'Active');
        } else if ($status == '0') {
            return MM_Utils::getIcon('circle-o', 'red', '1em', '2px', 'Inactive');
        } else {
            return MM_NO_DATA;
        }
    }

    /**
     * This function returns a user object based on if a member is logged in or an admin. If an 
     * admin is logged in, a user object will be returned based on the current preview bar settings
     */
    public static function getCurrentUser()
    {
        global $user, $current_user;

        $user_obj = null;

        if (MM_Employee::isEmployee()) {
            $previewObj = MM_Preview::getData();

            if ($previewObj !== false) {
                return $previewObj->getUser();
            }
        }

        if (isset($user->ID) && intval($user->ID) > 0) {
            $user_obj = MM_User::create($user->ID);
        } else if (isset($user->data->ID) && intval($user->data->ID) > 0) {
            $user_obj = MM_User::create($user->data->ID);
        } else if (isset($current_user->ID) && intval($current_user->ID) > 0) {
            $user_obj = MM_User::create($current_user->ID);
        }

        return $user_obj;
    }


    public static function getPluginWarnings()
    {
        $pluginWarningData = [
            [
                "name" => "W3 Total Cache",
                "location" => "w3-total-cache/w3-total-cache.php",
                "noticeId" => "w3-total-cache-warning",
            ],
            [
                "name" => "WP Super Cache",
                "location" => "wp-super-cache/wp-cache.php",
                "noticeId" => "wp-super-cache-warning",
            ],
        ];

        $plugins = get_option('active_plugins');

        foreach ($pluginWarningData as $pluginData) {
            if (in_array($pluginData["location"], $plugins)) {
                $titleText = __("MemberMouse Plugin Compatibility Warning", "membermouse");
                $messageText = sprintf(
                    __('The third-party plugin %1$s is known to cause problems with MemberMouse. %2$sLearn More%3$s', 'membermouse'),
                    '<em>' . $pluginData["name"] . '</em>',
                    '<a href="https://membermouse.com/docs/plugins-that-cause-problems" target="_blank">',
                    '</a>'
                );

                $title = "<i class=\"fa fa-exclamation-triangle\"></i> <strong>{$titleText}</strong>";
                $message = "<p>{$messageText}</p>";

                MM_AdminNotice::add(
                    $pluginData["noticeId"],
                    $title,
                    $message,
                    MM_AdminNotice::TYPE_WARNING,
                    ["behavior" => "dismiss"]
                );
            }
        }

        // check if client is using WP Engine
        if (class_exists("WPE_API", false) || defined("WPE_APIKEY") || defined("WPE_ISP")) {
            $titleText = __("MemberMouse Environmental Compatibility Warning", "membermouse");
            $messageText = sprintf(
                __('You are using WP Engine. Follow the instructions in our article %1$sConfigure WP Engine%2$s to ensure your server is configured to allow MemberMouse to run properly.', 'membermouse'),
                '<a href="https://membermouse.com/docs/configuring-wp-engine-hosting" target="_blank">',
                '</a>'
            );

            $title = "<i class=\"fa fa-exclamation-triangle\"></i> <strong>{$titleText}</strong>";
            $message = "<p>{$messageText}</p>";

            MM_AdminNotice::add(
                "wp-engine-warning",
                $title,
                $message,
                MM_AdminNotice::TYPE_WARNING,
                ["behavior" => "dismiss"]
            );
        }
    }


    public static function inAdmin()
    {
        return is_admin();
    }


    public static function isSiteAdmin()
    {
        return current_user_can('manage_options');
    }

    public static function inMembermouseAdmin()
    {
        if (is_admin()) {
            $module = MM_ModuleUtils::getModule();      // Returns empty string when not a MM page or module
            $request_base = isset($_SERVER["PHP_SELF"]) ? basename($_SERVER["PHP_SELF"]) : "";
            $additional_pages = array("edit.php", "post.php", "post-new.php");
            return ($module != "" || in_array($request_base, $additional_pages));
        } else {
            return false;
        }
    }

    /**
     * Pluralizes a word if quantity is not one.
     *
     * @param int $quantity Number of items
     * @param string $singular Singular form of word
     * @param string $plural Plural form of word; function will attempt to deduce plural form from singular if not provided
     * @return string Pluralized word if quantity is not one, otherwise singular
     */
    public static function pluralize($quantity, $singular, $plural = null)
    {
        if ($quantity == 1 || !strlen($singular)) return $singular;
        if ($plural !== null) return $plural;

        $last_letter = strtolower($singular[strlen($singular) - 1]);
        switch ($last_letter) {
            case 'y':
                return substr($singular, 0, -1) . 'ies';
            case 's':
                return $singular . 'es';
            default:
                return $singular . 's';
        }
    }

    /**
     * Translates billing intervals for display
     *
     * @param string $interval Interval to translate, expected values are "days", "weeks", "months", or "years"
     * @param integer $quantity Optional number of units, determines pluralization, default is to pluralize
     * @return string Translation of the billing interval. Invalid input is returned unaltered.
     */
    public static function translateInterval($interval, $quantity = 0)
    {
        $pluralize = ($quantity != 1);
        switch ($interval) {
            case "days":
                $translated = ($pluralize) ? __("days", "membermouse") : __("day", "membermouse");
                break;
            case "weeks":
                $translated = ($pluralize) ? __("weeks", "membermouse") : __("week", "membermouse");
                break;
            case "months":
                $translated = ($pluralize) ? __("months", "membermouse") : __("month", "membermouse");
                break;
            case "years":
                $translated = ($pluralize) ? __("years", "membermouse") : __("year", "membermouse");
                break;
            default:
                return $interval;
        }
        return $translated;
    }


    /**
     * Get the current locale, filtered using plugin_locale
     *
     * @param string $domain The text domain, defaults to MM_LANGUAGE_DOMAIN
     * @return string A locale, e.g. en_US
     */
    public static function getLocale($domain = MM_LANGUAGE_DOMAIN): string
    {
        return apply_filters('plugin_locale', determine_locale(), $domain);
    }


    /**
     * Gets the path of the translations directory. Applies the filter MM_Filters::TRANSLATIONS_PATH,
     * allowing users to override the default, which is the full path to 'wp-content/languages/membermouse'
     * 
     * @return string The full path of the translations directory
     */
    public static function getTranslationsPath(): string
    {
        return apply_filters(MM_Filters::$TRANSLATIONS_PATH, WP_LANG_DIR . '/' . MM_LANGUAGE_DOMAIN);
    }


    public static function loadMembermouseTextDomain(): bool
    {
        return self::loadTextDomain(MM_LANGUAGE_DOMAIN, MM_PLUGIN_ABSPATH, 'languages');
    }


    /**
     * Loads the translation file for the given text domain. Intended for use by MemberMouse and extensions. By default,
     * attempts to load from the following locations in order:
     * 
     *     1. A custom translations directory specified using the filter MM_Filters::TRANSLATIONS_PATH
     *     2. If the custom folder is not set, the default languages directory 'wp-content/languages/membermouse'
     *     3. The directory 'wp-content/plugins/membermouse/languages'
     *     
     * An alternative text domain and plugin path can be specified for extensions.
     * 
     * @param string $domain The text domain. If null is passed, defaults to MM_LANGUAGE_DOMAIN
     * @param string $pluginPath The plugin path, defaults to MM_PLUGIN_ABSPATH
     * @param string $languagesDirectory The languages directory, defaults to 'languages'
     * @return bool True if the text domain was loaded, false otherwise
     */
    public static function loadTextDomain($domain, $pluginPath = null, $languagesDirectory = null): bool
    {
        $domain ??= MM_LANGUAGE_DOMAIN;
        $pluginPath ??= MM_PLUGIN_ABSPATH;
        $languagesDirectory ??= "languages";

        unload_textdomain($domain);
        $textDomain = load_textdomain(
            $domain,
            sprintf('%1$s/%2$s-%3$s.mo', self::getTranslationsPath(), $domain, self::getLocale($domain)),
        );
        $pluginTextDomain = load_plugin_textdomain(
            $domain,
            false,
            sprintf('%1$s/%2$s', basename($pluginPath), $languagesDirectory)
        );

        return ($textDomain || $pluginTextDomain);
    }


    /**
     * Attempts to set script translations for a given handle. Several locations are checked and the first found location is used.
     * Checks the following locations:
     *     1. The custom translations path specified using the filter MM_Filters::TRANSLATIONS_PATH
     *     2. If the filter is not set, the default languages directory 'wp-content/languages/membermouse'
     *     3. The directory 'wp-content/languages/plugins' which is default for wp_set_script_translations
     *     
     * @param string $handle The script handle
     * @param string $domain The text domain, defaults to MM_LANGUAGE_DOMAIN
     */
    public static function setScriptTranslations($handle, $domain = MM_LANGUAGE_DOMAIN): void
    {
        $location = self::getTranslationsPath();

        // TODO: Add support for other locations, like the plugin languages directory. Note that wp_set_script_translations doesn't 
        // return false on load from an empty location, and loading from an empty location wipes out previous successful loads.
        // Simply iterating through an array of locations does not work correctly.

        wp_set_script_translations($handle, $domain, $location);
    }
}
