<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * The Push Notification Engine translates registered events into Actions. 
 */

class MM_PushNotificationEngine extends MM_ActionDispatchEngineBase
{

    public function productPurchase($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$PRODUCT_PURCHASE);
        $this->params = $params;
        $this->logActivity(MM_Event::$PRODUCT_PURCHASE, $params);
        $this->performActions();
    }

    public function memberAdded($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$MEMBER_ADD);
        $this->params = $params;
        $this->logActivity(MM_Event::$MEMBER_ADD, $params);
        $this->performActions();
    }

    public function memberMerged($params)
    {
        $this->params = $params;
        $this->logActivity(MM_Event::$MEMBER_MERGED, $params);
    }

    public function memberStatusChanged($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$MEMBER_STATUS_CHANGE);
        $this->params = $params;
        $this->logActivity(MM_Event::$MEMBER_STATUS_CHANGE, $params);
        $this->performActions();
    }

    public function memberMembershipChanged($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$MEMBER_MEMBERSHIP_CHANGE);
        $this->params = $params;
        $this->logActivity(MM_Event::$MEMBER_MEMBERSHIP_CHANGE, $params);
        $this->performActions();
    }

    public function memberAccountUpdated($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$MEMBER_ACCOUNT_UPDATE);
        $this->params = $params;
        $this->performActions();
    }

    public function memberDeleted($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$MEMBER_DELETE);
        $this->params = $params;
        $this->performActions();
    }

    public function bundleAdded($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$BUNDLE_ADD);
        $this->params = $params;
        $this->logActivity(MM_Event::$BUNDLE_ADD, $params);
        $this->performActions();
    }

    public function bundleStatusChanged($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$BUNDLE_STATUS_CHANGE);
        $this->params = $params;
        $this->logActivity(MM_Event::$BUNDLE_STATUS_CHANGE, $params);
        $this->performActions();
    }

    public function paymentReceived($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$PAYMENT_RECEIVED);
        $this->params = $params;
        $this->logActivity(MM_Event::$PAYMENT_RECEIVED, $params);
        $this->performActions();
    }

    public function rebillPaymentReceived($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$PAYMENT_REBILL);
        $this->params = $params;
        $this->logActivity(MM_Event::$PAYMENT_REBILL, $params);
        $this->performActions();
    }

    public function rebillPaymentDeclined($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$PAYMENT_REBILL_DECLINED);
        $this->params = $params;
        $this->logActivity(MM_Event::$PAYMENT_REBILL_DECLINED, $params);
        $this->performActions();
    }

    public function billingSubscriptionUpdated($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$BILLING_SUBSCRIPTION_UPDATED);
        $this->params = $params;
        $this->logActivity(MM_Event::$BILLING_SUBSCRIPTION_UPDATED, $params);
        $this->performActions();
    }

    public function billingSubscriptionRebillDateChanged($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$BILLING_SUBSCRIPTION_REBILL_DATE_CHANGED);
        $this->params = $params;
        $this->logActivity(MM_Event::$BILLING_SUBSCRIPTION_REBILL_DATE_CHANGED, $params);
        $this->performActions();
    }

    public function billingSubscriptionCanceled($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$BILLING_SUBSCRIPTION_CANCELED);
        $this->params = $params;
        $this->logActivity(MM_Event::$BILLING_SUBSCRIPTION_CANCELED, $params);
        $this->performActions();
    }

    public function refundIssued($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$REFUND_ISSUED);
        $this->params = $params;
        $this->logActivity(MM_Event::$REFUND_ISSUED, $params);
        $this->performActions();
    }

    public function affiliateInfoChanged($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$AFFILIATE_INFO_CHANGED);
        $this->params = $params;
        $this->logActivity(MM_Event::$AFFILIATE_INFO_CHANGED, $params);
        $this->performActions();
    }

    public function initialCommission($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$COMMISSION_INITIAL);
        $this->params = $params;
        $this->performActions();
    }

    public function rebillCommission($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$COMMISSION_REBILL);
        $this->params = $params;
        $this->performActions();
    }

    public function cancelCommission($params)
    {
        $this->actions = MM_Action::findActionsByEventType(MM_Event::$CANCEL_COMMISSION);
        $this->params = $params;
        $this->performActions();
    }

    protected function logActivity($eventType, $params)
    {
        global $wpdb;

        if (isset($params["member_id"])) {
            $logParams = array();

            $user = new MM_User($params["member_id"]);

            if ($user->isValid()) {
                switch ($eventType) {
                    case MM_Event::$MEMBER_ADD:
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_EVENT] = $eventType;
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_ID] = $params["membership_level"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_IS_COMPLIMENTARY] = $params["is_complimentary"];
                        MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_ACCESS_RIGHTS, $logParams);
                        break;

                    case MM_Event::$MEMBER_MERGED:
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_EVENT] = $eventType;
                        $logParams["member_id"] = $params["secondary_user"]["member_id"];
                        $logParams["email"] = $params["secondary_user"]["email"];
                        MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_ACCESS_RIGHTS, $logParams);
                        break;

                    case MM_Event::$MEMBER_STATUS_CHANGE:
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_EVENT] = $eventType;
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_ID] = $params["membership_level"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_IS_COMPLIMENTARY] = $params["is_complimentary"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_STATUS] = $params["status"];

                        if ($params["status"] == MM_Status::$ERROR) {
                            $logParams[MM_ActivityLog::$PARAM_ACCESS_STATUS_MESSAGE] = $user->getStatusMessage();
                        }

                        MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_ACCESS_RIGHTS, $logParams);
                        break;

                    case MM_Event::$MEMBER_MEMBERSHIP_CHANGE:
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_EVENT] = $eventType;
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_ID] = $params["membership_level"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_IS_COMPLIMENTARY] = $params["is_complimentary"];
                        MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_ACCESS_RIGHTS, $logParams);
                        break;

                    case MM_Event::$BUNDLE_ADD:
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_EVENT] = $eventType;
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_ID] = $params["bundle_id"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_IS_COMPLIMENTARY] = $params["bundle_is_complimentary"];
                        MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_ACCESS_RIGHTS, $logParams);
                        break;

                    case MM_Event::$BUNDLE_STATUS_CHANGE:
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_EVENT] = $eventType;
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_ID] = $params["bundle_id"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_IS_COMPLIMENTARY] = $params["bundle_is_complimentary"];
                        $logParams[MM_ActivityLog::$PARAM_ACCESS_STATUS] = $params["bundle_status"];
                        MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_ACCESS_RIGHTS, $logParams);
                        break;

                    case MM_Event::$PAYMENT_RECEIVED:
                    case MM_Event::$PAYMENT_REBILL:
                    case MM_Event::$PAYMENT_REBILL_DECLINED:
                    case MM_Event::$REFUND_ISSUED:
                    case MM_Event::$BILLING_SUBSCRIPTION_UPDATED:
                    case MM_Event::$BILLING_SUBSCRIPTION_REBILL_DATE_CHANGED:
                    case MM_Event::$BILLING_SUBSCRIPTION_CANCELED:
                    case MM_Event::$AFFILIATE_INFO_CHANGED:
                        $logParams[MM_ActivityLog::$PARAM_BILLING_EVENT] = $eventType;
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_ID] = $params["order_id"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_NUMBER] = $params["order_number"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_TOTAL] = $params["order_total"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_DISCOUNT] = $params["order_discount"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_IP_ADDRESS] = $params["order_ip_address"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_PRODUCTS] = $params["order_products"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_COUPONS] = $params["order_coupons"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_PRORATIONS] = $params["order_prorations"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_AFFFILIATE_ID] = $params["order_affiliate_id"];
                        $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_SUBAFFFILIATE_ID] = $params["order_subaffiliate_id"];

                        $sql = $wpdb->prepare("SELECT payment_id FROM " . MM_TABLE_ORDERS . " WHERE id = %s LIMIT 1", $params["order_id"]);
                        $pymtServiceId = $wpdb->get_var($sql);

                        if ($pymtServiceId !== false) {
                            $pymtService = MM_PaymentServiceFactory::getPaymentServiceById($pymtServiceId);

                            if (!is_null($pymtService) && !$pymtService->supportsFeature(MM_PaymentServiceFeatures::MODIFY_REBILL_DATE)) {
                                $logParams[MM_ActivityLog::$PARAM_BILLING_ORDER_TOTAL] = "See " . $pymtService->getName() . " for details. ";
                            }
                        }

                        // get current rebill date for 'rebill date changed' event
                        if ($eventType == MM_Event::$BILLING_SUBSCRIPTION_REBILL_DATE_CHANGED && !empty($params["order_item_id"])) {
                            $orderItem = new MM_OrderItem($params["order_item_id"]);

                            if ($orderItem->isValid()) {
                                $scheduledPaymentEvent = MM_ScheduledPaymentEvent::findNextScheduledEventByOrderItemId($orderItem->getId());
                                if ($scheduledPaymentEvent instanceof MM_ScheduledPaymentEvent) {
                                    if ($scheduledPaymentEvent->isValid()) {
                                        $crntRebillDate = date("M d, Y", strtotime($scheduledPaymentEvent->getScheduledDate()));
                                        $logParams[MM_ActivityLog::$PARAM_BILLING_NEXT_REBILL_DATE] = $crntRebillDate;
                                    }
                                } else if ($scheduledPaymentEvent !== false) {
                                    $crntRebillDate = date("M d, Y", strtotime($scheduledPaymentEvent));
                                    $logParams[MM_ActivityLog::$PARAM_BILLING_NEXT_REBILL_DATE] = $crntRebillDate;
                                }
                            }
                        }

                        if ($eventType == MM_Event::$AFFILIATE_INFO_CHANGED) {
                            $logParams[MM_ActivityLog::$PARAM_AFFILIATE_TRACKING_EVENT] = $eventType;
                            MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_AFFILIATE_TRACKING, $logParams);
                        } else {
                            MM_ActivityLog::log($user, MM_ActivityLog::$EVENT_TYPE_BILLING, $logParams);
                        }

                        break;
                    default:
                        //call to parent;
                }
            }
        }
    }


    protected function checkEventAttributes(MM_Action $action)
    {
        $doContinue = true;

        switch ($action->getEventType()) {
            case MM_Event::$PRODUCT_PURCHASE:
                $eventAttributes = $action->getEventAttributes();
                if (is_array($eventAttributes) && isset($eventAttributes["product_id"])) {
                    $productId = $eventAttributes["product_id"];
                    if (intval($productId) > 0 && intval($productId) != intval($this->params["product_id"])) {
                        $doContinue = false;
                    }
                }
                break;

                // for the member added event, check to make sure that the membership level of the member
                // equals the push notification configuration
            case MM_Event::$MEMBER_ADD:
                $eventAttributes = $action->getEventAttributes();
                if (is_array($eventAttributes) && isset($eventAttributes["membership_level_id"])) {
                    $membershipLevelId = $eventAttributes["membership_level_id"];
                    if (intval($membershipLevelId) > 0 && intval($membershipLevelId) != intval($this->params["membership_level"])) {
                        $doContinue = false;
                    }
                }
                break;

                // for the membership level status change event, check to make sure that the membership level
                // and the status of the membership level equals the push notification configuration
            case MM_Event::$MEMBER_STATUS_CHANGE:
                $eventAttributes = $action->getEventAttributes();
                if (is_array($eventAttributes)) {
                    if (isset($eventAttributes["membership_level_id"]) || isset($eventAttributes["status_id"])) {
                        if (isset($eventAttributes["membership_level_id"])) {
                            $membershipLevelId = $eventAttributes["membership_level_id"];
                            if (intval($membershipLevelId) > 0 && intval($membershipLevelId) != intval($this->params["membership_level"])) {
                                $doContinue = false;
                            }
                        }

                        if (isset($eventAttributes["status_id"])) {
                            $statusId = $eventAttributes["status_id"];
                            if (intval($statusId) > 0 && intval($statusId) != intval($this->params["status"])) {
                                $doContinue = false;
                            }
                        }
                    }
                }
                break;

                // for the bundle added event, check to make sure that the bundle being added
                // equals the push notification configuration
            case MM_Event::$BUNDLE_ADD:
                $eventAttributes = $action->getEventAttributes();
                if (is_array($eventAttributes) && isset($eventAttributes["bundle_id"])) {
                    $bundleId = $eventAttributes["bundle_id"];
                    if (intval($bundleId) > 0 && intval($bundleId) != intval($this->params["bundle_id"])) {
                        $doContinue = false;
                    }
                }
                break;

                // for the bundle status change event, check to make sure that the bundle and the status
                // of the bundle equals the push notification configuration
            case MM_Event::$BUNDLE_STATUS_CHANGE:
                $eventAttributes = $action->getEventAttributes();
                if (is_array($eventAttributes)) {
                    if (isset($eventAttributes["bundle_id"]) || isset($eventAttributes["status_id"])) {
                        if (isset($eventAttributes["bundle_id"])) {
                            $bundleId = $eventAttributes["bundle_id"];
                            if (intval($bundleId) > 0 && intval($bundleId) != intval($this->params["bundle_id"])) {
                                $doContinue = false;
                            }
                        }

                        if (isset($eventAttributes["status_id"])) {
                            $statusId = $eventAttributes["status_id"];
                            if (intval($statusId) > 0 && intval($statusId) != intval($this->params["bundle_status"])) {
                                $doContinue = false;
                            }
                        }
                    }
                }
                break;
        }

        return $doContinue;
    }


    protected function createEmailContext(MM_Action $action)
    {
        $eventType = $action->getEventType();
        $actionValue = $action->getActionValue();
        $user = new MM_User($this->params["member_id"]);
        $fromEmployee = new MM_Employee($actionValue["emailFromId"]);

        switch ($eventType) {
            case MM_Event::$PRODUCT_PURCHASE:
                $context = new MM_Context($user, $fromEmployee);
                $context->setProduct(new MM_Product($this->params["product_id"]));
                $context->setOrder(new MM_Order($this->params["order_id"]));
                return $context;

            case MM_Event::$MEMBER_ADD:
            case MM_Event::$MEMBER_ACCOUNT_UPDATE:
            case MM_Event::$MEMBER_DELETE:
            case MM_Event::$MEMBER_MEMBERSHIP_CHANGE:
            case MM_Event::$MEMBER_STATUS_CHANGE:
            case MM_Event::$BUNDLE_ADD:
            case MM_Event::$BUNDLE_STATUS_CHANGE:
                return new MM_Context($user, $fromEmployee);

            case MM_Event::$PAYMENT_RECEIVED:
            case MM_Event::$PAYMENT_REBILL:
            case MM_Event::$PAYMENT_REBILL_DECLINED:
            case MM_Event::$REFUND_ISSUED:
            case MM_Event::$COMMISSION_INITIAL:
            case MM_Event::$COMMISSION_REBILL:
            case MM_Event::$CANCEL_COMMISSION:
                $order = new MM_Order($this->params["order_id"]);
                $context = new MM_Context($user, $fromEmployee, $order);
                $orderAttributes = $this->getOrderAttributes($eventType);
                $context->setOrderAttributes($orderAttributes);
                return $context;

            default:
                return null;
        }
    }


    protected function getOrderAttributes(string $eventType)
    {
        $isRebill = in_array($eventType, [
            MM_Event::$PAYMENT_REBILL,
            MM_Event::$COMMISSION_REBILL,
            MM_Event::$PAYMENT_REBILL_DECLINED
        ]);

        $orderAttributes = ["is_rebill" => $isRebill];

        if (isset($this->params["order_transaction_id"])) {
            $orderAttributes["transaction_id"] = $this->params["order_transaction_id"];
        }

        return $orderAttributes;
    }
}
