<?php
class MM_Email
{
    public static $BR = "<br>";

    private $subject;
    private $body;
    private $toName;
    private $ccAddress = array();
    private $ccAddressStrings = array();
    private $toAddress;
    private $fromName;
    private $fromAddress;
    private $attachments = array();
    private $context;
    private $doLog = true;

    public function send()
    {
        $req = array();

        if (empty($this->toAddress)) {
            return new MM_Response(__("Not a valid 'To' address.", "membermouse"), MM_Response::$ERROR);
        }

        if (empty($this->fromAddress)) {
            return new MM_Response(__("Not a valid 'From' address.", "membermouse"), MM_Response::$ERROR);
        }

        if (empty($this->fromName)) {
            return new MM_Response(__("No 'From Name' supplied.", "membermouse"), MM_Response::$ERROR);
        }

        if (empty($this->subject)) {
            return new MM_Response(__("Subject may not be empty.", "membermouse"), MM_Response::$ERROR);
        }
        if ($this->context instanceof MM_Context) {
            $subject = MM_SmartTagUtil::processContent($this->subject, $this->context);
            $body = MM_SmartTagUtil::processContent($this->body, $this->context);
        } else {
            $subject = $this->subject;
            $body = $this->body;
        }

        // execute short codes that aren't smart tags.
        $body = do_shortcode($body);
        $subject = do_shortcode($subject);

        $headers  = "MIME-Version: 1.0\r\n";
        $headers .= "Content-type: text/html; charset=utf-8\r\n";

        // Additional Headers
        if (is_array($this->ccAddress) && count($this->ccAddress) > 0) {
            foreach ($this->ccAddress as $email => $name) {
                $name = (empty($name)) ? $email : $name;
                $headers .= "Cc: {$name} <{$email}>\r\n";

                $this->ccAddressStrings[] = $email;
            }
        }

        $headers .= "From: {$this->fromName} <{$this->fromAddress}>\r\n";

        if (function_exists("wp_mail")) {
            @wp_mail($this->toAddress, $subject, nl2br($body), $headers, $this->attachments);
        } else {
            // PHP mail doesn't support attachments 
            @mail($this->toAddress, $subject, nl2br($body), $headers);
        }

        // log email being sent
        if ($this->doLog && $this->context instanceof MM_Context && !is_null($this->context->getUser())) {
            $params = array();
            $params[MM_ActivityLog::$PARAM_EMAIL_TO_ADDRESS] = $this->toAddress;
            $params[MM_ActivityLog::$PARAM_EMAIL_CC_ADDRESSES] = (is_array($this->ccAddressStrings) && count($this->ccAddressStrings) > 0) ? implode(", ", $this->ccAddressStrings) : "";
            $params[MM_ActivityLog::$PARAM_EMAIL_FROM_ADDRESS] = $this->fromAddress;
            $params[MM_ActivityLog::$PARAM_EMAIL_SUBJECT] = $subject;

            // remove password from email for security reasons
            $hiddenPasswordMsg = "## " . __("password hidden for security", "membermouse") . " ##";
            $this->body = str_ireplace("[mm_member_data name='password']", $hiddenPasswordMsg, $this->body);
            $this->body = str_ireplace('[mm_member_data name="password"]', $hiddenPasswordMsg, $this->body);

            $params[MM_ActivityLog::$PARAM_EMAIL_BODY] = MM_SmartTagUtil::processContent($this->body, $this->context);

            MM_ActivityLog::log($this->context->getUser(), MM_ActivityLog::$EVENT_TYPE_EMAIL, $params);
        }

        return new MM_Response();
    }

    public function disableLogging()
    {
        $this->doLog = false;
    }

    public function enableLogging()
    {
        $this->doLog = true;
    }

    public function setSubject($str)
    {
        $this->subject = $str;
    }

    public function getSubject()
    {
        return $this->subject;
    }

    public function setBody($str)
    {
        $this->body = $str;
    }

    public function getBody()
    {
        return $this->body;
    }

    public function setToName($str)
    {
        $this->toName = $str;
    }

    public function getToName()
    {
        return $this->toName;
    }

    public function setToAddress($str)
    {
        $this->toAddress = $str;
    }

    public function getToAddress()
    {
        return $this->toAddress;
    }

    public function setFromName($str)
    {
        $this->fromName = $str;
    }

    public function getCC()
    {
        return $this->ccAddress;
    }

    public function addCC($email, $name = "")
    {
        $this->ccAddress[$email] = $name;
    }

    public function getFromName()
    {
        return $this->fromName;
    }

    public function setFromAddress($str)
    {
        $this->fromAddress = $str;
    }

    public function getFromAddress()
    {
        return $this->fromAddress;
    }

    public function setAttachments($arr)
    {
        $this->attachments = $arr;
    }

    public function getAttachments()
    {
        return $this->attachments;
    }

    public function setContext(MM_Context $context)
    {
        $this->context = $context;
    }

    public function getContext()
    {
        return $this->context;
    }
}
