<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_Event
{
    public static $MEMBER_ADD = "mm_member_add";
    public static $MEMBER_MERGED = "mm_member_merged";
    public static $PRODUCT_PURCHASE = "mm_product_purchase";
    public static $MEMBER_STATUS_CHANGE = "mm_member_status_change";
    public static $MEMBER_MEMBERSHIP_CHANGE = "mm_member_membership_change";
    public static $MEMBER_ACCOUNT_UPDATE = "mm_member_account_update";
    public static $MEMBER_DELETE = "mm_member_delete";
    public static $BUNDLE_ADD = "mm_bundles_add";
    public static $BUNDLE_STATUS_CHANGE = "mm_bundles_status_change";
    public static $PAYMENT_RECEIVED = "mm_payment_received";
    public static $PAYMENT_REBILL = "mm_payment_rebill";
    public static $PAYMENT_REBILL_DECLINED = "mm_payment_rebill_declined";
    public static $REFUND_ISSUED = "mm_refund_issued";
    public static $BILLING_SUBSCRIPTION_UPDATED = "mm_billing_subscription_updated";
    public static $BILLING_SUBSCRIPTION_REBILL_DATE_CHANGED = "mm_billing_subscription_rebill_date_changed";
    public static $BILLING_SUBSCRIPTION_CANCELED = "mm_billing_subscription_canceled";
    public static $AFFILIATE_INFO_CHANGED = "mm_affiliate_info_changed";
    public static $COMMISSION_INITIAL = "mm_commission_initial";
    public static $COMMISSION_REBILL = "mm_commission_rebill";
    public static $CANCEL_COMMISSION = "mm_commission_cancel";
    public static $QUIZ_ATTEMPT_COMPLETED = "quiz-attempt-completed";
    public static $LESSON_STARTED = "lesson-started";
    public static $COURSE_STARTED = "course-started";
    public static $LESSON_COMPLETED = "lesson-completed";
    public static $COURSE_COMPLETED = "course-completed";
    public static $DUNNING_INTERVAL = "mm_billing_subscription_dunning_interval";
    public static $BILLING_SCAN_INTERVAL = "mm_billing_scan_interval";
    public static $REMINDER_SCAN_INTERVAL = "mm_reminder_scan_interval";
    public static $PURCHASE_COMPLETE = "mm_purchase_complete";

    //reminder event types
    public static $BILLING_REMINDER = "mm_billing_reminder";

    public static function packageMemberData($memberId, $acctInfoChanges = null)
    {
        $userData = array();

        $user = new MM_User($memberId);

        if ($user->isValid()) {
            $userData = array(
                'member_id' => $user->getId(),
                'registered' => $user->getRegistrationDate(),
                'cancellation_date' => "",
                'last_logged_in' => $user->getLastLoginDate(),
                'last_updated' => $user->getLastUpdatedDate(),
                'days_as_member' => $user->getDaysAsMember(),
                'status' => $user->getStatus(true),
                'status_name' => MM_Status::getName($user->getStatus(true)),
                'membership_level' => $user->getMembershipId(),
                'membership_level_name' => $user->getMembershipName(),
                'is_complimentary' => $user->isComplimentary() ? 'true' : 'false',
                'first_name' => $user->getFirstName(),
                'last_name' => $user->getLastName(),
                'username' => $user->getUsername(),
                'email' => $user->getEmail(),
                'phone' => $user->getPhone(),
                'billing_address' => $user->getBillingAddress(),
                'billing_address2' => $user->getBillingAddress2(),
                'billing_city' => $user->getBillingCity(),
                'billing_state' => $user->getBillingState(),
                'billing_zip_code' => $user->getBillingZipCode(),
                'billing_country' => $user->getBillingCountryName(),
                'shipping_address' => $user->getShippingAddress(),
                'shipping_address2' => $user->getShippingAddress2(),
                'shipping_city' => $user->getShippingCity(),
                'shipping_state' => $user->getShippingState(),
                'shipping_zip_code' => $user->getShippingZipCode(),
                'shipping_country' => $user->getShippingCountryName()
            );

            if ($user->isPendingCancellation()) {
                $userData['cancellation_date'] = $user->getCancellationDate();
            }

            // last values for account info changes
            if (!is_null($acctInfoChanges) && count($acctInfoChanges) > 0) {
                foreach ($acctInfoChanges as $key => $value) {
                    $userData["last_{$key}"] = $value;
                }
            }

            // custom fields
            $fields = MM_CustomField::getCustomFieldsList();
            foreach ($fields as $id => $val) {
                $customField = new MM_CustomField($id);

                if ($customField->isValid()) {
                    $userData["cf_" . $customField->getId()] = htmlentities($user->getCustomDataByFieldId($customField->getId())->getValue(), ENT_QUOTES, 'UTF-8', true);
                }
            }
        }

        return $userData;
    }

    public static function packageBundleData($memberId, $bundleId)
    {
        $userData = array();

        $user = new MM_User($memberId);

        if ($user->isValid()) {
            $userData = self::packageMemberData($memberId);
            $appliedBundle = MM_AppliedBundle::getAppliedBundle($memberId, $bundleId);

            if ($appliedBundle->isValid()) {
                $userData["bundle_id"] = $appliedBundle->getBundleId();

                if ($bundleId == MM_AppliedBundle::$SAMPLE_APPLIED_BUNDLE_ID) {
                    $userData["bundle_name"] = __("Sample Bundle", "membermouse");
                    $userData["days_with_bundle"] = "5";
                } else {
                    $userData["bundle_name"] = $appliedBundle->getBundle()->getName();
                    $userData["days_with_bundle"] = $user->getDaysWithBundle($appliedBundle->getBundleId());
                }

                $userData["bundle_is_complimentary"] = $appliedBundle->isComplimentary() ? 'true' : 'false';
                $userData["bundle_status"] = $appliedBundle->getStatus();
                $userData["bundle_status_name"] = MM_Status::getName($appliedBundle->getStatus());
                $userData["bundle_date_added"] = $appliedBundle->getApplyDate();
                $userData["bundle_last_updated"] = $appliedBundle->getStatusUpdatedDate();
                $userData['bundle_cancellation_date'] = "";

                if ($appliedBundle->isPendingCancellation()) {
                    $userData['bundle_cancellation_date'] = $appliedBundle->getCancellationDate();;
                }
            }
        }

        return $userData;
    }


    public static function packageProductData($productId, $userId, $orderId, $orderItemId = null)
    {
        $userProductData = self::packageOrderData($userId, $orderId, $orderItemId);
        $product = new MM_Product($productId);
        if ($product->isValid()) {
            $key = "product_";
            $userProductData[$key . "id"] = $product->getId();
            $userProductData[$key . "reference_key"] = $product->getReferenceKey();
            $userProductData[$key . "name"] = $product->getName();
            $userProductData[$key . "sku"] = $product->getSku();
            $userProductData[$key . "description"] = $product->getDescription();
            $userProductData[$key . "price"] = $product->getPrice(false);
            $userProductData[$key . "currency"] = $product->getCurrency();

            $rebillData = MM_OrderUtils::getRebillAmounts(new MM_Order($orderId));
            $userProductData[$key . "rebill_total"] = $rebillData["rebill_total"];
            $userProductData[$key . "rebill_shipping"] = $rebillData["rebill_shipping"];
            $userProductData[$key . "rebill_subtotal"] = $rebillData["rebill_subtotal"];
            $userProductData[$key . "rebill_discount"] = $rebillData["rebill_discount"];

            $userProductData[$key . "rebill_period"] = $product->getRebillPeriod();
            $userProductData[$key . "rebill_frequency"] = $product->getRebillFrequency();
            $userProductData[$key . "limit_payments"] = $product->doLimitPayments();
            $userProductData[$key . "number_of_payments"] = 0;
            if ($product->doLimitPayments()) {
                $userProductData[$key . "number_of_payments"] = $product->getNumberOfPayments();
            }

            $userProductData[$key . "has_trial"] = $product->hasTrial();
            $userProductData[$key . "limit_trial"] = $product->doLimitTrial();
            $userProductData[$key . "limit_trial_alt_product_id"] = 0;
            if ($product->doLimitTrial()) {
                $userProductData[$key . "limit_trial_alt_product_id"] = $product->getLimitTrialAltProductId();
            }

            $userProductData[$key . "is_shippable"] = $product->isShippable();
            $userProductData[$key . "trial_amount"] = 0;
            $userProductData[$key . "trial_duration"] = 0;
            $userProductData[$key . "trial_frequency"] = 0;
            if ($product->hasTrial()) {
                $userProductData[$key . "trial_amount"] = $product->getTrialAmount();
                $userProductData[$key . "trial_duration"] = $product->getTrialDuration();
                $userProductData[$key . "trial_frequency"] = $product->getTrialFrequency();
            }
        }
        return $userProductData;
    }


    public static function packageOrderData($memberId, $orderId, $orderItemId = null, $transactionId = null, $isRebill = false)
    {
        $userData = array();

        $user = new MM_User($memberId);

        if ($user->isValid()) {
            $userData = self::packageMemberData($memberId);
            $order = new MM_Order($orderId);

            if ($order->isValid()) {
                $userData["order_id"] = $orderId;
                $userData["order_number"] = $order->getOrderNumber();

                $userData["order_item_id"] = "";
                if (!empty($orderItemId)) {
                    $userData["order_item_id"] = $orderItemId;
                }

                $userData["order_transaction_id"] = "";
                if (!empty($transactionId)) {
                    $userData["order_transaction_id"] = $transactionId;
                }

                $userData["order_ip_address"] = $order->getIPAddress();
                $userData["order_currency"] = $order->getCurrency();
                $userData["order_shipping_method"] = $order->getShippingOptionDescription();
                $userData["order_billing_address"] = $order->getBillingAddress()->getAddressLine1();
                $userData["order_billing_address2"] = $order->getBillingAddress()->getAddressLine2();
                $userData["order_billing_city"] = $order->getBillingAddress()->getCity();
                $userData["order_billing_state"] = $order->getBillingAddress()->getState();
                $userData["order_billing_zipcode"] = $order->getBillingAddress()->getPostalCode();
                $userData["order_billing_country"] = $order->getBillingAddress()->getCountry();
                $userData["order_shipping_address"] = $order->getShippingAddress()->getAddressLine1();
                $userData["order_shipping_address2"] = $order->getShippingAddress()->getAddressLine2();
                $userData["order_shipping_city"] = $order->getShippingAddress()->getCity();
                $userData["order_shipping_state"] = $order->getShippingAddress()->getState();
                $userData["order_shipping_zipcode"] = $order->getShippingAddress()->getPostalCode();
                $userData["order_shipping_country"] = $order->getShippingAddress()->getCountry();
                $userData["order_affiliate_id"] = $order->getAffiliateId();
                $userData["order_subaffiliate_id"] = $order->getSubAffiliateId();

                $orderItems = $order->getProducts();
                $productInfo = array();

                foreach ($orderItems as $orderItem) {
                    $product = new MM_Product($orderItem->getItemId());

                    if ($product->isValid()) {
                        $crntProductInfo = array();
                        $crntProductInfo["id"] = $product->getId();
                        $crntProductInfo["name"] = $product->getName();
                        $crntProductInfo["sku"] = $product->getSku();
                        $crntProductInfo["amount"] = $orderItem->getAmount();
                        $crntProductInfo["quantity"] = $orderItem->getQuantity();
                        $crntProductInfo["total"] = $orderItem->getTotal();
                        $crntProductInfo["is_recurring"] = $orderItem->isRecurring();
                        $crntProductInfo["recurring_amount"] = $orderItem->getRecurringAmount();
                        $crntProductInfo["rebill_period"] = $orderItem->getRebillPeriod();
                        $crntProductInfo["rebill_frequency"] = $orderItem->getRebillFrequency();

                        $productInfo[] = $crntProductInfo;
                    }
                }

                $userData["order_products"] = json_encode($productInfo);

                // determine order total and discount amount based on if this is a rebill payment or not
                if (!$isRebill) {
                    $userData["order_date"] = $order->getDateAdded();
                    $userData["order_total"] = $order->getTotal();
                    $userData["order_subtotal"] = $order->getSubtotal();
                    $userData["order_shipping"] = $order->getShipping();
                    $userData["order_discount"] = abs(floatval($order->getDiscount()));
                } else {
                    $data = MM_OrderUtils::getRebillAmounts($order);
                    $userData["order_date"] = $data["rebill_date"];
                    $userData["order_total"] = $data["rebill_total"];
                    $userData["order_subtotal"] = $data["rebill_subtotal"];
                    $userData["order_shipping"] = $data["rebill_shipping"];
                    $userData["order_discount"] = $data["rebill_discount"];
                }

                $orderItems = $order->getCoupons();
                $couponInfo = array();

                foreach ($orderItems as $orderItem) {
                    $coupon = new MM_Coupon($orderItem->getItemId());

                    if ($coupon->isValid()) {
                        if ((!$isRebill) || ($isRebill && $coupon->getRecurringBillingSetting() == MM_Coupon::$TYPE_RECURRING_BILLING_SETTING_ALL)) {
                            $crntCouponInfo = array();
                            $crntCouponInfo["id"] = $coupon->getId();
                            $crntCouponInfo["name"] = $coupon->getCouponName();
                            $crntCouponInfo["code"] = $coupon->getCouponCode();

                            $couponInfo[] = $crntCouponInfo;
                        }
                    }
                }

                $userData["order_coupons"] = json_encode($couponInfo);

                $orderItems = $order->getProrations();
                $prorationInfo = array();

                foreach ($orderItems as $orderItem) {
                    $crntProrationInfo = array();
                    $crntProrationInfo["amount"] = $orderItem->getTotal();
                    $crntProrationInfo["description"] = $orderItem->getDescription();

                    $prorationInfo[] = $crntProrationInfo;
                }

                $userData["order_prorations"] = json_encode($prorationInfo);
            }
        }

        return $userData;
    }

    public static function packageAffiliateData(MM_CommissionProfile $commissionProfile, $affiliateId, $orderNumber, $orderTotal, $orderData, $isRebill = false)
    {
        $affiliateData = array();

        $affiliateData["member_id"] = $orderData["member_id"];
        $affiliateData["order_id"] = $orderData["order_id"];
        $affiliateData["order_affiliate_id"] = $affiliateId;
        $affiliateData["order_subaffiliate_id"] = $orderData["order_subaffiliate_id"];
        $affiliateData["order_number"] = $orderNumber;
        $affiliateData["order_total"] = $orderTotal;
        $affiliateData["order_ip_address"] = $orderData["order_ip_address"];
        $affiliateData["order_products"] = $orderData["order_products"];
        $affiliateData["order_coupons"] = $orderData["order_coupons"];
        $affiliateData["order_prorations"] = $orderData["order_prorations"];

        $affiliateData["rebill_commission_flat_rate"] = "";
        $affiliateData["rebill_commission_percent"] = "";

        if ($isRebill && $commissionProfile->getRebillCommissionType() != MM_CommissionProfile::$COMMISSION_TYPE_DEFAULT) {
            switch ($commissionProfile->getRebillCommissionType()) {
                case MM_CommissionProfile::$COMMISSION_TYPE_PERCENT:
                    $affiliateData["rebill_commission_percent"] = intval($commissionProfile->getRebillCommissionValue()) / 100;
                    break;

                case MM_CommissionProfile::$COMMISSION_TYPE_FLATRATE:
                    $affiliateData["rebill_commission_flat_rate"] = $commissionProfile->getRebillCommissionValue();
                    break;
            }
        }

        return $affiliateData;
    }


    public static function packageReminderData(MM_ScheduledReminder $scheduledReminder)
    {
        $reminderData = [];
        $reminder = MM_Reminder::create($scheduledReminder->getReminderId());
        if (($scheduledReminder != null) && $reminder->isValid()) {
            $reminderData["reminder_id"] = $reminder->getId();
            $reminderData["reminder_unit_value"] = $reminder->getUnitValue();
            $reminderData["reminder_unit_of_measure"] = $reminder->getUnitOfMeasure();

            $orderData = static::packageOrderData($scheduledReminder->getUserId(), $scheduledReminder->getOrderId(), $scheduledReminder->getOrderItemId(), null, true); //reminders are always for upcoming rebills
            $reminderData = array_merge($reminderData, $orderData);
        }
        return $reminderData;
    }


    public static function getName($eventType, $doLowercase = false)
    {
        $eventName = "";

        switch ($eventType) {
            case self::$PRODUCT_PURCHASE:
                $eventName = __("Product Purchased", "membermouse");
                break;

            case self::$MEMBER_ADD:
                $eventName = __("Member Added", "membermouse");
                break;

            case self::$MEMBER_ACCOUNT_UPDATE:
                $eventName = __("Member Account Updated", "membermouse");
                break;

            case self::$MEMBER_STATUS_CHANGE:
                $eventName = __("Membership Status Changed", "membermouse");
                break;

            case self::$MEMBER_MEMBERSHIP_CHANGE:
                $eventName = __("Membership Level Changed", "membermouse");
                break;

            case self::$MEMBER_DELETE:
                $eventName = __("Member Deleted", "membermouse");
                break;

            case self::$BUNDLE_ADD:
                $eventName = __("Bundle Added", "membermouse");
                break;

            case self::$BUNDLE_STATUS_CHANGE:
                $eventName = __("Bundle Status Changed", "membermouse");
                break;

            case self::$PAYMENT_RECEIVED:
                $eventName = __("Payment Received", "membermouse");
                break;

            case self::$PAYMENT_REBILL:
                $eventName = __("Rebill Payment Received", "membermouse");
                break;

            case self::$PAYMENT_REBILL_DECLINED:
                $eventName = __("Rebill Payment Failed", "membermouse");
                break;

            case self::$REFUND_ISSUED:
                $eventName = __("Refund Issued", "membermouse");
                break;

            case self::$COMMISSION_INITIAL:
                $eventName = __("Affiliate Commission (Initial)", "membermouse");
                break;

            case self::$COMMISSION_REBILL:
                $eventName = __("Affiliate Commission (Rebill)", "membermouse");
                break;

            case self::$CANCEL_COMMISSION:
                $eventName = __("Cancel Affiliate Commission", "membermouse");
                break;

            case self::$BILLING_REMINDER:
                $eventName = "Billing Reminder";
                break;

            case self::$PURCHASE_COMPLETE:
                $eventName = "Purchase Complete";
                break;
        }

        return ($doLowercase) ? strtolower($eventName) : $eventName;
    }

    public static function getEventTypesList($selectedEvent = "")
    {
        $list = array();
        // member events
        $list[MM_Event::$MEMBER_ADD] = MM_Event::getName(MM_Event::$MEMBER_ADD);
        $list[MM_Event::$MEMBER_ACCOUNT_UPDATE] = MM_Event::getName(MM_Event::$MEMBER_ACCOUNT_UPDATE);
        $list[MM_Event::$MEMBER_STATUS_CHANGE] = MM_Event::getName(MM_Event::$MEMBER_STATUS_CHANGE);
        $list[MM_Event::$MEMBER_MEMBERSHIP_CHANGE] = MM_Event::getName(MM_Event::$MEMBER_MEMBERSHIP_CHANGE);
        $list[MM_Event::$MEMBER_DELETE] = MM_Event::getName(MM_Event::$MEMBER_DELETE);

        // bundle events
        $list[MM_Event::$BUNDLE_ADD] = MM_Event::getName(MM_Event::$BUNDLE_ADD);
        $list[MM_Event::$BUNDLE_STATUS_CHANGE] = MM_Event::getName(MM_Event::$BUNDLE_STATUS_CHANGE);

        // order events
        $list[MM_Event::$PRODUCT_PURCHASE] = MM_Event::getName(MM_Event::$PRODUCT_PURCHASE);
        $list[MM_Event::$PAYMENT_RECEIVED] = MM_Event::getName(MM_Event::$PAYMENT_RECEIVED);
        $list[MM_Event::$PAYMENT_REBILL] = MM_Event::getName(MM_Event::$PAYMENT_REBILL);
        $list[MM_Event::$PAYMENT_REBILL_DECLINED] = MM_Event::getName(MM_Event::$PAYMENT_REBILL_DECLINED);
        $list[MM_Event::$REFUND_ISSUED] = MM_Event::getName(MM_Event::$REFUND_ISSUED);
        //$list[MM_Event::$PURCHASE_COMPLETE] = MM_Event::getName(MM_Event::$PURCHASE_COMPLETE);

        // affiliate events
        $list[MM_Event::$COMMISSION_INITIAL] = MM_Event::getName(MM_Event::$COMMISSION_INITIAL);
        $list[MM_Event::$COMMISSION_REBILL] = MM_Event::getName(MM_Event::$COMMISSION_REBILL);
        $list[MM_Event::$CANCEL_COMMISSION] = MM_Event::getName(MM_Event::$CANCEL_COMMISSION);

        return $list;
    }


    public static function getReminderTypesList($selectedEvent = "")
    {
        //reminder event types
        $list = [static::$BILLING_REMINDER => MM_Event::getName(static::$BILLING_REMINDER)];
        return $list;
    }


    /**
     * safeDoAction wraps calls to WP do_action so that errors generated by actions hooked to critical functions can be intercepted and logged without crashing in the middle 
     * of the operation. Use this sparingly
     * 
     * @param string $actionName The name of the WP action
     * @param mixed ...$args The args to pass to the WP action
     */
    public static function safeDoAction($actionName, ...$args)
    {
        try {
            return do_action($actionName, ...$args);
        } catch (Error | Exception $e) {
            $logMessage = "MemberMouse caught " . get_class($e) . " in Action Hook: " . $e->getMessage() . " in " . $e->getFile() . ":" . $e->getLine();
            $logMessage .= "\nStack Trace:\n" . $e->getTraceAsString();
            error_log($logMessage);
            MM_DiagnosticLog::log(MM_DiagnosticLog::$PHP_ERROR, $logMessage);
            return;
        }
    }
}
