<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_Reminder extends MM_Action
{
    //reminder-specific properties
    protected $unitValue;
    protected $unitOfMeasure;
    protected $sortValue;

    protected $productIds = [];

    public function getData()
    {
        global $wpdb;

        $sql = "SELECT * FROM " . MM_TABLE_REMINDERS . " WHERE id='" . $this->id . "';";
        $result = $wpdb->get_row($sql);

        if ($result) {
            $this->setData($result);
        } else {
            parent::invalidate();
        }
    }


    public function setData($data)
    {
        global $wpdb;

        try {
            $this->eventType = MM_Event::$BILLING_REMINDER;
            $this->actionType = $data->action_type;
            $this->actionValue = unserialize($data->action_value);
            $this->eventAttributes = unserialize($data->event_attributes);
            $this->status = $data->status;
            $this->unitValue = $data->unit_value;
            $this->unitOfMeasure = $data->unit_of_measure;
            $this->sortValue = $data->sort_value;
            parent::validate();

            if ($data->has_product_filter == 1) {
                $this->productIds = $wpdb->get_col($wpdb->prepare("SELECT product_id FROM " . MM_TABLE_REMINDERS_PRODUCTS . " WHERE reminder_id = %d", $this->id));
            }
        } catch (Exception $ex) {
            parent::invalidate();
        }
    }

    /**
     * Insert or update this reminder into the database
     *
     * @return MM_Response Success response if save was successful, error response otherwise
     */
    public function commitData()
    {
        global $wpdb;

        if (intval($this->id) > 0) {
            $result = $wpdb->update(MM_TABLE_REMINDERS, [
                'action_type'          => $this->actionType,
                'action_value'          => serialize($this->actionValue),
                'event_attributes'      => serialize($this->eventAttributes),
                'status'              => $this->status,
                'unit_value'          => $this->unitValue,
                'unit_of_measure'      => $this->unitOfMeasure,
                'sort_value'          => $this->getSortValue(),
                'has_product_filter' => $this->hasProductFilter() ? 1 : 0
            ], ["id" => $this->id]);

            if ($result === false) {
                return new MM_Response("Unable to save reminder", MM_Response::$ERROR);
            }
            parent::validate();
        } else {
            $result = $wpdb->insert(MM_TABLE_REMINDERS, [
                'action_type'          => $this->actionType,
                'action_value'          => serialize($this->actionValue),
                'event_attributes'      => serialize($this->eventAttributes),
                'status'              => $this->status,
                'unit_value'          => $this->unitValue,
                'unit_of_measure'      => $this->unitOfMeasure,
                'sort_value'          => $this->getSortValue(),
                'has_product_filter' => $this->hasProductFilter() ? 1 : 0
            ]);

            if ($result === false) {
                return new MM_Response("Unable to create reminder", MM_Response::$ERROR);
            } else {
                $this->id = $wpdb->insert_id;
                parent::validate();
            }
        }
        $wpdb->delete(MM_TABLE_REMINDERS_PRODUCTS, ["reminder_id" => $this->id]);
        if ($this->hasProductFilter()) {
            foreach ($this->productIds as $productId) {
                $wpdb->insert(MM_TABLE_REMINDERS_PRODUCTS, ["reminder_id" => $this->id, "product_id" => $productId]);
            }
        }
        return new MM_Response();
    }


    /**
     * Delete this reminder
     * 
     * @return MM_Response Success response if delete was successful, error response otherwise
     */
    public function delete()
    {
        global $wpdb;

        $res = $wpdb->delete(MM_TABLE_REMINDERS, ["id" => $this->id]);
        if ($res) {
            $wpdb->delete(MM_TABLE_REMINDERS_PRODUCTS, ["reminder_id" => $this->id]);
            $wpdb->delete(MM_TABLE_SCHEDULED_REMINDERS, ["reminder_id" => $this->id, "status" => MM_ScheduledReminder::PENDING_STATUS]);
            $this->invalidate();
            return new MM_Response();
        }

        return new MM_Response("Unable to delete reminder", MM_Response::$ERROR);
    }


    /**
     * Whether or not this reminder has a product filter
     * 
     * @return boolean true if product filter exists, false otherwise
     */
    public function hasProductFilter()
    {
        return (count($this->productIds) > 0);
    }


    public function setProductFilter($products)
    {
        if (!is_array($products)) {
            $this->productIds = [];
            return;
        }
        $this->productIds = array_map('intval', array_filter($products, 'is_numeric'));
    }


    public function getProductIds()
    {
        return $this->productIds;
    }

    /**
     * Convert the trigger time period into days for use as a sort comparator
     * 
     * @param integer $unitValue The number of units for the trigger time period
     * @param String $unitOfMeasure Should be one of "days","weeks","months","years", and denotes the unit of measure for $unitValue
     * @return number The number of days for the trigger time period
     */
    protected function getSortValue($unitValue = null, $unitOfMeasure = null)
    {
        $unitValue = $unitValue ?? $this->unitValue;
        $unitOfMeasure = strtolower($unitOfMeasure ?? $this->unitOfMeasure);
        if (!is_numeric($unitValue) || !in_array($unitOfMeasure, ["days", "weeks", "months", "years"])) {
            return 0;
        }

        //convert "{$unitValue} {$unitOfMeasure}" to days
        switch ($unitOfMeasure) {
            case "weeks":
                return $unitValue * 7;
            case "months":
                return $unitValue * 30;
            case "years":
                return $unitValue * 365;
            default:
                return $unitValue;
        }
    }


    /**
     * @return string
     */
    public function getUnitValue()
    {
        return $this->unitValue;
    }

    /**
     * @return string
     */
    public function getUnitOfMeasure()
    {
        return $this->unitOfMeasure;
    }

    /**
     * @param int $unitValue The number of time units
     */
    public function setUnitValue($unitValue)
    {
        $this->unitValue = intval($unitValue);
    }

    /**
     * @param string $unitOfMeasure The type of time unit that $unitValue represents
     */
    public function setUnitOfMeasure($unitOfMeasure)
    {
        $unitOfMeasure = strtolower($unitOfMeasure);
        if (!in_array($unitOfMeasure, ['days', 'weeks', 'months', 'years'])) {
            $unitOfMeasure = 'days';
        }
        $this->unitOfMeasure = $unitOfMeasure;
    }
}
