<?php
class MM_DefaultEmailServiceProvider extends MM_AbstractEmailServiceProvider
{
    public function __construct($provider_info)
    {
        parent::__construct($provider_info);
    }

    /**
     * Returns an empty array, since no lists can be configured using the default provider
     */
    public function getLists()
    {
        return new MM_Response(array());
    }

    /**
     * Returns an object capable of rendering the default options dialog
     * @param Array $post Any data posted necessary to draw the options dialog
     * @return Object of type MM_EmailServiceProviderDialog representing the dialog to draw
     */
    public function renderOptionsDialog($post)
    {
        $saveButtonText = __("Save Settings", "membermouse");
        $form = new MM_EmailServiceProviderDialog();
        $form->controls = '<button id="email-service-provider-save" class="mm-ui-button blue" onClick="mmjs.providerOptionsSave(\'\'); return false;" style="cursor:pointer;">' . $saveButtonText . '</button>';
        return $form;
    }

    /**
     * Responds to data collected from the form rendered by renderOptionsDialog
     * @see MM_AbstractEmailServiceProvider::handleOptionsDialog()
     * @param $optionData Array containing the posted values
     * @return MM_Response object indicating success or failure
     */
    public function handleOptionsDialog($optionData)
    {
        if (is_array($optionData)) {
            $provider_token = $optionData['provider_token'];

            $provider_entity = new MM_EmailServiceProvider();
            $provider_entity->setToken($provider_token);
            $provider_entity->getData();
            if ($provider_entity->isValid()) {
                //first write the provider's configuration options
                $provider_entity->setActive(true);
                $commit_response = $provider_entity->commitData();
                if ($commit_response->type == MM_Response::$ERROR) {
                    //pass through errors committing to the database
                    return $commit_response;
                } else {
                    return new MM_Response(__("The Default Mail Service Provider has been configured successfully", "membermouse"), MM_Response::$SUCCESS);
                }
            } else {
                //failed to load entity from table
                return new MM_Response("Failed to save settings", MM_Response::$ERROR);
            }
        }
    }
}
