<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 *
 */
abstract class MM_Extension
{
    public static $OPTION_PREFIX               = "mm-extension-";
    public static $USERVOICE_TOKEN          = "USERVOICE";
    public static $FACEBOOK_LOGIN_TOKEN     = "Facebook";
    public static $LINKEDIN_LOGIN_TOKEN     = "LinkedIn";
    public static $GOOGLE_LOGIN_TOKEN       = "Google"; //social login tokens must start with an uppercase letter, followed by all lowercase!
    public static $TWITTER_LOGIN_TOKEN      = "Twitter";
    public static $GOOGLE_ECOMMERCE_TOKEN    = "google_ecommerce";
    public static $BBPRESS_TOKEN              = "bbpress";
    public static $MM_COURSES_TOKEN          = "mmcourses";
    public static $PDF_RECEIPTS_TOKEN        = "pdf_receipts";

    protected $token = "";

    public function __construct($autoloadDependencies = true, $configData = "") {}

    /**
     * Allow extensions to execute code in response to the WordPress init hook
     */
    public function doInitHook() {}

    public static function performInitActions()
    {
        if (class_exists("MM_ExtensionsFactory")) {
            $socialLoginProviders = array();
            $activeExtensions = MM_ExtensionsFactory::getActiveExtensions();
            foreach ($activeExtensions as $activeExtension) {
                if (!($activeExtension instanceof MM_AbstractSocialLoginExtension)) {
                    $activeExtension->doInitHook();
                } else {
                    $socialLoginProviders[] = $activeExtension;
                }
            }

            if (count($socialLoginProviders) > 0) {
                //doInitHook only needs to be called for one of the social login providers, because as a group they share the same init hook
                $hookHasRun = false;
                do {
                    $slp = array_shift($socialLoginProviders);
                    if ($slp->isActive() && $slp->hasPermission()) {
                        $slp->doInitHook();
                        $hookHasRun = true;
                    }
                } while ((count($socialLoginProviders) > 0) && (!$hookHasRun));
            }
        }
    }

    public function hasPermission()
    {
        return false;
    }

    public function displayListingItem()
    {
        return "";
    }

    public function displayConfigScreen()
    {
        return "";
    }

    public function isActive()
    {
        return false;
    }

    public function getOption($name, $defaultValue = "")
    {
        return get_option(self::$OPTION_PREFIX . strtolower($this->token) . "-{$name}", $defaultValue);
    }

    public function setOption($name, $value)
    {
        update_option(self::$OPTION_PREFIX . strtolower($this->token) . "-{$name}", $value);
    }

    public function loadDependencies() {}
}
