<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 *
 */
class MM_PdfReceiptsExtension extends MM_PluginExtension
{
    /**
     * Constructor
     * @param boolean $autoloadDependencies Whether to autoload dependencies, default true.
     * @param string $configData Config data. Not currently used with this extension.
     */
    public function __construct($autoloadDependencies = true, $configData = "")
    {
        $this->token = MM_Extension::$PDF_RECEIPTS_TOKEN;
        parent::__construct($autoloadDependencies, $configData);
    }

    /**
     * Initializes extension data. Called by the parent constructor if the extension is not marked as initialized.
     * @return bool Whether the extension data was successfully initialized.
     */
    protected function initializeExtensionData(): bool
    {
        // Extension Information
        static::$EXTENSION_NAME = "MemberMouse PDF Receipts";
        static::$EXTENSION_MAIN_FILE = "membermouse-pdf-receipts.php";
        static::$EXTENSION_MAIN_CLASSNAME = "MemberMouse_PDF_Receipts";
        static::$EXTENSION_ADMIN_PATH = "admin.php?page=membermouse_pdf_receipts_settings";
        static::$EXTENSION_REQUIRED_PERMISSION = "";

        static::$EXTENSION_LOGO_URL = MM_IMAGES_URL . "pdf-receipts-logo.png";
        static::$EXTENSION_INFO_IMAGE_URL = MM_IMAGES_URL . "pdf-receipts-sample.png";
        static::$EXTENSION_SUPPORT_URL = "https://membermouse.com/docs/send-automated-customizable-pdf-receipts-to-members";

        if (function_exists("__")) {
            static::$EXTENSION_NAME = __("MemberMouse PDF Receipts", "membermouse");
            static::$EXTENSION_UPGRADE_MESSAGE = "";
            static::$EXTENSION_DESCRIPTION = __(
                "Effortlessly generate and send itemized PDF receipts to your members with every payment, and " .
                    "provide self-service receipt downloads for previous transactions.",
                "membermouse"
            );

            static::$EXTENSION_FEATURES = [
                __("Fully customizable HTML email with attached PDF", "membermouse"),
                __("Include your business name, address, and a logo", "membermouse"),
                __("Two receipt footer areas that accept custom HTML", "membermouse"),
                __("Self-service receipt downloads", "membermouse"),
                __("Allow members to specify an alternative billing address and a CC email address", "membermouse"),
            ];
        } else {
            // Translations weren't available. This shouldn't happen, but if it does, we want to try again
            return false;
        }

        return true;
    }


    /**
     * Gets the URL of the plugin package. In the future, the URL will be retrieved from Mothership. Ideally, this
     * would be abstracted to MM_PluginExtension. Overriding here because that system is not yet in place.
     * @return string Extension download URL.
     */
    protected function getExtensionDownloadUrl(): string
    {
        return "https://membermouse.com/resources/membermouse-pdf-receipts-latest.zip";
    }
}
