<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 *
 */
abstract class MM_PluginExtension extends MM_Extension
{
    // Extension Information
    protected static $EXTENSION_NAME = "";
    protected static $EXTENSION_MAIN_FILE = "";
    protected static $EXTENSION_MAIN_CLASSNAME = "";
    protected static $EXTENSION_ADMIN_PATH = "";
    protected static $EXTENSION_REQUIRED_PERMISSION = "";
    protected static $EXTENSION_INITIALIZED = false;

    protected static $EXTENSION_LOGO_URL = "";
    protected static $EXTENSION_INFO_IMAGE_URL = "";
    protected static $EXTENSION_SUPPORT_URL = "";
    protected static $EXTENSION_DOWNLOAD_URL = "";

    protected static $EXTENSION_DESCRIPTION = "";
    protected static $EXTENSION_UPGRADE_MESSAGE = "";
    protected static $EXTENSION_FEATURES = [];

    // Settings
    protected static $SETTINGS_POST_INSTALL_REDIRECT = false;
    protected static $SETTINGS_ACTIVE_EXTENSION_REDIRECT = true;


    /**
     * Initializes extension data. Called by the constructor the first time the extension is loaded.
     * @return bool Whether the extension data was successfully initialized.
     */
    protected function initializeExtensionData(): bool
    {
        return false;
    }


    /**
     * Constructor
     * @param boolean $autoloadDependencies Whether to autoload dependencies, default true.
     * @param string $configData Config data. Not currently used with this extension.
     */
    public function __construct($autoloadDependencies = true, $configData = "")
    {
        parent::__construct($autoloadDependencies, $configData);

        if (!static::$EXTENSION_INITIALIZED) {
            static::$EXTENSION_INITIALIZED = $this->initializeExtensionData();
        }

        if (wp_doing_ajax()) {
            add_action("wp_ajax_mm_{$this->token}_landing_action", array($this, "extensionLandingAjaxHandler"));
        }
    }


    /**
     * Gets data about this extension, which will be passed to the template.
     * @return array An associative array of extension data.
     */
    public function getExtensionInformation(): array
    {
        return [
            "extensionToken"        => $this->token,
            "extensionName"            => static::$EXTENSION_NAME,
            "extensionDescription"    => static::$EXTENSION_DESCRIPTION,
            "extensionFeatures"        => static::$EXTENSION_FEATURES,
            "extensionLogoUrl"        => static::$EXTENSION_LOGO_URL,
            "infoImageUrl"            => static::$EXTENSION_INFO_IMAGE_URL,
            "supportArticleUrl"        => static::$EXTENSION_SUPPORT_URL,
            "isInstalled"            => $this->isInstalled(),
            "isActive"                => $this->isActive(),
            "hasPermission"            => $this->hasPermission(),
            "requiredPermission"    => static::$EXTENSION_REQUIRED_PERMISSION,
            "upgradeMessage"        => static::$EXTENSION_UPGRADE_MESSAGE,
        ];
    }


    /**
     * Indicates whether this extension has been installed.
     * @return boolean Whether the extension is installed.
     */
    public function isInstalled(): bool
    {
        $plugins = get_plugins();
        $isInstalled = false;

        foreach ($plugins as $pluginSlug => $pluginData) {
            if (strpos($pluginSlug, static::$EXTENSION_MAIN_FILE) !== false) {
                $isInstalled = true;
                break;
            }
        }

        return $isInstalled;
    }


    /**
     * Indicates whether this extension is active.
     * @return boolean Whether the extension is active.
     */
    public function isActive(): bool
    {
        return class_exists(static::$EXTENSION_MAIN_CLASSNAME);
    }


    /**
     * Indicates whether this extension has permission to run on the current license.
     * @return boolean Whether the extension has permission to run.
     */
    public function hasPermission(): bool
    {
        if (!empty(static::$EXTENSION_REQUIRED_PERMISSION)) {
            return (MM_MemberMouseService::hasPermission(static::$EXTENSION_REQUIRED_PERMISSION) == MM_MemberMouseService::$ACTIVE);
        }

        return true;
    }


    /**
     * Display the listing for this extension on the Extensions page. Called for each extension
     * when the listing is rendered.
     */
    public function displayListingItem(): void
    {
        $shouldRedirect = (static::$SETTINGS_ACTIVE_EXTENSION_REDIRECT && $this->isActive());
        $extUrl = $shouldRedirect ? admin_url(static::$EXTENSION_ADMIN_PATH) : $this->getExtensionLandingPageUrl();
?>
        <div class="mm-extension <?php echo $this->isActive() ? 'mm-extension-active' : ''; ?>">
            <a href="<?php echo $extUrl; ?>"><img src="<?php echo static::$EXTENSION_LOGO_URL; ?>" /></a>
        </div>
<?php
    }


    /**
     * Invokes the template module to display the landing page for this extension. Within the template,
     * data about this extension is provided via the $info variable, which is an associative array.
     * @see MM_PdfReceiptsExtension::getExtensionData for the data passed to the template.
     */
    public function displayConfigScreen(): void
    {
        $this->addExtensionManagementScript();
        echo MM_TEMPLATE::generate(
            MM_MODULES . "/extensions_landing.php",
            $this->getExtensionInformation()
        );
    }


    /**
     * Adds the extension management script.
     * @return bool Whether the script was successfully added.
     */
    protected function addExtensionManagementScript(): bool
    {
        $nonce = wp_create_nonce("mm_{$this->token}_landing_action");
        $script = $this->buildExtensionManagementScript($nonce);
        return wp_add_inline_script("common", $script, "after");
    }


    /**
     * Builds the extension management script.
     * @param string $nonce A WP nonce.
     * @return string The extension management script.
     */
    protected function buildExtensionManagementScript($nonce): string
    {
        $ajaxUrl = esc_url(admin_url("admin-ajax.php"));
        return <<<EOD
		jQuery(function() {
			var actionButton = document.querySelector(".mm-{$this->token}-action");
			actionButton.addEventListener("click", async function(event){
				event.preventDefault();
				actionButton.disabled = true;
				var operation = actionButton.dataset.action;
				var notice = document.getElementById("mm-action-notice");
				var noticeMessage = document.getElementById("mm-action-notice-message");
				var stepTitle = document.getElementById("mm-extension-step-title");
				fetch("{$ajaxUrl}", {
					method: "POST",
					headers:{
						"Content-Type": "application/x-www-form-urlencoded"
					},
					body: new URLSearchParams({
						"action": "mm_{$this->token}_landing_action",
						"nonce": "{$nonce}",
						"type": operation
					})
				}).then((response) => {
					if (response.status >= 400 && response.status < 600) {
						throw new Error(response.status + " " + response.statusText);
					}
					const data = response.json().then((result) => {
						if (result.success && result.data.redirectUrl.length > 0) {
								window.location.href = result.data.redirectUrl;
							} else {
								notice.classList.add("notice-" + result.data.messageClass);
								noticeMessage.innerHTML = result.data.message;
								notice.style.display = "block";
								if (result.success) {
									stepTitle.innerHTML = result.data.nextStepTitle;
									actionButton.innerHTML = result.data.nextButtonText;
									actionButton.dataset.action = result.data.nextButtonAction;
								}
								actionButton.disabled = false;
							}
					})
				}).catch((error) => {
					console.log("MemberMouse Extension Landing AJAX Request Failed - " + error);
				});
			});
		});
		EOD;
    }


    /**
     * AJAX handler for the landing page.
     */
    public function extensionLandingAjaxHandler(): void
    {
        if (empty($_POST["nonce"]) || !wp_verify_nonce($_POST["nonce"], "mm_{$this->token}_landing_action")) {
            wp_send_json_error(__("Sorry, you don't have permission to perform this action.", "membermouse"));
        }

        if (!current_user_can("activate_plugins")) {
            wp_send_json_error(__("Sorry, you don't have permission to perform this action.", "membermouse"));
        }

        $type = sanitize_text_field($_POST['type']);

        switch ($type) {
            case "install":
                $result = $this->installExtension();
                break;
            case "install-activate":
                $result = $this->installExtension(true);
                break;
            case "activate":
                $result = $this->activateExtension();
                break;
            case "configure":
                $result = new MM_Response("", MM_Response::$SUCCESS);
                $result->url = admin_url(static::$EXTENSION_ADMIN_PATH);
                break;
            default:
                wp_send_json_error("Invalid extension action type: {$type}");
                break;
        }

        $ajaxResponse = $this->generateAjaxResponse($type, $result);
        wp_send_json($ajaxResponse);
    }


    /**
     * Generates the AJAX response.
     * @param string $action The action being performed.
     * @param object $result The MM_Response object resulting from the action.
     * @return array
     */
    protected function generateAjaxResponse($action, $result): array
    {
        $success = MM_Response::isSuccess($result);
        $nextStepTitle = "";
        $nextButtonText = "";
        $nextButtonAction = "";

        if ($success) {
            switch ($action) {
                case "install":
                    $nextButtonText = __("Enable Add-on", "membermouse");
                    $nextButtonAction = "activate";
                    $nextStepTitle = sprintf(
                        __("%s is Installed", "membermouse"),
                        static::$EXTENSION_NAME
                    );
                    break;
                case "install-activate":
                case "activate":
                    $nextButtonText = __("Configure Add-on", "membermouse");
                    $nextButtonAction = "configure";
                    $nextStepTitle = sprintf(
                        __('%s is Installed and Active', 'membermouse'),
                        static::$EXTENSION_NAME
                    );
                    break;
            }
        }

        return [
            "success" => $success,
            "data" => [
                "redirectUrl" => $result->url,
                "messageClass" => $result->type,
                "message" => $result->message,
                "nextStepTitle" => $nextStepTitle,
                "nextButtonText" => $nextButtonText,
                "nextButtonAction" => $nextButtonAction,
            ]
        ];
    }


    /**
     * Installs the extension.
     * @param boolean $activate Whether to activate the extension after installation. Default false.
     * @return MM_Response Status of installation
     */
    public function installExtension($activate = false): object
    {
        $license = MM_License::getCurrentLicense();

        if (strpos($_SERVER["PHP_SELF"], "plugins.php") === false) {
            MM_MemberMouseService::validateLicense($license);
        }

        if ($license->isUnlicensed()) {
            return new MM_Response(
                __("Your license is not valid. Please contact MemberMouse support for assistance.", "membermouse"),
                MM_Response::$ERROR
            );
        }

        // TODO: Mothership transition may require some work here.

        static::$EXTENSION_DOWNLOAD_URL = $this->getExtensionDownloadUrl();

        if (empty(static::$EXTENSION_DOWNLOAD_URL)) {
            $message = sprintf(
                __("%s could not be retrieved. Please contact MemberMouse support for assistance.", "membermouse"),
                static::$EXTENSION_NAME
            );
            return new MM_Response($message, MM_Response::$ERROR);
        }

        if ($this->isInstalled()) {
            $message = sprintf(
                __(
                    "%s is already installed. Please visit the Plugins page to activate or remove it. If you need " .
                        "further assistance, please contact MemberMouse support.",
                    "membermouse"
                ),
                static::$EXTENSION_NAME
            );
            return new MM_Response($message, MM_Response::$ERROR);
        }

        wp_clean_plugins_cache();

        set_current_screen("membermouse_page_{" . static::$EXTENSION_MAIN_FILE . "}");

        // Get filesystem credentials
        $url = $this->getExtensionLandingPageUrl();
        $credentials = request_filesystem_credentials($url, '', false, WP_PLUGIN_DIR);

        if ($credentials === false || !WP_Filesystem($credentials)) {
            $message = __(
                "Credentials to access the filesystem are missing or invalid. Please contact your hosting provider " .
                    "for assistance.",
                "membermouse"
            );
            return new MM_Response($message, MM_Response::$ERROR);
        }

        require_once ABSPATH . "wp-admin/includes/class-wp-upgrader.php";

        // Prevent language pack upgrade (breaks response)
        remove_action("upgrader_process_complete", ["Language_Pack_Upgrader", "async_upgrade"], 20);

        $installer = new Plugin_Upgrader(new MM_AddonInstallSkin());
        $installResult = $installer->install(stripslashes(static::$EXTENSION_DOWNLOAD_URL));

        if ($installResult instanceof WP_Error) {
            $message = sprintf(
                __('%1$s could not be installed. Error: %2$s', "membermouse"),
                static::$EXTENSION_NAME,
                $installResult->get_error_message()
            );
            return new MM_Response($message, MM_Response::$ERROR);
        }

        wp_cache_flush();

        if ($activate) {
            $activationResponse = $this->activateExtension();

            if (MM_Response::isSuccess($activationResponse)) {
                $message = sprintf(
                    __("%s has been installed and activated successfully. Enjoy!", "membermouse"),
                    static::$EXTENSION_NAME
                );

                $response = new MM_Response($message, MM_Response::$SUCCESS);
                $response->url = $activationResponse->url;
                return $response;
            } else {
                return $activationResponse;
            }
        }

        $message = sprintf(
            __("%s has been installed successfully. Enjoy!", "membermouse"),
            static::$EXTENSION_NAME
        );

        return new MM_Response($message, MM_Response::$SUCCESS);
    }


    /**
     * Activates the extension.
     * @return MM_Response Status of activation
     */
    public function activateExtension(): object
    {
        $extensionDirectory = $this->getExtensionDirectory();

        if (empty($extensionDirectory)) {
            $message = sprintf(
                __("%s could not be activated. Please contact MemberMouse support for assistance.", "membermouse"),
                static::$EXTENSION_NAME
            );
            return new MM_Response($message, MM_Response::$ERROR);
        }

        $extensionSlug = $extensionDirectory . "/" . static::$EXTENSION_MAIN_FILE;
        $activated = activate_plugin($extensionSlug);

        if ($activated == NULL) {
            $message = sprintf(
                __("%s has been activated successfully. Enjoy!", "membermouse"),
                static::$EXTENSION_NAME
            );

            $response = new MM_Response($message, MM_Response::$SUCCESS);
            if (static::$SETTINGS_POST_INSTALL_REDIRECT) {
                $response->url = static::$EXTENSION_ADMIN_PATH;
            }
        } else if ($activated instanceof WP_Error) {
            $message = printf(
                __(
                    '%1$s could not be activated. Please contact MemberMouse support for assistance. Error: %2$s',
                    'membermouse'
                ),
                static::$EXTENSION_NAME,
                $activated->get_error_message()
            );

            $response = new MM_Response($message, MM_Response::$ERROR);
        } else {
            $response = new MM_Response("Unexpected response during extension activation.", MM_Response::$ERROR);
        }

        return $response;
    }

    /**
     * Gets the directory where the extension is installed.
     * @return string Directory where the extension is installed, or empty string if not installed.
     */
    protected function getExtensionDirectory(): string
    {
        $plugins = get_plugins();
        $directory = "";

        foreach ($plugins as $pluginSlug => $pluginData) {
            if (strpos($pluginSlug, static::$EXTENSION_MAIN_FILE) !== false) {
                $pathComponents = explode("/", $pluginSlug);
                $directory = reset($pathComponents);
                break;
            }
        }

        return $directory;
    }


    /**
     * Gets the URL of the plugin package, in the future, from Mothership.
     * @return string Extension download URL.
     */
    protected function getExtensionDownloadUrl(): string
    {
        return "";
    }


    /**
     * Get the URL of the extension landing page.
     * @return string Extension landing page URL.
     */
    public function getExtensionLandingPageUrl(): string
    {
        return esc_url_raw(
            add_query_arg(
                [MM_Session::$PARAM_SUBMODULE => $this->token],
                MM_ModuleUtils::getUrl(MM_ModuleUtils::getPage(), MM_MODULE_EXTENSIONS)
            )
        );
    }
}
