<?php
class MM_AbstractForm
{
    public static $TYPE_LOGIN = "login";
    public static $TYPE_FORGOT_PASSWORD = "forgotpassword";
    public static $TYPE_RESET_PASSWORD = "resetpassword";
    public static $TYPE_CHECKOUT = "checkout";
    public static $TYPE_MYACCOUNT = "myaccount";
    public static $TYPE_1CLICK_PURCHASE = "1clickpurchase";
    public static $TYPE_CUSTOM = "custom";


    protected $isValid = false;
    public $error = "";


    // abstract functions
    public function initialize() {}
    public function generateFormHeader() {}
    public function generateFormFooter() {}
    public function mm_form_section($content, $atts, $code) {}
    public function mm_form_field($content, $atts, $code) {}
    public function mm_form_data($content, $atts, $code) {}
    public function mm_form_message($content, $atts, $code) {}
    public function mm_form_button($content, $atts, $code) {}


    /**
     * This factory function returns the appropriate instance of the MM_AbstractForm class based on 
     * the type passed.
     * @param String $type Indicates the type of form (i.e. checkout, login, customfield, etc)
     * @param Array $params 
     * @return object Instance of MM_AbstractForm class (ie MM_CheckoutForm, MM_LoginForm) based on the type parameter
     */
    public static function getForm($type, $params)
    {
        switch (strtolower($type)) {
            case self::$TYPE_CHECKOUT:
                return new MM_CheckoutForm($params);
                break;

            case self::$TYPE_MYACCOUNT:
                return new MM_MyAccountForm($params);
                break;

            case self::$TYPE_LOGIN:
                return new MM_LoginForm($params);
                break;

            case self::$TYPE_FORGOT_PASSWORD:
                return new MM_ForgotPasswordForm($params);
                break;

            case self::$TYPE_RESET_PASSWORD:
                return new MM_ResetPasswordForm($params);
                break;

            case self::$TYPE_1CLICK_PURCHASE:
                return new MM_1ClickPurchaseForm($params);
                break;

            case self::$TYPE_CUSTOM:
                return new MM_CustomForm($params);
                break;

            default:
                return new MM_AbstractForm($params);
                break;
        }
    }

    /**
     * Default constructor for all forms.
     * @param Array $params an associative array of form configuration parameters
     */
    public function __construct($params)
    {
        $this->isValid = false;
        $this->initialize();
    }

    public function isValid()
    {
        return $this->isValid;
    }
}
