<?php

/**
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */

use MemberMouse\GroundLevel\Container\Container;
use MemberMouse\GroundLevel\InProductNotifications\Service as IPNService;
use MemberMouse\GroundLevel\Mothership\Service as MoshService;

class MM_GroundLevelControl
{
    /**
     * Bootstrap Ground Level
     */
    public static function init()
    {
        $container = new Container(); // @TODO: Bootstrap properly via a PluginConfig and GrdLvl Bootstrap.

        // Instantiate the Mothership Service.
        $container->addService(
            MoshService::class,
            static function () use ($container): MoshService {
                return new MoshService(
                    $container,
                    new MM_MothershipConnector()
                );
            },
            true
        );

        // Get the product slug.
        $license     = MM_License::getCurrentLicense();
        $productSlug = $license->getLicenseType('', 'slug');

        // Set IPN Service parameters.
        $container->addParameter(IPNService::PRODUCT_SLUG, $productSlug);
        $container->addParameter(IPNService::PREFIX, 'mm');
        $container->addParameter(
            IPNService::THEME,
            [
                'primaryColor'        => '#066cd2',
                'inboxBtnBorderWidth' => '2px',
                'inboxBtnSize'        => '23px',
            ]
        );
        $container->addParameter(
            IPNService::USER_CAPABILITY,
            'remove_users'
        );
        $container->addParameter(
            IPNService::RENDER_HOOK,
            'mm_dashboard_header_actions'
        );

        $container->addService(
            IPNService::class,
            static function () use ($container): IPNService {
                return new IPNService($container);
            },
            true
        );

        // Add the filter to wrap the IPN button in a list element.
        add_filter('mm_ipn_root_element_html', function (string $html): string {
            return '<li id="mm-dashboard-ipns">' . $html . '</li>';
        });
    }
}
