<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * CustomerAddress is an abstraction of an address associated with a customer, ie a billing or shipping address
 */

class MM_CustomerAddress
{
    /**
     * The types of states that can be returned.
     *
     * @var array
     */
    public const ADDRESS_TYPES = ['billing', 'shipping'];

    private $firstName = "";
    private $lastName = "";
    private $address1 = "";
    private $address2 = "";
    private $city = "";
    private $state = "";
    private $postalCode = "";
    private $province = "";
    private $country = "";
    private $phone = "";


    /**
     * Constructor. Allows population of the address object on creation
     */
    public function __construct($firstName = "", $lastName = "", $address1 = "", $address2 = "", $city = "", $state = "", $postalCode = "", $province = "", $country = "", $phone = "")
    {
        $fields = array("firstName", "lastName", "address1", "address2", "city", "state", "postalCode", "province", "country", "phone");
        foreach ($fields as $field) {
            $this->$field = $$field;
        }
    }

    /**
     * Retrieve the first name associated with this address
     */
    public function getFirstName()
    {
        return $this->firstName;
    }

    /**
     * Set the first name associated with this address
     * 
     * @param String $newFirstName The new first name
     */
    public function setFirstName($newFirstName)
    {
        $this->firstName = $newFirstName;
    }

    /**
     * Retrieve the last name associated with this address
     */
    public function getLastName()
    {
        return $this->lastName;
    }

    /**
     * Set the last name associated with this address
     *
     * @param String $newLastName The new last name
     */
    public function setLastName($newLastName)
    {
        $this->lastName = $newLastName;
    }

    /**
     * Retrieve the address line1
     */
    public function getAddressLine1()
    {
        return $this->address1;
    }

    /**
     * Sets the address line1
     */
    public function setAddressLine1($newAddress1)
    {
        $this->address1 = $newAddress1;
    }

    /**
     * Retrieve the address line2
     */
    public function getAddressLine2()
    {
        return $this->address2;
    }

    /**
     * Sets the address line2
     */
    public function setAddressLine2($newAddress2)
    {
        $this->address2 = $newAddress2;
    }

    /**
     * Retrieve the address city
     */
    public function getCity()
    {
        return $this->city;
    }

    /**
     * Sets the address city
     */
    public function setCity($newCity)
    {
        $this->city = $newCity;
    }


    /**
     * Retrieve the state associated with this address
     */
    public function getState()
    {
        return $this->state;
    }

    /**
     * Set the state associated with this address
     *
     * @param String $newState The new state
     */
    public function setState($newState)
    {
        $this->state = $newState;
    }

    /**
     * Retrieve the postal code associated with this address
     */
    public function getPostalCode()
    {
        return $this->postalCode;
    }

    /**
     * Set the postal code associated with this address
     *
     * @param String $newPostalCode The new postal code
     */
    public function setPostalCode($newPostalCode)
    {
        $this->postalCode = $newPostalCode;
    }

    /**
     * Retrieve the address province
     */
    public function getProvince()
    {
        return $this->province;
    }

    /**
     * Sets the address province
     */
    public function setProvince($newProvince)
    {
        $this->province = $newProvince;
    }

    /**
     * Retrieve the address country
     */
    public function getCountry()
    {
        return $this->country;
    }

    /**
     * Sets the address country
     */
    public function setCountry($newCountry)
    {
        $this->country = $newCountry;
    }

    /**
     * Retrieve the address phone
     */
    public function getPhone()
    {
        return $this->phone;
    }

    /**
     * Sets the address phone
     */
    public function setPhone($newPhone)
    {
        $this->phone = $newPhone;
    }


    public function setBillingAddress($data)
    {
        $this->firstName = $data->billing_first_name;
        $this->lastName = $data->billing_last_name;
        $this->phone = $data->billing_phone;
        $this->address1 = $data->billing_address1;
        $this->address2 = $data->billing_address2;
        $this->city = $data->billing_city;
        $this->province = $data->billing_province;
        $this->state = $data->billing_state;
        $this->postalCode = $data->billing_postal_code;
        $this->country = $data->billing_country;
    }


    public function setShippingAddress($data)
    {
        $this->firstName = $data->shipping_first_name;
        $this->lastName = $data->shipping_last_name;
        $this->phone = $data->shipping_phone;
        $this->address1 = $data->shipping_address1;
        $this->address2 = $data->shipping_address2;
        $this->city = $data->shipping_city;
        $this->province = $data->shipping_province;
        $this->state = $data->shipping_state;
        $this->postalCode = $data->shipping_postal_code;
        $this->country = $data->shipping_country;
    }
}
