<?php

/**
 *
 *
 MemberMouse(TM) (http://www.membermouse.com)
 (c) MemberMouse, LLC. All rights reserved.
 */

/**
 * MM_PaymentServiceResponse is a container meant to abstract a response from a specific payment service/gateway into a status/message
 * that MemberMouse understands on a higher level.
 */

class MM_PaymentServiceResponse
{
    public static $SUCCESS = 1;
    public static $FAILED = 2;
    public static $PENDING = 3;
    public static $ERROR = 4;

    public $status;
    public $message;
    public $url = "";
    public $data = [];

    public function __construct($message = "", $status = 1)
    {
        $this->message = $message;
        $this->status = $status;
    }

    public function getStatus()
    {
        return $this->status;
    }

    public function setStatus($status)
    {
        $this->status = $status;
    }

    /** 
     * Returns the message (contents) of the response object
     * 
     * @return mixed The message (contents) of the response object
     * */
    public function getMessage()
    {
        return $this->message;
    }

    public function setMessage($message)
    {
        $this->message = $message;
    }

    public static function isError($response)
    {
        if (($response instanceof MM_PaymentServiceResponse) && ($response->getStatus() == MM_PaymentServiceResponse::$ERROR)) {
            return true;
        }

        return false;
    }

    public static function isSuccess($response)
    {
        if (($response instanceof MM_PaymentServiceResponse) && ($response->getStatus() == MM_PaymentServiceResponse::$SUCCESS)) {
            return true;
        }

        return false;
    }

    public static function isFailed($response)
    {
        if (($response instanceof MM_PaymentServiceResponse) && ($response->getStatus() == MM_PaymentServiceResponse::$FAILED)) {
            return true;
        }

        return false;
    }


    public static function translate(MM_Response $mmResponse)
    {
        $paymentServiceResponse = new MM_PaymentServiceResponse($mmResponse->message, ($mmResponse->type == MM_Response::$SUCCESS) ? MM_PaymentServiceResponse::$SUCCESS : MM_PaymentServiceResponse::$ERROR);
        if (isset($mmResponse->url)) {
            $paymentServiceResponse->url = $mmResponse->url;
        }
        return $paymentServiceResponse;
    }
}
