<?php

/**
 *
 *
 MemberMouse(TM) (http://www.membermouse.com)
 (c) MemberMouse. All rights reserved.
 */

/**
 * MM_ShoppingCartItem is an abstraction of an item in a shopping cart. There are two types of purchases MemberMouse plans to support;
 * memberships, and e-commerce product purchases. While the MM_MembershipLevel and MM_Product types could be added directly to the cart, 
 * each would need to be treated differently, and this is further complicated by the fact that a membership purchase is currently the
 * result of a product purchase. In the future hopefully the underlying object model will be revised, either creating an inheritance 
 * relationship between membership and product, or completely separating these concepts. This class provides an abstraction over the 
 * purchase type, hopefully insulating the rest of the code from these future changes
 */

class MM_ShoppingCartItem
{
    public static $CART_PRODUCT = 1;
    public static $CART_COUPON = 2;

    private $display_name;
    private $item;
    private $item_type;
    private $quantity = 1;

    public function __construct($display_name, $item, $item_type, $quantity = 1)
    {
        $this->display_name = $display_name;
        $this->item = $item;
        if ((intval($item_type) < 1) || (intval($item_type) > 2)) {
            throw Exception("Cart item type not supported");
        }
        $this->item_type = $item_type;
    }

    public function getDisplayName()
    {
        return $this->display_name;
    }

    public function setDisplayName($display_name)
    {
        $this->display_name = $display_name;
    }

    public function getItem()
    {
        return $this->item;
    }

    public function setItem($item)
    {
        $this->item = $item;
    }

    public function getItemType()
    {
        return $this->item_type;
    }

    public function setItemType($item_type)
    {
        $this->item_type = $item_type;
    }

    public function setQuantity($quantity)
    {
        $this->quantity = $quantity;
    }

    public function getQuantity()
    {
        return $this->quantity;
    }
}
