<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) Membermouse, LLC. All rights reserved.
 *
 * MM_ShippingOption is a value object meant to standardize the way shipping option data is presented across shipping methods
 *
 */

class MM_ShippingOption
{
    //unique identifier that uniquely identifies this option and the method it is associated with. Keys have the form "<TOKEN>-<IDENTIFIER>"
    private $key;

    //the display name
    private $name;

    //the cost of using this rate, generally used in the context of order placement
    private $rate;

    //optional, the id used by this option in its backing table (if there is one). This simplifies some of the interface code
    private $id;

    //the shipping method this option is part of
    private $methodName;

    //the currency this rate is measured in
    private $currency;


    public function __construct($key = "", $name = "", $rate = "", $id = "", $methodName = "", $currency = "")
    {
        $params = array("key", "name", "rate", "id", "methodName");
        foreach ($params as $param) {
            if ($$param !== "") {
                $this->$param = $$param;
            }
        }

        $this->currency = !empty($currency) ? strtoupper($currency) : MM_CurrencyUtil::getActiveCurrency();
    }


    public function getKey()
    {
        return $this->key;
    }


    public function setKey($key)
    {
        $this->key = $key;
    }


    public function getName()
    {
        return $this->name;
    }


    public function setName($name)
    {
        $this->name = $name;
    }


    public function getRate($doFormat = false)
    {
        if ($doFormat) {
            return _mmf($this->rate, $this->currency);
        } else {
            return number_format(floatval($this->rate), MM_CurrencyUtil::getPrecisionForCurrency($this->currency), '.', '');
        }
    }


    public function setRate($rate)
    {
        $this->rate = $rate;
    }


    public function getId()
    {
        return $this->id;
    }


    public function setId($id)
    {
        $this->id = $id;
    }

    public function getMethodName()
    {
        return $this->methodName;
    }


    public function setMethodName($methodName)
    {
        $this->methodName = $methodName;
    }
}
