<?php
class MM_CustomFieldTagProcessor
{
    public static $TAG_CUSTOM_FIELD_DATA = "mm_customfield_data";


    /**
     * Takes a SmartTag string and returns true if the SmartTag is related to custom fields.
     * 
     * @param String $smartTag the name of the SmartTag
     * @return Boolean returns true if the SmartTag passed is related to custom fields.
     */
    public static function isCustomFieldSmartTag($smartTag)
    {
        switch ($smartTag) {
            case self::$TAG_CUSTOM_FIELD_DATA:
                return true;

            default:
                return false;
        }
    }


    /**
     * Takes a SmartTag name and a number of properties related to the SmartTag and makes a call to 
     * the appropriate function based on the SmartTag name.
     * @param String $smartTag the name of the SmartTag
     * @param String $content the content to process
     * @param Array $atts an associative array of attributes associated with the SmartTag
     * @param String $code the raw version of the SmartTag name (i.e. not case-insentive, not broken up into parts)
     * @return String a string based on the $content passed with all SmartTags processed
     */
    public static function processSmartTag($smartTag, $content, $atts, $code)
    {
        switch ($smartTag) {
            case self::$TAG_CUSTOM_FIELD_DATA:
                return self::mm_customfield_data($content, $atts, $code);

            default:
                return "Invalid Custom Field SmartTag {$smartTag} found";
        }
    }


    /**
     * This function processes the MM_CustomField_Data tag. It is responsible for 
     * returning data associated with the custom field ID passed.
     * @param String $content the content contained within the SmartTag (if any)
     * @param Array $atts an associative array of attributes associated with the SmartTag
     * @param String $code the raw version of the SmartTag name (i.e. not case-insentive, not broken up into parts)
     * @return String a string based on the $content passed with all SmartTags processed
     */
    public static function mm_customfield_data($content, $atts, $code)
    {
        if (!empty($atts)) {
            foreach ($atts as $k => $v) {
                $$k = $v;
            }
        }

        if (isset($id) && isset($name)) {
            $customField = new MM_CustomField($id);

            if ($customField->isValid()) {
                switch (strtolower($name)) {
                    case "displayname":
                        return $customField->getDisplayName();

                    default:
                        return "Invalid SmartTag Configuration: Invalid name '{$name}' passed to SmartTag {$code}";
                }
            } else {
                return "Invalid SmartTag Configuration: Invalid custom field ID of '{$id}' passed to SmartTag {$code}";
            }
        } else {
            return "Invalid SmartTag Configuration: 'id' and 'name' attributes are required for SmartTag {$code}";
        }
    }
}
