<?php
class MM_EmployeeTagProcessor
{
    public static $TAG_EMPLOYEE_DATA = "mm_employee_data";


    /**
     * Takes a SmartTag string and returns true if the SmartTag is related to employees.
     * 
     * @param String $smartTag the name of the SmartTag
     * @return Boolean returns true if the SmartTag passed is related to employees.
     */
    public static function isEmployeeSmartTag($smartTag)
    {
        switch ($smartTag) {
            case self::$TAG_EMPLOYEE_DATA:
                return true;

            default:
                return false;
        }
    }


    /**
     * Takes a SmartTag name and a number of properties related to the SmartTag and makes a call to 
     * the appropriate function based on the SmartTag name.
     * @param String $smartTag the name of the SmartTag
     * @param String $content the content to process
     * @param Array $atts an associative array of attributes associated with the SmartTag
     * @param String $code the raw version of the SmartTag name (i.e. not case-insentive, not broken up into parts)
     * @return String a string based on the $content passed with all SmartTags processed
     */
    public static function processSmartTag($smartTag, $content, $atts, $code)
    {
        switch ($smartTag) {
            case self::$TAG_EMPLOYEE_DATA:
                return self::mm_employee_data($content, $atts, $code);

            default:
                return "Invalid Employee SmartTag {$smartTag} found.";
        }
    }


    /**
     * This function gets the employee associated with the ID passed in the attributes or the default employee
     * 
     * @param Array $atts an associative array of attributes
     * @return MM_Employee the employee associated with the ID passed in the attributes or the default employee
     */
    private static function getEmployee($atts)
    {
        if (is_array($atts) && count($atts) > 0) {
            foreach ($atts as $k => $v) {
                $key = strtolower($k);
                $$key = $v;
            }
        }

        $employee = new MM_Employee();

        // get employee
        if (isset($id)) {
            $employee = new MM_Employee($id);
        } else {
            $employee = MM_Employee::getDefault();
        }

        return $employee;
    }


    /**
     * This function processes the MM_Employee_Data tag. It is responsible for 
     * returning the employee data based on the name attribute.
     * @param String $content the content contained within the SmartTag (if any)
     * @param Array $atts an associative array of attributes associated with the SmartTag
     * @param String $code the raw version of the SmartTag name (i.e. not case-insentive, not broken up into parts)
     * @return String a string based on the $content passed with all SmartTags processed
     */
    public static function mm_employee_data($content, $atts, $code)
    {
        $employee = self::getEmployee($atts);

        if ($employee->isValid()) {
            if (is_array($atts) && count($atts) > 0) {
                foreach ($atts as $k => $v) {
                    $key = strtolower($k);
                    $$key = $v;
                }
            }

            if (isset($name)) {
                switch (strtolower($name)) {
                    case "displayname":
                        return $employee->getDisplayName();

                    case "firstname":
                        return $employee->getFirstName();

                    case "lastname":
                        return $employee->getLastName();

                    case "email":
                        return $employee->getEmail();

                    case "phone":
                        return $employee->getPhone();

                    default:
                        return "Invalid SmartTag Configuration: The 'name' attribute '{$name}' is invalid for SmartTag {$code}.";
                }
            } else {
                return "Invalid SmartTag Configuration: The 'name' attribute is required for SmartTag {$code}.";
            }
        } else {
            return "Invalid SmartTag Configuration: Could not find employee for SmartTag {$code}.";
        }
    }
}
