<?php
class MM_ErrorTagProcessor
{
    public static $TAG_ERROR_MESSAGE = "mm_error_message";


    /**
     * Takes a SmartTag string and returns true if the SmartTag is related to errors.
     * 
     * @param String $smartTag the name of the SmartTag
     * @return Boolean returns true if the SmartTag passed is related to errors.
     */
    public static function isErrorSmartTag($smartTag)
    {
        switch ($smartTag) {
            case self::$TAG_ERROR_MESSAGE:
                return true;

            default:
                return false;
        }
    }


    /**
     * Takes a SmartTag name and a number of properties related to the SmartTag and makes a call to 
     * the appropriate function based on the SmartTag name.
     * @param String $smartTag the name of the SmartTag
     * @param String $content the content to process
     * @param Array $atts an associative array of attributes associated with the SmartTag
     * @param String $code the raw version of the SmartTag name (i.e. not case-insentive, not broken up into parts)
     * @return String a string based on the $content passed with all SmartTags processed
     */
    public static function processSmartTag($smartTag, $content, $atts, $code)
    {
        switch ($smartTag) {
            case self::$TAG_ERROR_MESSAGE:
                return self::mm_error_message($content, $atts, $code);

            default:
                return "Invalid Error SmartTag {$smartTag} found.";
        }
    }


    /**
     * This function processes the MM_Error_Message tag. It is responsible for 
     * returning the error message based on the error code or message passed in the URL.
     * @param String $content the content contained within the SmartTag (if any)
     * @param Array $atts an associative array of attributes associated with the SmartTag
     * @param String $code the raw version of the SmartTag name (i.e. not case-insentive, not broken up into parts)
     * @return String a string based on the $content passed with all SmartTags processed
     */
    public static function mm_error_message($content, $atts, $code)
    {
        $message = MM_Error::getMessage();
        return $message;
    }
}
