<?php
class MM_TagProcessor
{
    private static $isProcessing = false;
    private static $storedShortcodes = array();

    public static function processSmartTags($content)
    {
        global $shortcode_tags;

        if (strpos($content, '[') === false) {
            return $content;
        }

        $smartTagRegex = MM_SmartTagUtil::getSmartTagsRegex();
        if (empty($smartTagRegex)) {
            return $content;
        }

        $pattern = "/\\[($smartTagRegex)/s";

        if (preg_match($pattern, $content) !== 1) {
            //we want to scope our processing to content containing MemberMouse SmartTags
            return $content;
        }

        $shouldRestoreShortcodes = false;
        if (!self::$isProcessing) {
            //Temporarily remove all non-SmartTag shortcodes from the global array. This allows us to recursively call the 
            //wordpress shortcode processing functions and only process MemberMouse SmartTags
            self::$storedShortcodes = $shortcode_tags;
            $smartTagNamePattern = "/({$smartTagRegex})/s";

            $validKeys = array_filter(array_keys($shortcode_tags), function ($k) use ($smartTagNamePattern) {
                return (preg_match($smartTagNamePattern, $k) === 1);
            });

            $shortcode_tags = array_intersect_key($shortcode_tags, array_flip($validKeys));
            self::$isProcessing = true;
            $shouldRestoreShortcodes = true;
        }


        //Process shortcodes in the content block
        $pattern = get_shortcode_regex();
        $content = preg_replace_callback("/{$pattern}/s", 'do_shortcode_tag', $content);

        if ($shouldRestoreShortcodes) {
            $shortcode_tags = self::$storedShortcodes;
            self::$storedShortcodes = array();
            self::$isProcessing = false;
        }

        return $content;
    }
}
