<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_CountrySubdivisionUtils
{
    /**
     * Returns all subdivisions (states, provinces, etc) within specified countries
     * @param  array $iso An array of ISO country codes. Default empty array, which returns all codes
     * @return array A two-dimensional array in the format [{iso}][{subdivCode}] = name  
     */
    public static function getCountrySubdivsionList($iso = array())
    {
        global $wpdb;

        $iso = esc_sql((array)$iso);

        foreach ($iso as $key => $val) {
            if (!is_string($val)) {
                unset($iso[$key]);
            }
        }

        $sql = "SELECT iso, code, name FROM " . MM_TABLE_COUNTRY_SUBDIVISIONS . " ";

        if (count($iso) > 0) {
            $sql .= "WHERE iso IN ('" . implode("', '", $iso) . "') ";
        }

        $sql .= "ORDER BY name ASC";

        $results = $wpdb->get_results($sql, OBJECT);
        $list = array();

        foreach ($results as $iso => $row) {
            $list[$row->iso][$row->code] = $row->name;
        }

        return $list;
    }


    /**
     * Returns a subdivision name.
     * @param  string $iso  An ISO 3166-1 alpha-2 Country Code
     * @param  string $code An ISO 3166-2 alpha-2 Subdivision Code
     * @return string The subdivision name, or a locale code if no matching subdivision is found
     */
    public static function getCountrySubdivsionName($iso, $code)
    {
        global $wpdb;

        $sql = "SELECT name FROM " . MM_TABLE_COUNTRY_SUBDIVISIONS . " WHERE iso = %s AND code = %s LIMIT 1";
        $row = $wpdb->get_row($wpdb->prepare($sql, $iso, $code));

        if (isset($row->name)) {
            return $row->name;
        } else {
            return "{$iso}-{$code}";
        }
    }


    /**
     * This function takes country code and a state code and returns the text version of the state name
     * @param  string $country	Country Code
     * @param  string $state	State Code
     * @return string The state name, or empty string if not found
     */
    public static function getStateName($country, $state)
    {
        $states = MM_CountrySubdivisionUtils::getCountrySubdivsionList(array($country));
        if (isset($states[$country]) && !is_null($states[$country])) {
            if (isset($states[$country][$state]) && !is_null($states[$country][$state])) {
                return $states[$country][$state];
            }
        }
        return "";
    }
}
