<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * CurrencyUtil provides methods to enable proper display of international currencies.
 *
 * The format function was adapted from the public domain money_format in the comments located at http://php.net/manual/en/function.money-format.php 
 */

class MM_CurrencyUtil
{

    protected static $currencies = null;

    public static function __initializeCurrencyFormats()
    {
        // Currency Format Adjustments
        // CAD, PLN, NOK, QAR (5/6/2014)

        //world's 10 most traded currencies (as of Sept. 2013)
        self::$currencies['USD'] = array('name' => 'United States Dollar',  'int_curr_symbol' => '&#85;&#83;&#68;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['CAD'] = array('name' => 'Canadian Dollar',  'int_curr_symbol' => '&#67;&#65;&#68;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['EUR'] = array('name' => 'Euro',  'int_curr_symbol' => '&#45;&#55;&#52;&#20;',  'currency_symbol' => '&euro;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['JPY'] = array('name' => 'Japanese Yen',  'int_curr_symbol' => '&#74;&#80;&#89;&#32;',  'currency_symbol' => '&#65509;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '0',  'frac_digits' => '0',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '4',  'n_sign_posn' => '4',  'int_p_sep_by_space' => '2',  'int_n_sep_by_space' => '2',);
        self::$currencies['GBP'] = array('name' => 'United Kingdom Pound',  'int_curr_symbol' => '&#71;&#66;&#80;&#32;',  'currency_symbol' => '&pound;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['CHF'] = array('name' => 'Swiss Franc',  'int_curr_symbol' => '&#67;&#72;&#70;&#32;',  'currency_symbol' => 'CHF',  'mon_decimal_point' => '.',  'mon_thousands_sep' => '&#39;',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '4',  'n_sign_posn' => '4',);
        self::$currencies['AUD'] = array('name' => 'Australian Dollar',  'int_curr_symbol' => '&#65;&#85;&#68;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['SEK'] = array('name' => 'Swedish Krona',  'int_curr_symbol' => '&#83;&#69;&#75;&#32;',  'currency_symbol' => '&#107;&#114;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['HKD'] = array('name' => 'Hong Kong Dollar',  'int_curr_symbol' => '&#72;&#75;&#68;&#32;',  'currency_symbol' => '&#72;&#75;&#36;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '0',);
        self::$currencies['NOK'] = array('name' => 'Norwegian Krone',  'int_curr_symbol' => '&#83;&#69;&#75;&#32;',  'currency_symbol' => '&#107;&#114;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '4',  'n_sign_posn' => '4',);

        //other currencies
        self::$currencies['AED'] = array('name' => 'Emirati Dirham',  'int_curr_symbol' => '&#65;&#69;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1573;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['ALL'] = array('name' => 'Albanian Lek',  'int_curr_symbol' => '&#65;&#76;&#76;&#32;',  'currency_symbol' => '&#76;&#101;&#107;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['AMD'] = array('name' => 'Armenian Dram',  'int_curr_symbol' => '&#65;&#77;&#68;&#32;',  'currency_symbol' => '&#1332;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['ARS'] = array('name' => 'Argentine Peso',  'int_curr_symbol' => '&#65;&#82;&#83;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['AZN'] = array('name' => 'Azerbaijani New Manat',  'int_curr_symbol' => '&#65;&#90;&#77;&#32;',  'currency_symbol' => '&#109;&#97;&#110;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BAM'] = array('name' => 'Bosnian Convertible Marka',  'int_curr_symbol' => '&#66;&#65;&#77;&#32;',  'currency_symbol' => '&#75;&#77;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BDT'] = array('name' => 'Bangladeshi Taka',  'currency_symbol' => '&#2547;',  'int_curr_symbol' => '&#66;&#68;&#84;&#32;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;2',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BGN'] = array('name' => 'Bulgarian Lev',  'int_curr_symbol' => '&#66;&#71;&#78;&#32;',  'currency_symbol' => '&#1083;&#1074;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '&#160;',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BHD'] = array('name' => 'Bahraini Dinar',  'int_curr_symbol' => '&#66;&#72;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1576;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['BOB'] = array('name' => 'Bolivian Boliviano',  'int_curr_symbol' => '&#66;&#79;&#66;&#32;',  'currency_symbol' => '&#36;&#98;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BRL'] = array('name' => 'Brazilian Real',  'int_curr_symbol' => '&#66;&#82;&#76;&#32;',  'currency_symbol' => '&#82;&#36;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BWP'] = array('name' => 'Botswana Pula',  'int_curr_symbol' => '&#66;&#87;&#80;&#32;',  'currency_symbol' => '&#80;&#117;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['BYR'] = array('name' => 'Belarusian Ruble',  'int_curr_symbol' => '&#66;&#89;&#82;&#32;',  'currency_symbol' => '&#1088;&#1091;&#1073;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['CLP'] = array('name' => 'Chilean Peso',  'int_curr_symbol' => '&#67;&#76;&#80;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['CNY'] = array('name' => 'Chinese Yuan Renminbi',  'int_curr_symbol' => '&#67;&#78;&#89;&#32;',  'currency_symbol' => '&#65509;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'int_p_cs_precedes' => '1',  'int_p_sep_by_space' => '0',  'int_n_cs_precedes' => '1',  'int_n_sep_by_space' => '0',  'p_sign_posn' => '4',  'n_sign_posn' => '4',  'int_p_sign_posn' => '1',  'int_n_sign_posn' => '1',);
        self::$currencies['COP'] = array('name' => 'Colombian Peso',  'int_curr_symbol' => '&#67;&#79;&#80;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['CRC'] = array('name' => 'Costa Rican Colon',  'int_curr_symbol' => '&#67;&#82;&#67;&#32;',  'currency_symbol' => '&#67;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['CZK'] = array('name' => 'Czech Koruna',  'int_curr_symbol' => '&#67;&#90;&#75;&#32;',  'currency_symbol' => '&#75;&#269;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '&#160;',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['DJF'] = array('name' => 'Djiboutian Franc',  'int_curr_symbol' => '&#68;&#74;&#70;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['DKK'] = array('name' => 'Danish Krone',  'int_curr_symbol' => '&#68;&#75;&#75;&#32;',  'currency_symbol' => '&#107;&#114;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '2',  'n_cs_precedes' => '1',  'n_sep_by_space' => '2',  'p_sign_posn' => '4',  'n_sign_posn' => '4',);
        self::$currencies['DOP'] = array('name' => 'Dominican Peso',  'int_curr_symbol' => '&#68;&#79;&#80;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['DZD'] = array('name' => 'Algerian Dinar',  'int_curr_symbol' => '&#68;&#90;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1580;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['EGP'] = array('name' => 'Egyptian Pound',  'int_curr_symbol' => '&#69;&#71;&#80;&#32;',  'currency_symbol' => '&#1580;&#46;&#1605;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['ERN'] = array('name' => 'Eritrean Nakfa',  'int_curr_symbol' => '&#69;&#82;&#78;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '0',  'frac_digits' => '0',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['ETB'] = array('name' => 'Ethiopian Birr',  'int_curr_symbol' => '&#69;&#84;&#66;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['GEL'] = array('name' => 'Georgian Lari',  'int_curr_symbol' => '&#71;&#69;&#76;&#32;',  'currency_symbol' => '&#4314;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['GTQ'] = array('name' => 'Guatemalan Quetzal',  'int_curr_symbol' => '&#71;&#84;&#81;&#32;',  'currency_symbol' => '&#81;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['HNL'] = array('name' => 'Honduran Lempira',  'int_curr_symbol' => '&#72;&#78;&#76;&#32;',  'currency_symbol' => '&#76;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['HRK'] = array('name' => 'Croatian Kuna',  'int_curr_symbol' => '&#72;&#82;&#75;&#32;',  'currency_symbol' => '&#75;&#110;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['HUF'] = array('name' => 'Hungarian Forint',  'int_curr_symbol' => '&#72;&#85;&#70;&#32;',  'currency_symbol' => '&#70;&#116;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'int_p_cs_precedes' => '1',  'int_p_sep_by_space' => '1',  'int_n_cs_precedes' => '1',  'int_n_sep_by_space' => '1',  'int_p_sign_posn' => '1',  'int_n_sign_posn' => '1',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['IDR'] = array('name' => 'Indonesian Rupiah',  'int_curr_symbol' => '&#73;&#68;&#82;&#32;',  'currency_symbol' => '&#82;&#112;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['ILS'] = array('name' => 'Israeli Shekel',  'int_curr_symbol' => '&#73;&#76;&#83;&#32;',  'currency_symbol' => '&#8362;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '2',  'n_sign_posn' => '2',);
        self::$currencies['INR'] = array('name' => 'Indian Rupee',  'int_curr_symbol' => '&#73;&#78;&#82;&#32;',  'currency_symbol' => '&#8377;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;2',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['IQD'] = array('name' => 'Iraqi Dinar',  'int_curr_symbol' => '&#73;&#81;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1593;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['IRR'] = array('name' => 'Iranian Rial',  'int_curr_symbol' => '&#73;&#82;&#82;&#32;',  'currency_symbol' => '&#1585;&#1740;&#1575;&#1604;',  'mon_decimal_point' => '&#1643;',  'mon_thousands_sep' => '&#1644;',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '0',  'frac_digits' => '0',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',  'int_p_cs_precedes' => '0',  'int_p_sep_by_space' => '1',  'int_n_cs_precedes' => '0',  'int_n_sep_by_space' => '1',  'int_p_sign_posn' => '1',  'int_n_sign_posn' => '1',);
        self::$currencies['ISK'] = array('name' => 'Icelandic Krona',  'int_curr_symbol' => '&#73;&#83;&#75;&#32;',  'currency_symbol' => '&#107;&#114;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '0',  'frac_digits' => '0',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['JOD'] = array('name' => 'Jordanian Dinar',  'int_curr_symbol' => '&#74;&#79;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1571;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['KES'] = array('name' => 'Kenyan Shilling',  'int_curr_symbol' => '&#75;&#69;&#83;&#32;',  'currency_symbol' => '&#75;&#115;&#104;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['KGS'] = array('name' => 'Kyrgyzstani Som',  'int_curr_symbol' => '&#75;&#71;&#83;&#32;',  'currency_symbol' => '&#1089;&#1086;&#1084;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['KHR'] = array('name' => 'Cambodian Riel',  'int_curr_symbol' => '&#75;&#72;&#82;&#32;',  'currency_symbol' => '&#6107;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '0',  'n_cs_precedes' => '0',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',  'int_p_cs_precedes' => '0',  'int_p_sep_by_space' => '0',  'int_n_cs_precedes' => '0',  'int_n_sep_by_space' => '0',  'int_p_sign_posn' => '1',  'int_n_sign_posn' => '1',);
        self::$currencies['KRW'] = array('name' => 'South Korean Won',  'int_curr_symbol' => '&#75;&#82;&#87;&#32;',  'currency_symbol' => '&#65510;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '0',  'frac_digits' => '0',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '4',);
        self::$currencies['KWD'] = array('name' => 'Kuwaiti Dinar',  'int_curr_symbol' => '&#75;&#87;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1603;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['KZT'] = array('name' => 'Kazakhstani Tenge',  'int_curr_symbol' => '&#75;&#90;&#84;&#32;',  'currency_symbol' => '&#1090;&#1075;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['LAK'] = array('name' => 'Laotian Kip',  'int_curr_symbol' => '&#76;&#65;&#75;&#32;',  'currency_symbol' => '&#8365;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '2',  'n_cs_precedes' => '1',  'n_sep_by_space' => '2',  'p_sign_posn' => '4',  'n_sign_posn' => '4',);
        self::$currencies['LBP'] = array('name' => 'Lebanese Pound',  'int_curr_symbol' => '&#76;&#66;&#80;&#32;',  'currency_symbol' => '&#1604;&#46;&#1604;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['LKR'] = array('name' => 'Sri Lankan Rupee',  'int_curr_symbol' => '&#76;&#75;&#82;&#32;',  'currency_symbol' => '&#8360;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['LTL'] = array('name' => 'Lithuanian Litas',  'int_curr_symbol' => '&#76;&#84;&#76;&#32;',  'currency_symbol' => '&#76;&#116;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['LVL'] = array('name' => 'Latvian Lat',  'int_curr_symbol' => '&#76;&#86;&#76;&#32;',  'currency_symbol' => '&#76;&#115;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '&#160;',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '3',  'n_sign_posn' => '3',);
        self::$currencies['LYD'] = array('name' => 'Libyan Dinar',  'int_curr_symbol' => '&#76;&#89;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1604;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['MAD'] = array('name' => 'Moroccan Dirham',  'int_curr_symbol' => '&#77;&#65;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1605;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['MGA'] = array('name' => 'Malagasy Ariary',  'int_curr_symbol' => '&#77;&#71;&#65;&#32;',  'currency_symbol' => '&#65;&#82;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['MKD'] = array('name' => 'Macedonian Denar',  'int_curr_symbol' => '&#77;&#75;&#68;&#32;',  'currency_symbol' => '&#1076;&#1077;&#1085;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['MNT'] = array('name' => 'Mongolian Tughrik',  'int_curr_symbol' => '&#77;&#78;&#84;&#32;',  'currency_symbol' => '&#8366;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['MXN'] = array('name' => 'Mexican Peso',  'int_curr_symbol' => '&#77;&#88;&#78;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['MYR'] = array('name' => 'Malaysian Ringgit',  'int_curr_symbol' => '&#77;&#89;&#82;&#32;',  'currency_symbol' => '&#82;&#77;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '0',);
        self::$currencies['NIO'] = array('name' => 'Nicaraguan Cordoba',  'int_curr_symbol' => '&#78;&#73;&#79;&#32;',  'currency_symbol' => '&#36;&#67;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['NPR'] = array('name' => 'Nepalese Rupee',  'int_curr_symbol' => '&#73;&#78;&#82;&#32;',  'currency_symbol' => '&#2352;&#2370;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['NZD'] = array('name' => 'New Zealand Dollar',  'int_curr_symbol' => '&#78;&#90;&#68;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['OMR'] = array('name' => 'Omani Rial',  'int_curr_symbol' => '&#79;&#77;&#82;&#32;',  'currency_symbol' => '&#1585;&#46;&#1593;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['PAB'] = array('name' => 'Panamanian Balboa',  'int_curr_symbol' => '&#80;&#65;&#66;&#32;',  'currency_symbol' => '&#66;&#47;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['PEN'] = array('name' => 'Peruvian Nuevo Sol',  'int_curr_symbol' => '&#80;&#69;&#78;&#32;',  'currency_symbol' => '&#83;&#47;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['PHP'] = array('name' => 'Philippine Peso',  'int_curr_symbol' => '&#80;&#72;&#80;&#32;',  'currency_symbol' => '&#80;&#104;&#80;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['PKR'] = array('name' => 'Pakistani Rupee',  'int_curr_symbol' => '&#80;&#75;&#82;&#32;',  'currency_symbol' => '&#82;&#115;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '2',  'n_sign_posn' => '1',);
        self::$currencies['PLN'] = array('name' => 'Polish Zloty',  'int_curr_symbol' => '&#80;&#76;&#78;&#32;',  'currency_symbol' => '&#122;&#322;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['PYG'] = array('name' => 'Paraguayan Guarani',  'int_curr_symbol' => '&#80;&#89;&#71;&#32;',  'currency_symbol' => '&#71;&#115;&#46;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['QAR'] = array('name' => 'Qatari Riyal',  'int_curr_symbol' => '&#81;&#65;&#82;&#32;',  'currency_symbol' => '&#81;&#65;&#82;&#32;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['RON'] = array('name' => 'Romanian New Leu',  'int_curr_symbol' => '&#82;&#79;&#78;&#32;',  'currency_symbol' => '&#76;&#101;&#105;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['RUB'] = array('name' => 'Russian Ruble',  'int_curr_symbol' => '&#82;&#85;&#66;&#32;',  'currency_symbol' => '&#1088;&#1091;&#1073;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['RWF'] = array('name' => 'Rwandan Franc',  'int_curr_symbol' => '&#82;&#87;&#70;&#32;',  'currency_symbol' => '&#70;&#114;&#119;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['SAR'] = array('name' => 'Saudi Arabian Riyal',  'int_curr_symbol' => '&#83;&#65;&#82;&#32;',  'currency_symbol' => '&#1585;&#1610;&#1575;&#1604;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => '',  'mon_grouping' => '-1',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['SDG'] = array('name' => 'Sudanese Pound',  'int_curr_symbol' => '&#83;&#68;&#68;&#32;',  'currency_symbol' => '&#1580;&#46;&#1587;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['SGD'] = array('name' => 'Singapore Dollar',  'int_curr_symbol' => '&#83;&#71;&#68;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '0',);
        self::$currencies['SOS'] = array('name' => 'Somali Shilling',  'int_curr_symbol' => '&#83;&#79;&#83;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['SVC'] = array('name' => 'Salvadoran Colon',  'int_curr_symbol' => '&#83;&#86;&#67;&#32;',  'currency_symbol' => '&cent;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['SYP'] = array('name' => 'Syrian Pound',  'int_curr_symbol' => '&#83;&#89;&#80;&#32;',  'currency_symbol' => '&#1604;&#46;&#1587;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['THB'] = array('name' => 'Thai Baht',  'int_curr_symbol' => '&#84;&#72;&#66;&#32;',  'currency_symbol' => '&#3647;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '2',  'n_cs_precedes' => '1',  'n_sep_by_space' => '2',  'p_sign_posn' => '4',  'n_sign_posn' => '4',);
        self::$currencies['TJS'] = array('name' => 'Tajikistani Somoni',  'int_curr_symbol' => '&#84;&#74;&#83;&#32;',  'currency_symbol' => '&#1088;&#1091;&#1073;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['TND'] = array('name' => 'Tunisian Dinar',  'int_curr_symbol' => '&#84;&#78;&#68;&#32;',  'currency_symbol' => '&#1583;&#46;&#1578;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['TRY'] = array('name' => 'Turkish Lira',  'int_curr_symbol' => '&#84;&#82;&#89;',  'currency_symbol' => '&#8378;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['TWD'] = array('name' => 'Taiwan New Dollar',  'int_curr_symbol' => '&#84;&#87;&#68;&#32;',  'currency_symbol' => '&#78;&#84;&#36;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'int_p_cs_precedes' => '1',  'int_p_sep_by_space' => '0',  'int_n_cs_precedes' => '1',  'int_n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',  'int_p_sign_posn' => '1',  'int_n_sign_posn' => '1',);
        self::$currencies['UAH'] = array('name' => 'Ukrainian Hryvna',  'int_curr_symbol' => '&#85;&#65;&#72;&#32;',  'currency_symbol' => '&#1075;&#1088;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ' ',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  'p_sep_by_space' => '1',  'n_cs_precedes' => '0',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['UGX'] = array('name' => 'Ugandan Shilling',  'int_curr_symbol' => '&#85;&#71;&#88;&#32;',  'currency_symbol' => '&#47;&#45;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '0',  '%' => 'values.',  'p_sep_by_space' => '0',  'n_cs_precedes' => '0',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['UYU'] = array('name' => 'Uruguayan Peso',  'int_curr_symbol' => '&#85;&#89;&#85;&#32;',  'currency_symbol' => '$',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['UZS'] = array('name' => 'Uzbekistani Som',  'int_curr_symbol' => '&#85;&#90;&#83;&#32;',  'currency_symbol' => '&#115;&#111;&#39;&#109;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['VEF'] = array('name' => 'Venezuelan Bolivar',  'int_curr_symbol' => '&#86;&#69;&#66;&#32;',  'currency_symbol' => '&#66;&#115;&#46;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['VND'] = array('name' => 'Vietnamese Dong',  'int_curr_symbol' => '&#86;&#78;&#68;&#32;',  'currency_symbol' => '&#8363;',  'mon_decimal_point' => ',',  'mon_thousands_sep' => '.',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '4',  'frac_digits' => '4',  'p_cs_precedes' => '0',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['YER'] = array('name' => 'Yemeni Rial',  'int_curr_symbol' => '&#89;&#69;&#82;&#32;',  'currency_symbol' => '&#1585;&#46;&#1610;&#46;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '3',  'frac_digits' => '3',  'p_cs_precedes' => '1',  'p_sep_by_space' => '1',  'n_cs_precedes' => '1',  'n_sep_by_space' => '1',  'p_sign_posn' => '1',  'n_sign_posn' => '2',);
        self::$currencies['ZAR'] = array('name' => 'South African Rand',  '%' => '-',  'int_curr_symbol' => '&#90;&#65;&#82;&#32;',  'currency_symbol' => '&#82;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
        self::$currencies['ZWD'] = array('name' => 'Zimbabwean Dollar',  'int_curr_symbol' => '&#90;&#87;&#68;&#32;',  'currency_symbol' => '&#90;&#36;',  'mon_decimal_point' => '.',  'mon_thousands_sep' => ',',  'mon_grouping' => '3;3',  'positive_sign' => '',  'negative_sign' => '',  'int_frac_digits' => '2',  'frac_digits' => '2',  'p_cs_precedes' => '1',  'p_sep_by_space' => '0',  'n_cs_precedes' => '1',  'n_sep_by_space' => '0',  'p_sign_posn' => '1',  'n_sign_posn' => '1',);
    }


    public static function format($format, $number, $currencyCode = "", $formatOverride = "")
    {
        $regex  = '/%((?:[\^!\-]|\+|\(|\=.)*)([0-9]+)?' .
            '(?:#([0-9]+))?(?:\.([0-9]+))?([in%])/';

        $currency = strtoupper($currencyCode);
        if (empty($currency) || !isset(self::$currencies[$currencyCode])) {
            $currency = MM_OptionUtils::getOption(MM_OptionUtils::$OPTION_KEY_CURRENCY);
            if (empty($currency)) {
                $locale = self::$currencies['USD'];
            }
        } else {
            $locale = self::$currencies[$currencyCode];
        }

        if (is_array($formatOverride)) {
            $locale = $formatOverride + $locale;
        }


        preg_match_all($regex, $format, $matches, PREG_SET_ORDER);
        foreach ($matches as $fmatch) {
            $value = floatval($number);
            $flags = array(
                'fillchar'  => preg_match('/\=(.)/', $fmatch[1], $match) ?
                    $match[1] : ' ',
                'nogroup'   => preg_match('/\^/', $fmatch[1]) > 0,
                'usesignal' => preg_match('/\+|\(/', $fmatch[1], $match) ?
                    $match[0] : '+',
                'nosimbol'  => preg_match('/\!/', $fmatch[1]) > 0,
                'isleft'    => preg_match('/\-/', $fmatch[1]) > 0
            );
            $width      = trim($fmatch[2]) ? (int)$fmatch[2] : 0;
            $left       = trim($fmatch[3]) ? (int)$fmatch[3] : 0;
            $right      = trim($fmatch[4]) ? (int)$fmatch[4] : $locale['int_frac_digits'];
            $conversion = $fmatch[5];

            $positive = true;
            if ($value < 0) {
                $positive = false;
                $value  *= -1;
            }
            $letter = $positive ? 'p' : 'n';

            $prefix = $suffix = $cprefix = $csuffix = $signal = '';

            $signal = $positive ? $locale['positive_sign'] : $locale['negative_sign'];
            switch (true) {
                case $locale["{$letter}_sign_posn"] == 1 && $flags['usesignal'] == '+':
                    $prefix = $signal;
                    break;
                case $locale["{$letter}_sign_posn"] == 2 && $flags['usesignal'] == '+':
                    $suffix = $signal;
                    break;
                case $locale["{$letter}_sign_posn"] == 3 && $flags['usesignal'] == '+':
                    $cprefix = $signal;
                    break;
                case $locale["{$letter}_sign_posn"] == 4 && $flags['usesignal'] == '+':
                    $csuffix = $signal;
                    break;
                case $flags['usesignal'] == '(':
                case $locale["{$letter}_sign_posn"] == 0:
                    $prefix = '(';
                    $suffix = ')';
                    break;
            }
            if (!$flags['nosimbol']) {
                $currency = $cprefix .
                    ($conversion == 'i' ? $locale['int_curr_symbol'] : $locale['currency_symbol']) .
                    $csuffix;
            } else {
                $currency = '';
            }
            $space  = $locale["{$letter}_sep_by_space"] ? ' ' : '';

            $value = number_format(
                $value,
                $right,
                $locale['mon_decimal_point'],
                $flags['nogroup'] ? '' : $locale['mon_thousands_sep']
            );
            $value = @explode($locale['mon_decimal_point'], $value);

            $n = strlen($prefix) + strlen($currency) + strlen($value[0]);
            if ($left > 0 && $left > $n) {
                $value[0] = str_repeat($flags['fillchar'], $left - $n) . $value[0];
            }
            $value = implode($locale['mon_decimal_point'], $value);
            if ($locale["{$letter}_cs_precedes"]) {
                $value = $prefix . $currency . $space . $value . $suffix;
            } else {
                $value = $prefix . $value . $space . $currency . $suffix;
            }
            if ($width > 0) {
                $value = str_pad($value, $width, $flags['fillchar'], $flags['isleft'] ?
                    STR_PAD_RIGHT : STR_PAD_LEFT);
            }

            $format = str_replace($fmatch[0], $value, $format);
        }
        $postfixIso = MM_OptionUtils::getOption(MM_OptionUtils::$OPTION_KEY_CURRENCY_FORMAT_POSTFIX_ISO);
        $postfixIso = (empty($postfixIso) || ($postfixIso == 'false')) ? false : $postfixIso;

        //allow this setting to be overriden
        $postfixIso = isset($locale['postfix_iso']) ? $locale['postfix_iso'] : $postfixIso;

        if ($postfixIso) {
            $format .= " {$currencyCode}";
        }
        return $format;
    }


    /**
     * Get an array of supported currencies
     * 
     * @return array of supported currencies keyed by iso code
     */
    public static function getSupportedCurrencies()
    {
        $supportedCurrencies = array();
        foreach (self::$currencies as $isoCode => $metadata) {
            $supportedCurrencies[$isoCode] = "{$metadata['name']} ({$metadata['currency_symbol']})";
        }
        return $supportedCurrencies;
    }


    public static function isSupportedCurrency($currencyCode)
    {
        return isset(self::$currencies[$currencyCode]);
    }


    public static function getActiveCurrency()
    {
        return MM_OptionUtils::getOption(MM_OptionUtils::$OPTION_KEY_CURRENCY);
    }


    public static function getCurrencyMetadata($currencyCode)
    {
        $postfixIso = MM_OptionUtils::getOption(MM_OptionUtils::$OPTION_KEY_CURRENCY_FORMAT_POSTFIX_ISO);
        $postfixIso = (empty($postfixIso) || ($postfixIso == 'false')) ? false : $postfixIso;
        if (isset(self::$currencies[$currencyCode])) {
            $metadata = array("currency" => $currencyCode, "postfixIso" => $postfixIso) + self::$currencies[$currencyCode];
            return $metadata;
        } else {
            return null;
        }
    }


    public static function getActiveCurrencyMetadata()
    {
        $activeCurrency = self::getActiveCurrency();
        return self::getCurrencyMetadata($activeCurrency);
    }


    public static function getPrecisionForCurrency($currencyCode)
    {
        $currency = empty($currencyCode) ? MM_CurrencyUtil::getActiveCurrency() : $currencyCode;
        $currencyMetadata = self::getCurrencyMetadata($currency);
        $precision = isset($currencyMetadata['frac_digits']) ? $currencyMetadata['frac_digits'] : 2;
        return $precision;
    }
}
MM_CurrencyUtil::__initializeCurrencyFormats(); //static initialization
