<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_HtmlUtils
{
    public static function getEmployees($selections = null)
    {
        $emails = MM_Employee::getEmployeesList();

        if (is_null($selections) || $selections == "") {
            $selections = $emails->default;
        }

        return self::generateSelectionsList($emails->list, $selections);
    }

    public static function getWordPressRoles($selections = null)
    {
        global $wp_roles;
        $roles = $wp_roles->get_names();

        $list = array();

        foreach ($roles as $role_id => $role_name) {
            $list[$role_id] = $role_name;
        }

        $list[""] = "&mdash; " . __("No role for this site", "membermouse") . " &mdash;";
        $list[MM_Role::$ROLE_IGNORE] = "&mdash;" . __("Don't set or change role", "membermouse") . " &mdash;";

        return self::generateSelectionsList($list, $selections);
    }

    public static function getWordPressCategories($selections = null)
    {
        $categories = get_categories();

        $list = array();

        foreach ($categories as $category) {
            $list[$category->term_id] = $category->name;
        }

        return self::generateSelectionsList($list, $selections);
    }

    public static function getMembershipProducts($membershipId, $selections = null, $disabled = null, $excludeArchived = false)
    {
        $list = MM_Product::getMembershipProductList($membershipId, $excludeArchived);
        return self::generateSelectionsList($list, $selections, $disabled);
    }

    public static function getBundleProducts($bundleId, $selections = null, $disabeld = null, $excludeArchived = false)
    {
        $list = MM_Product::getBundleProductList($bundleId, $excludeArchived);
        return self::generateSelectionsList($list, $selections, $disabeld);
    }

    public static function getCustomFields($selections = null, $includeHidden = true)
    {
        $list = MM_CustomField::getCustomFieldsList(false, $includeHidden);
        return self::generateSelectionsList($list, $selections);
    }

    public static function getMemberships($selections = null, $activeStatusOnly = false, $filterBySubType = "", $excludeArchived = false)
    {
        $list = MM_MembershipLevel::getMembershipLevelsList($activeStatusOnly, $filterBySubType, $excludeArchived);
        return self::generateSelectionsList($list, $selections);
    }

    public static function getBundles($selections = null, $activeStatusOnly = false, $filterBySubType = "", $excludeArchived = false)
    {
        $list = MM_Bundle::getBundlesList($activeStatusOnly, $filterBySubType, $excludeArchived);
        return self::generateSelectionsList($list, $selections);
    }

    public static function getProducts($selections = null, $excludeProductIds = null, $excludeArchived = false)
    {
        $list = MM_Product::getAll($excludeProductIds, $excludeArchived);
        return self::generateSelectionsList($list, $selections);
    }


    public static function getCouponCodes($selections = null, $excludeCouponIds = null)
    {
        $list = MM_Coupon::getAllCodes();
        return self::generateSelectionsList($list, $selections);
    }


    public static function getNonFreeCouponCodes($selections = null, $excludeCouponIds = null)
    {
        $list = MM_Coupon::getAllCodes(true);
        return self::generateSelectionsList($list, $selections);
    }

    public static function getMemberStatusList($selections = null)
    {
        $list = MM_Status::getStatusTypesList();
        return self::generateSelectionsList($list, $selections);
    }

    public static function getCustomFieldTypeList($selections = null)
    {
        $list = MM_CustomField::getFieldTypeList();
        return self::generateSelectionsList($list, $selections);
    }

    public static function getEventTypesList($selections = null)
    {
        $list = MM_Event::getEventTypesList($selections);
        return self::generateSelectionsList($list, $selections);
    }

    public static function getReminderTypesList($selections = null)
    {
        $list = MM_Event::getReminderTypesList($selections);
        return self::generateSelectionsList($list, $selections);
    }

    public static function getActionsList($selections = null)
    {
        $list = MM_Action::getActionsList();
        return self::generateSelectionsList($list, $selections);
    }

    public static function getFullCountryList($selections = "")
    {
        if (is_null($selections) || $selections == "") {
            $selections = MM_OptionUtils::$DEFAULT_COUNTRY_ISO;
        }

        $list = MM_CountryUtils::getFullCountryList();

        return self::generateSelectionsList($list, $selections);
    }

    public static function getCountryList($selection = "")
    {
        if (is_null($selection) || $selection == "") {
            $selection = MM_OptionUtils::$DEFAULT_COUNTRY_ISO;
        }

        $list = MM_CountryUtils::getCountryList();

        return self::generateSelectionsList($list, $selection);
    }

    public static function getCommissionProfilesList($selections = null)
    {
        $list = MM_CommissionProfile::getCommissionProfilesList();
        return self::generateSelectionsList($list, $selections);
    }

    public static function getUSStatesList()
    {
        $html = "<option value='AL'>Alabama</option>
<option value='AK'>Alaska</option>
<option value='AS'>American Samoa</option>
<option value='AZ'>Arizona</option>
<option value='AR'>Arkansas</option>
<option value='AE'>Armed Forces Africa</option>
<option value='AA'>Armed Forces Americas</option>
<option value='AE'>Armed Forces Canada</option>
<option value='AE'>Armed Forces Europe</option>
<option value='AE'>Armed Forces Middle East</option>
<option value='AP'>Armed Forces Pacific</option>
<option value='CA'>California</option>
<option value='CO'>Colorado</option>
<option value='CT'>Connecticut</option>
<option value='DE'>Delaware</option>
<option value='DC'>District of Columbia</option>
<option value='FL'>Florida</option>
<option value='GA'>Georgia</option>
<option value='GU'>Guam</option>
<option value='HI'>Hawaii</option>
<option value='ID'>Idaho</option>
<option value='IL'>Illinois</option>
<option value='IN'>Indiana</option>
<option value='IA'>Iowa</option>
<option value='KS'>Kansas</option>
<option value='KY'>Kentucky</option>
<option value='LA'>Louisiana</option>
<option value='ME'>Maine</option>
<option value='MH'>Marshall Islands</option>
<option value='MD'>Maryland</option>
<option value='MA'>Massachusetts</option>
<option value='MI'>Michigan</option>
<option value='MN'>Minnesota</option>
<option value='MS'>Mississippi</option>
<option value='MO'>Missouri</option>
<option value='MT'>Montana</option>
<option value='NE'>Nebraska</option>
<option value='NV'>Nevada</option>
<option value='NH'>New Hampshire</option>
<option value='NJ'>New Jersey</option>
<option value='NM'>New Mexico</option>
<option value='NY'>New York</option>
<option value='NC'>North Carolina</option>
<option value='ND'>North Dakota</option>
<option value='OH'>Ohio</option>
<option value='OK'>Oklahoma</option>
<option value='OR'>Oregon</option>
<option value='PA'>Pennsylvania</option>
<option value='PR'>Puerto Rico</option>
<option value='RI'>Rhode Island</option>
<option value='SC'>South Carolina</option>
<option value='SD'>South Dakota</option>
<option value='TN'>Tennessee</option>
<option value='TX'>Texas</option>
<option value='UT'>Utah</option>
<option value='VT'>Vermont</option>
<option value='VI'>Virgin Islands</option>
<option value='VA'>Virginia</option>
<option value='WA'>Washington</option>
<option value='WV'>West Virginia</option>
<option value='WI'>Wisconsin</option>
<option value='WY'>Wyoming</option>";
        return $html;
    }

    public static function getShippingMethodList($selection = "", $order = "")
    {
        $options = MM_ShippingMethod::getAvailableShippingOptions($order);
        $list = array();

        // NOTE it's important that the format of option text be 'name (price)' because the checkout
        // form uses javascript to parse the selected shipping method to get the price to use in onpage
        // calculations. This specific format is required so the string can be parsed correctly.
        foreach ($options as $shippingOption) {
            $list[$shippingOption->getKey()] = "{$shippingOption->getName()} ({$shippingOption->getRate(true)})";
        }
        return self::generateSelectionsList($list, $selection);
    }

    public static function getCCExpMonthList($selection = "")
    {
        $list = array();

        $list["01"] = "(01) " . __("Jan", "membermouse");
        $list["02"] = "(02) " . __("Feb", "membermouse");
        $list["03"] = "(03) " . __("Mar", "membermouse");
        $list["04"] = "(04) " . __("Apr", "membermouse");
        $list["05"] = "(05) " . __("May", "membermouse");
        $list["06"] = "(06) " . __("Jun", "membermouse");
        $list["07"] = "(07) " . __("Jul", "membermouse");
        $list["08"] = "(08) " . __("Aug", "membermouse");
        $list["09"] = "(09) " . __("Sep", "membermouse");
        $list["10"] = "(10) " . __("Oct", "membermouse");
        $list["11"] = "(11) " . __("Nov", "membermouse");
        $list["12"] = "(12) " . __("Dec", "membermouse");

        return self::generateSelectionsList($list, $selection);
    }

    public static function getCCExpYearList($selection = "")
    {
        $list = array();
        $crntYear = intval(date("Y"));
        $maxYear = $crntYear + 20;
        for ($i = $crntYear; $i <= $maxYear; $i++) {
            $list[$i] = $i;
        }

        return self::generateSelectionsList($list, $selection);
    }

    public static function getDataGridResultsCount($selection = "")
    {
        $list = ["10" => 10, "20" => "20", "50" => "50", "100" => "100", "500" => "500", "1000" => "1000"];
        return self::generateSelectionsList($list, $selection);
    }

    public static function getMMObjectTypes($selection = "")
    {
        $list = array();

        $list[MM_TYPE_PRODUCT] = __("Products", "membermouse");
        $list[MM_TYPE_BUNDLE] = __("Bundles", "membermouse");
        $list[MM_TYPE_MEMBERSHIP_LEVEL] = __("Membership Levels", "membermouse");
        $list[MM_TYPE_EMPLOYEE_ACCOUNT] = __("Employees", "membermouse");
        if (MM_CustomField::hasCustomFields()) {
            $list[MM_TYPE_CUSTOM_FIELD] = __("Custom Fields", "membermouse");
        }
        $list[MM_TYPE_POST] = __("Protected Content", "membermouse");

        return self::generateSelectionsList($list, $selection);
    }


    public static function getSupportedCurrenciesList($selection = "")
    {
        return self::generateSelectionsList(MM_CurrencyUtil::getSupportedCurrencies(), $selection);
    }


    public static function generateSelectionsList($list, $selections = null, $disabled = null)
    {
        $html = "";
        if (!empty($list)) {
            foreach ($list as $value => $key) {
                $isSelected = "";

                if (!is_null($selections)) {
                    if (is_array($selections)) {
                        if (array_key_exists($value, $selections)) {
                            $isSelected = "selected";
                        }
                    } else if (trim($selections) == trim($value)) {
                        $isSelected = "selected";
                    }
                }

                $isDisabled = "";
                if (!is_null($disabled)) {
                    if (is_array($disabled)) {
                        if (array_key_exists($value, $disabled)) {
                            $isDisabled = "disabled='disabled'";
                        }
                    } else if ($disabled == $value) {
                        $isDisabled = "disabled='disabled'";
                    }
                }

                $html .= "<option value='{$value}' {$isSelected} {$isDisabled}>" . stripslashes($key) . "</option>
";
            }
        }

        return $html;
    }

    public static function createCheckboxGroup($obj, $name, $sel = null, $img = "", $disabled = null, $onchange = "", $valueWidth = false, $asRadioButton = false)
    {
        $onchange = (!empty($onchange)) ? "onchange=\"{$onchange}\"" : "";
        $image = "";
        if (!empty($img)) {
            $image = MM_Utils::getImageUrl($img);
            if (!empty($image))
                $image = "<img src='{$image}' />";
        }

        $select = "<table width='98%'>";
        if (!empty($obj)) {
            foreach ($obj as $k => $v) {
                $d = "";
                $s = "";
                if (!is_null($sel)) {
                    if (is_array($sel)) {
                        if (array_key_exists($k, $sel)) {
                            $s = "checked";
                        }
                    } else if ($sel == $k)
                        $s = "checked";
                }
                if (!is_null($disabled)) {
                    if (is_array($disabled)) {
                        if (array_key_exists($k, $disabled)) {
                            $d = "disabled";
                        }
                    } else if ($disabled == $k)
                        $d = "disabled";
                }
                $value = "";
                $image = "";
                $alt = "";
                if (is_object($v) && isset($v->value)) {
                    $value = $v->value;
                    if (isset($v->image) && empty($image)) {
                        if (preg_match("/(<i)/", $v->image)) {
                            $image = $v->image;
                        } else {
                            $image = MM_Utils::getImageUrl($v->image);
                            if (preg_match("/(http)/", $v->image)) {
                                $image = $v->image;
                            }
                            if (!empty($image) && !empty($v->image)) {
                                $image = "<img src='{$image}' />";
                            } else {
                                $image = "";
                            }
                        }
                    }
                    if (isset($v->alt)) {
                        $alt = $v->alt;
                    }
                } else if (is_object($v)) {
                    continue;
                } else
                    $value = $v;

                $title = (!empty($alt)) ? $alt : $value;
                if (intval($valueWidth) > 0) {
                    $value = MM_Utils::abbrevString($value, $valueWidth);
                } else {
                    $value = MM_Utils::abbrevString($value);
                }
                $fieldType = "checkbox";
                if ($asRadioButton) {
                    $fieldType = "radio";
                }
                $select .= "<tr title='{$title}'><td style='width:5px; vertical-align:top;'><input type='{$fieldType}' value='{$k}' name='{$name}' id='{$name}' {$s} {$d} {$onchange}></td><td width='5%'>{$image}</td><td width='90%'>" . stripslashes($value) . "</td></tr>";
            }
        }
        return $select . "</table>";
    }

    public static function getEmailServiceProvidersList($selection = "")
    {
        $list = MM_EmailServiceProvider::getEmailServiceProvidersList();
        return self::generateSelectionsList($list, $selection);
    }

    public static function getAffiliateProvidersList($selection = "")
    {
        $list = MM_AffiliateProvider::getAffiliateProvidersList();
        return self::generateSelectionsList($list, $selection);
    }
}
