<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_ModuleUtils
{

    /**
     * Checks to see if the current request is a REST request.
     * @returns boolean
     */
    public static function isRestRequest()
    {
        /// obtain the rest prefix for each endpoint.
        $prefix = rest_get_url_prefix();

        /// check if the WP supported constant is true
        /// if this is the case return true.
        /// in the case that 'plain' permalinks structure is setup, ensure
        /// we accommodate as such.
        if (
            defined('REST_REQUEST') && REST_REQUEST
            || isset($_GET['rest_route'])
            && strpos(trim($_GET['rest_route'], '\\/'), $prefix, 0) === 0
        )
            return true;

        /// sub folder installs
        $restUrl = wp_parse_url(site_url($prefix));
        $currentUrl = wp_parse_url(add_query_arg(array()));
        return strpos($currentUrl['path'], $restUrl['path'], 0) === 0;
    }


    public static function getPage()
    {
        return isset($_REQUEST["page"]) ? $_REQUEST["page"] : "";
    }

    public static function getPrimaryTab()
    {
        return self::getParentTab(self::getModule());
    }

    public static function getParentTab($tab)
    {
        switch ($tab) {
            case MM_MODULE_ACTIVITY_LOG:
            case MM_MODULE_GIFTS:
            case MM_MODULE_COUPON_LOG:
            case MM_MODULE_PAYPAL_IPN_LOG:
            case MM_MODULE_CAPTCHA_IPN_LOG:
            case MM_MODULE_CLICKBANK_IPN_LOG:
            case MM_MODULE_LIMELIGHT_LOG:
            case MM_MODULE_STICKYIO_LOG:
                return MM_MODULE_LOGS;

            case MM_MODULE_API:
            case MM_MODULE_PUSH_NOTIFICATIONS:
            case MM_MODULE_PHP_INTERFACE:
            case MM_MODULE_WORDPRESS_HOOKS:
                return MM_MODULE_DEVELOPER_TOOLS;

            case MM_MODULE_PAYMENT_METHODS:
            case MM_MODULE_TEST_DATA:
            case MM_MODULE_CANCELLATION_METHOD:
            case MM_MODULE_REMINDERS:
                return MM_MODULE_PAYMENT_SETTINGS;

            case MM_MODULE_AFFILIATE_INTEGRATION:
            case MM_MODULE_AFFILIATE_TRACKING:
            case MM_MODULE_COMMISSION_PROFILES:
                return MM_MODULE_AFFILIATE_SETTINGS;

            case MM_MODULE_EMAIL_INTEGRATION:
            case MM_EMAIL_TEMPLATES:
                return MM_MODULE_EMAIL_SETTINGS;

            case MM_MODULE_MEMBERSHIP_LEVELS:
            case MM_MODULE_BUNDLES:
            case MM_MODULE_PRODUCTS:
            case MM_MODULE_COUPONS:
            case MM_MODULE_DRIP_CONTENT_SCHEDULE:
            case MM_MODULE_LIMELIGHT_PRODUCTS:
            case MM_MODULE_LIMELIGHT_SHIPPING_METHODS:
                return MM_MODULE_PRODUCT_SETTINGS;

            case MM_MODULE_CUSTOM_FIELDS:
            case MM_MODULE_COUNTRIES:
            case MM_MODULE_SHIPPING:
            case MM_MODULE_CHECKOUT_OTHER_SETTINGS:
                return MM_MODULE_CHECKOUT_SETTINGS;

            case MM_MODULE_FREE_MEMBER_FORM:
            case MM_MODULE_LOGIN_FORM:
                return MM_MODULE_WEBFORMS;

            case MM_MODULE_VERSION_HISTORY:
            case MM_MODULE_REPAIR_INSTALL:
            case MM_MODULE_MIGRATE_DB:
            case MM_MODULE_EMPLOYEES:
            case MM_MODULE_DUPLICATE_SUBSCRIPTION_TOOL:
            case MM_MODULE_EXTENSIONS:
            case MM_MODULE_OTHER_SETTINGS:
            case MM_MODULE_SAFE_MODE:
            case MM_MODULE_DIAGNOSTICS:
            case MM_MODULE_SUPPORT:
                return MM_MODULE_GENERAL_SETTINGS;

            case MM_MODULE_MEMBER_DETAILS_GENERAL:
            case MM_MODULE_MEMBER_DETAILS_ACCESS_RIGHTS:
            case MM_MODULE_MEMBER_DETAILS_TRANSACTION_HISTORY:
            case MM_MODULE_MEMBER_DETAILS_SUBSCRIPTIONS:
            case MM_MODULE_MEMBER_DETAILS_GIFT_HISTORY:
            case MM_MODULE_MEMBER_DETAILS_CUSTOM_FIELDS:
            case MM_MODULE_MEMBER_DETAILS_ACTIVITY_LOG:
            case MM_MODULE_MEMBER_DETAILS_USER_DEFINED:
            case MM_MODULE_IMPORT_WIZARD:
            case MM_MODULE_USER_DEFINED_PAGES:
                return MM_MODULE_MEMBER_DETAILS;

            default:
                return "";
        }
    }

    public static function getModule()
    {
        $pageName = self::getPage();

        $module = isset($_REQUEST["module"]) ? $_REQUEST["module"] : "";

        if (!isset($module) || $module == "") {
            switch ($pageName) {
                case MM_MODULE_LOGS:
                    $module = MM_MODULE_ACTIVITY_LOG;
                    break;

                case MM_MODULE_DEVELOPER_TOOLS:
                    $module = MM_MODULE_PUSH_NOTIFICATIONS;
                    break;

                case MM_MODULE_PAYMENT_SETTINGS:
                    $module = MM_MODULE_PAYMENT_METHODS;
                    break;

                case MM_MODULE_AFFILIATE_SETTINGS:
                    $module = MM_MODULE_AFFILIATE_INTEGRATION;
                    break;

                case MM_MODULE_EMAIL_SETTINGS:
                    $module = MM_MODULE_EMAIL_INTEGRATION;
                    break;

                case MM_MODULE_PRODUCT_SETTINGS:
                    $module = MM_MODULE_PRODUCTS;
                    break;

                case MM_MODULE_CHECKOUT_SETTINGS:
                    $module = MM_MODULE_CUSTOM_FIELDS;
                    break;

                case MM_MODULE_WEBFORMS:
                    $module = MM_MODULE_FREE_MEMBER_FORM;
                    break;

                case MM_MODULE_GENERAL_SETTINGS:
                    $module = MM_MODULE_EMPLOYEES;
                    break;

                case MM_MODULE_COURSES:
                    $module = MM_MODULE_COURSES;
                    break;

                case MM_MODULE_DASHBOARD:
                    $module = MM_MODULE_DASHBOARD;
                    break;

                case MM_MODULE_MANAGE_MEMBERS:
                    $module = MM_MODULE_BROWSE_MEMBERS;
                    break;

                case MM_MODULE_MANAGE_TRANSACTIONS:
                    $module = MM_MODULE_MANAGE_TRANSACTIONS;
                    break;

                case MM_MODULE_UPCOMING_PAYMENTS:
                    $module = MM_MODULE_UPCOMING_PAYMENTS;
                    break;

                case MM_MODULE_REPORTING:
                case MM_MODULE_NEW_MEMBERS_REPORT:
                case MM_MODULE_NEW_MEMBERS_COMPARISON_REPORT:
                case MM_MODULE_SALES_BY_PAYMENT_SERVICE_REPORT:
                case MM_MODULE_TOTAL_VALUE_REPORT:
                case MM_MODULE_SALES_BY_PRODUCT_REPORT:
                case MM_MODULE_SIMPLE_AVERAGE_REPORT:
                case MM_MODULE_CUSTOMER_VALUE_REPORT:
                case MM_MODULE_AFFILIATE_REPORT:
                case MM_MODULE_SALES_BY_MEMBERSHIP_REPORT:
                case MM_MODULE_GET_REPORTING:
                    $module = MM_MODULE_REPORTING;
                    break;

                default:
                    $module = "";
                    break;
            }
        }

        return $module;
    }

    public static function getSubModule()
    {
        $submodule = isset($_REQUEST["submodule"]) ? $_REQUEST["submodule"] : "";
        return $submodule;
    }

    public static function getUrl($page, $module = "")
    {
        $url = admin_url("admin.php");
        $url .= "?page=" . $page;

        if (!empty($module)) {
            $url .= "&module=" . $module;
        }

        return $url;
    }

    public static function isMemberMousePage($page)
    {
        $validPages = array(
            MM_MODULE_DASHBOARD,
            MM_MODULE_MANAGE_MEMBERS,
            MM_MODULE_PRODUCT_SETTINGS,
            MM_MODULE_CHECKOUT_SETTINGS,
            MM_MODULE_PAYMENT_SETTINGS,
            MM_MODULE_AFFILIATE_SETTINGS,
            MM_MODULE_EMAIL_SETTINGS,
            MM_MODULE_DEVELOPER_TOOLS,
            MM_MODULE_LOGS,
            MM_MODULE_WEBFORMS,
            MM_MODULE_GENERAL_SETTINGS,
            MM_MODULE_REPORTING,
            MM_MODULE_SUPPORT
        );
        return in_array($page, $validPages);
    }
}
