<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * The object cache is meant to be a wrapper around an in-memory per-process cache 
 * 
 */
class MM_ObjectCache
{
    public static $KEY_LAST_FORM = "last-form";

    protected static $WP_CACHE_GROUP = "membermouse-cache";

    protected static $cacheData = array();


    /**
     * Retrieve a value for the supplied key
     * 
     * @param string $key The key to retrieve
     * @param boolean $exists If the key exists in the cache; used to disambiguate null return values. Variable is passed in by reference
     * 
     * @return mixed The stored value, null if not found
     */
    public static function get($key, &$exists = null)
    {
        if (isset(self::$cacheData[$key])) {
            $exists = true;
            return self::$cacheData[$key];
        } else {
            $exists = false;
            return null;
        }
    }


    /**
     * Store a value for the supplied key
     * 
     * @param string $key
     * @param mixed $value
     * 
     * @return boolean true on success, false on failure. false most likely indicates a failure of the underlying caching mechanism
     */
    public static function set($key, $value)
    {
        self::$cacheData[$key] = $value;
        return true;
    }


    /**
     * Delete a value with the associated key from the cache
     * 
     * @param string $key
     * 
     * @return boolean true on success, false on failure. false most likely indicates a failure of the underlying caching mechanism
     */
    public static function delete($key = "")
    {
        unset(self::$cacheData[$key]);
        return true;
    }


    /**
     * Delete all cached values from the cache
     * 
     * @return boolean true on success, false on failure. false most likely indicates a failure of the underlying caching mechanism
     */
    public static function flush()
    {
        self::$cacheData = array();
    }
}
