<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_AppliedBundlesView extends MM_RBACGeneralEmployeeView
{
    protected $headers = [];
    protected $renderForAdmin = true;


    /**
     * Constructor
     */
    public function __construct()
    {
        parent::__construct();
    }


    /**
     * AJAX Handler. Currently no functions defined.
     * @see    MM_View::performAction()
     * @param  array $post
     * @return mixed Response, generally an MM_Response object but parent can return other values
     */
    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }


    /**
     * Sets whether to render the grid for admin
     * @param boolean True to render for admin, false to render for the member
     */
    public function setRenderForAdmin($renderForAdmin): void
    {
        $this->renderForAdmin = (bool)$renderForAdmin;
    }


    /**
     * Retrieves column header data, for use by the grid.
     * @return array An associative array of data used by the grid. The key is the element ID, and the value is an
     *               associative array with keys "content" which is a string, and optionally "attr" which is a string
     *               with additional HTML attribute modifiers, ie 'style="color:red;"'
     */
    public function getHeaders(): array
    {
        return $this->headers;
    }


    /**
     * Generates column headers. Must be called after setRenderForAdmin() to accurately provide the appropriate
     * headers. This function sets a property of the View, and returns nothing. Header structure:
     *     An associative array of data used by the grid. The key is the element ID, and the value is an
     *     associative array with keys "content" which is a string, and optionally "attr" which is a string
     *     with additional HTML attribute modifiers, ie 'style="color:red;"'
     */
    public function generateHeaders(): void
    {
        if ($this->renderForAdmin) {
            $daysColumnTitle = __('Days with Bundle', 'membermouse');
            $daysColumnText = __('Days...', 'membermouse');

            $headers = [
                'bundle_name'  => ['content' => __("Bundle", "membermouse")],
                'status'       => ['content' => __("Status", "membermouse")],
                'days'         => ['content' => "<span title='{$daysColumnTitle}'>{$daysColumnText}</span>"],
                'apply_date'   => ['content' => __("First Applied", "membermouse")],
                'expire_date'  => ['content' => __("Expires On", "membermouse")],
                'cancel_date'  => ['content' => __("Cancels On", "membermouse")],
                'actions'      => ['content' => __("Actions", "membermouse")],
            ];
        } else {
            $headers = [
                'apply_date'   => ['content' => __("Start Date", "membermouse")],
                'bundle_name'  => ['content' => __("Description", "membermouse")],
                'status'       => ['content' => __("Status", "membermouse")],
                'pending_date' => ['content' => __("Access Until", "membermouse")],
                'actions'      => ['content' => __("Action", "membermouse"), 'attr' => 'style="text-align:center;"'],
            ];
        }

        $this->headers = $headers;
    }


    /**
     * Retrieve data used to construct the rows
     * @param integer $userId The ID of the member whose bundle data will be retrieved
     * @param MM_DataGrid $dg This MM_DataGrid contains sort and limit information
     * @return array An ordered array of stdClass objects, each representing one bundle
     */
    public function getViewData($userId, MM_DataGrid $dg = null)
    {
        global $wpdb;

        $data = [];
        $user = new MM_User($userId);

        if (!$user->isValid()) {
            return $data;
        }

        $levelId = $user->getMembershipId();

        $sql  = "SELECT ab.* FROM " . MM_TABLE_APPLIED_BUNDLES . " ab ";
        $sql .= "INNER JOIN " . MM_TABLE_BUNDLES . " b ON (ab.bundle_id = b.id) ";
        $sql .= "WHERE ((access_type_id = '{$userId}') AND (access_type = '" . MM_AppliedBundle::$ACCESS_TYPE_USER . "')) ";
        $sql .= "OR ((access_type_id = '{$levelId}') AND (access_type = '" . MM_AppliedBundle::$ACCESS_TYPE_MEMBERSHIP . "')) ";

        if (($dg !== null) && ($dg->sortBy !== null) && (!empty($dg->sortBy))) {
            $sql .= "ORDER BY {$dg->sortBy} {$dg->sortDir} ";
        }

        $results = $wpdb->get_results($sql);

        foreach ($results as $rowData) {
            $bundle = new MM_Bundle($rowData->bundle_id);

            if ($bundle->isValid()) {
                $rowObj = (object) [
                    "user_id"           => $userId,
                    "type"              => $rowData->access_type,
                    "bundle_id"         => $bundle->getId(),
                    "bundle_name"       => $bundle->getName(),
                    "status"            => null,
                    "complimentary"     => false,
                    "imported"          => false,
                    "days"              => null,
                    "apply_date"        => null,
                    "does_expire"       => false,
                    "expire_date"       => null,
                    "pending_cancel"    => false,
                    "cancel_date"       => null,
                ];

                if ($rowData->access_type == MM_AppliedBundle::$ACCESS_TYPE_USER) {
                    $appliedBundle = new MM_AppliedBundle();
                    $appliedBundle->setData($rowData);

                    if ($appliedBundle->isValid()) {
                        $doesExpire = $appliedBundle->doesExpire();
                        $pendingCancel = $appliedBundle->isPendingCancellation();

                        $rowObj->status = $appliedBundle->getStatus();
                        $rowObj->complimentary = $appliedBundle->isComplimentary();
                        $rowObj->imported = $appliedBundle->isImported();
                        $rowObj->days = $appliedBundle->getDaysWithBundle();
                        $rowObj->apply_date = $appliedBundle->getApplyDate();
                        $rowObj->does_expire = $doesExpire;
                        $rowObj->expire_date = ($doesExpire) ? $appliedBundle->getExpirationDate() : null;
                        $rowObj->pending_cancel = $pendingCancel;
                        $rowObj->cancel_date = ($pendingCancel) ? $appliedBundle->getCancellationDate() : null;
                    }
                }

                $data[] = $rowObj;
            }
        }
        return $data;
    }


    /**
     * Generate the rows
     * @param array $data An array of stdClass objects generated by getViewData
     * @return array An array of rows in a format that MM_DataGrid can use to render HTML
     */
    public function generateRows($data)
    {
        $rows = [];
        $headers = $this->headers;
        $dateFormat = ($this->renderForAdmin) ? "M j, Y g:i a" : "M j, Y";

        foreach ($data as $item) {
            $currentRow = [];
            foreach ($headers as $columnId => $columnData) {
                switch ($columnId) {
                    case "bundle_name":
                        $content = MM_Utils::abbrevString($item->bundle_name);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "status":
                        $content = $this->generateStatusInfo($item);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "days":
                        $content = $this->generateDaysWithBundleInfo($item);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "apply_date":
                        $content = $this->generateApplyDateInfo($item, $dateFormat);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "expire_date":
                        $content = $this->generateExpireDateInfo($item, $dateFormat);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "cancel_date":
                        $content = $this->generateCancelDateInfo($item, $dateFormat);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "pending_date":
                        $content = $this->generatePendingDateInfo($item, $dateFormat);
                        $currentRow[] = ["content" => $content];
                        break;
                    case "actions":
                        $content = $this->generateActionsInfo($item);
                        // Align status icon to column center on My Account
                        $currentRow[] = ($this->renderForAdmin) ? ["content" => $content] :
                            ["content" => $content, 'attr' => 'style="text-align:center;"'];
                        break;
                    default:
                        $content = "";
                        $currentRow[] = ["content" => $content];
                        break;
                }
            }
            $rows[] = $currentRow;
        }

        return $rows;
    }


    /**
     * Generates the bundle status indicator
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generateStatusInfo($item)
    {
        $status = "";

        if ($item->type == MM_AppliedBundle::$ACCESS_TYPE_USER) {
            $status = MM_Status::getImage($item->status);

            // On the My Account page, display the status name after the icon
            if (!$this->renderForAdmin) {
                $status .= "&nbsp;" . MM_Status::getName($item->status);
            }

            if ($item->complimentary) {
                $tooltip = __("Bundle is complimentary", "membermouse");
                $status .= " " . MM_Utils::getIcon('ticket', 'purple', '1.3em', '2px', $tooltip, "margin-left:4px;");
            }

            if ($item->imported) {
                $tooltip = __("Bundle applied through import", "membermouse");
                $status .= " " . MM_Utils::getIcon('sign-in', 'blue', '1.3em', '2px', $tooltip, "margin-left:4px;");
            }
        } elseif ($item->type == MM_AppliedBundle::$ACCESS_TYPE_MEMBERSHIP) {
            $tooltip = __("Bundle applied through membership", "membermouse");
            $status = MM_Utils::getIcon('user', 'blue', '1.3em', '2px', $tooltip, "padding-right:1px;padding-left:1px;");

            // On the My Account Page add 'From Membership' after the icon
            if (!$this->renderForAdmin) {
                $status .= "&nbsp;" . __("From Membership", "membermouse");
            }
        }

        return $status;
    }


    /**
     * Generates the Days with Bundle content
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generateDaysWithBundleInfo($item)
    {
        if ($item->type == MM_AppliedBundle::$ACCESS_TYPE_USER) {
            return sprintf('<span style="font-family:courier;">%s</span>', number_format($item->days));
        } else {
            return MM_NO_DATA;
        }
    }


    /**
     * Generates the content displaying the date the bundle was applied, if any. Used only on the admin grid.
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generateApplyDateInfo($item, $dateFormat)
    {
        return (isset($item->apply_date)) ? MM_Utils::dateToLocal($item->apply_date, $dateFormat) : MM_NO_DATA;
    }


    /**
     * Generates the content for the expiry date of the bundle, if any. Used only in the admin grid.
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generateExpireDateInfo($item, $dateFormat)
    {
        return (isset($item->expire_date)) ? MM_Utils::dateToLocal($item->expire_date, $dateFormat) : MM_NO_DATA;
    }


    /**
     * Generates the content for the cancellation date of the bundle, if any. Used only in the admin grid.
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generateCancelDateInfo($item, $dateFormat)
    {
        return (isset($item->cancel_date)) ? MM_Utils::dateToLocal($item->cancel_date, $dateFormat) : MM_NO_DATA;
    }


    /**
     * Generates content representing the cancellation or expiration date, whichever is soonest, and indicates
     * what status change will occur. Used only on the My Account page.
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generatePendingDateInfo($item, $dateFormat)
    {
        if (($item->cancel_date === null) && ($item->expire_date === null)) {
            return MM_NO_DATA;
        } elseif (($item->cancel_date !== null) && ($item->expire_date !== null)) {
            if (strtotime($item->cancel_date) < strtotime($item->expire_date)) {
                return sprintf(
                    __("Cancels %s", "membermouse"),
                    MM_Utils::dateToLocal($item->cancel_date, $dateFormat)
                );
            } else {
                return sprintf(
                    __("Expires %s", "membermouse"),
                    MM_Utils::dateToLocal($item->expire_date, $dateFormat)
                );
            }
        } elseif ($item->cancel_date !== null) {
            return sprintf(
                __("Cancels %s", "membermouse"),
                MM_Utils::dateToLocal($item->cancel_date, $dateFormat)
            );
        } else {
            return sprintf(
                __("Expires %s", "membermouse"),
                MM_Utils::dateToLocal($item->expire_date, $dateFormat)
            );
        }
    }


    /**
     * Generates content for the Actions column.
     * @param object $item An stdClass object containing data about the bundle and user
     * @return string HTML content that will be placed in a cell of the table
     */
    protected function generateActionsInfo($item)
    {
        $status = $item->status;
        $actions = "";

        if ($this->renderForAdmin)   // When displayed in the Admin Panel
        {
            $showCancel = false;
            $showPause = false;
            $showActivate = false;
            $showEditCalc = false;

            if ($item->type == MM_AppliedBundle::$ACCESS_TYPE_USER) {
                if ($status == MM_Status::$ACTIVE || $status == MM_Status::$EXPIRED || $status == MM_Status::$PAUSED) {
                    $showCancel = true;
                    $showEditCalc = true;

                    if ($status == MM_Status::$ACTIVE || $status == MM_Status::$EXPIRED) {
                        $showPause = true;
                    } elseif ($status == MM_Status::$PAUSED) {
                        $showActivate = true;
                    }
                } elseif ($status == MM_Status::$OVERDUE) {
                    $showCancel = true;
                    $showPause = true;
                } elseif ($status == MM_Status::$PENDING_CANCELLATION) {
                    $showActivate = true;
                    $showEditCalc = true;
                } else {
                    $showActivate = true;
                }
            } else {
                $actions .= MM_NO_DATA;
            }

            if ($showActivate) {
                $handler = "mmjs.changeBundleStatus('{$item->user_id}', '{$item->bundle_id}', '" . MM_Status::$ACTIVE . "')";
                $actions .= sprintf(
                    '<a style="cursor: pointer;" onclick="%1$s" title="%2$s %3$s">%4$s</a>',
                    $handler,
                    __('Activate', 'membermouse'),
                    $item->bundle_name,
                    MM_Utils::getIcon('play-circle', 'green', '1.3em', '3px', '', 'margin-right:3px;'),
                );
            }

            if ($showCancel) {
                $handler = "mmjs.changeBundleStatus('{$item->user_id}', '{$item->bundle_id}', '" . MM_Status::$CANCELED . "')";
                $actions .= sprintf(
                    '<a style="cursor: pointer;" onclick="%1$s" title="%2$s %3$s">%4$s</a>',
                    $handler,
                    __('Cancel', 'membermouse'),
                    $item->bundle_name,
                    MM_Utils::getIcon('stop', 'red', '1.2em', '2px', '', 'margin-right:3px;'),
                );
            }

            if ($showPause) {
                $handler = "mmjs.changeBundleStatus('{$item->user_id}', '{$item->bundle_id}', '" . MM_Status::$PAUSED . "')";
                $actions .= sprintf(
                    '<a style="cursor: pointer;" onclick="%1$s" title="%2$s %3$s">%4$s</a>',
                    $handler,
                    __('Pause', 'membermouse'),
                    $item->bundle_name,
                    MM_Utils::getIcon('pause', 'turq', '1.2em', '2px', '', 'margin-right:3px;'),
                );
            }

            if ($showEditCalc) {
                $editActionUrl = 'onclick="mmjs.editBundleConfiguration(\'' . $item->user_id . '\', \'' . $item->bundle_id . '\')"';
                $actions .= MM_Utils::getEditIcon(__("Edit Bundle Configuration", "membermouse"), '', $editActionUrl);
            }
        } else  // When displayed on My Account Page
        {
            if ($status == MM_Status::$ACTIVE || $status == MM_Status::$OVERDUE) {
                $cancelMethod = MM_OptionUtils::getOption(MM_OptionUtils::$OPTION_KEY_DFLT_CANCELLATION_METHOD);

                if ($cancelMethod == MM_CancellationMethod::$CANCEL_PAUSE) {
                    $newStatus = MM_Status::$PAUSED;
                } else {
                    $newStatus = MM_Status::$CANCELED;
                }

                $handler = "pymtutils_js.changeBundleStatus('{$item->user_id}', '{$item->bundle_id}', '" . $newStatus . "')";
                $actions .= sprintf(
                    '<a style="cursor: pointer;" onclick="%1$s" title="%2$s %3$s">%4$s</a>',
                    $handler,
                    __('Cancel', 'membermouse'),
                    $item->bundle_name,
                    MM_Utils::getIcon('stop', 'red', '1.2em', '2px', '', 'margin-right:3px;'),
                );
            } else {
                $actions .= sprintf(
                    '<span title="%1$s">%2$s</span>',
                    __('Only active bundles can be cancelled.', 'membermouse'),
                    MM_Utils::getIcon('stop', 'lightgrey', '1.2em', '2px', '', 'margin-right:3px;')
                );
            }
        }

        return $actions;
    }
}
