<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_DataGrid
{
    private $headers = array();
    private $rows = array();
    public $width = "";
    public $totalRecords = 0;
    public $sortBy = "id";
    public $sortDir = "desc";
    public $crntPage = 0;
    public $resultSize = 20;
    public $recordName = "records";
    public $attributes = null;
    public $showPagingControls = true;

    public function __construct($post = null, $dfltSortBy = "", $dfltSortDir = "", $dfltResultSize = "", $attributes = null)
    {
        if ($dfltSortBy != "") {
            $this->sortBy = $dfltSortBy;
        }

        if ($dfltSortDir != "") {
            $this->sortDir = $dfltSortDir;
        }

        if ($dfltResultSize != "") {
            $this->resultSize = $dfltResultSize;
        }

        if (!is_null($attributes)) {
            $this->attributes = $attributes;
        }

        if (isset($post)) {
            $this->setProperties($post);
        }
    }

    public function setAttribute($arr)
    {
        $this->attributes = $arr;
    }

    public function setHeaders($arr)
    {
        $this->headers = $arr;
    }

    public function setRows($arr)
    {
        $this->rows = $arr;
    }

    public function setTotalRecords($data)
    {
        if ($data && is_array($data) && count($data) > 0) {
            $this->totalRecords = $data[0]->total;
        } else {
            $this->totalRecords = $data;
        }
    }

    public function setProperties($post)
    {
        $this->sortBy = (isset($post["sortBy"])) ? $post["sortBy"] : $this->sortBy;
        $this->sortDir = (isset($post["sortDir"])) ? $post["sortDir"] : $this->sortDir;
        $this->crntPage = (isset($post["crntPage"])) ? $post["crntPage"] : $this->crntPage;
        $this->resultSize = (isset($post["resultSize"])) ? $post["resultSize"] : $this->resultSize;
    }

    public function getLimitSql()
    {
        $sql = "";

        if (isset($this->crntPage) && isset($this->resultSize)) {
            $sql .= " LIMIT " . intval($this->crntPage) * intval($this->resultSize) . ", {$this->resultSize}";
        }

        return $sql;
    }

    public function generateHtml($customTemplate = "")
    {
        $html = "";

        if (!empty($this->headers) && !empty($this->rows) && count($this->rows) > 0) {
            // handle the case where sorting a result set from the wp_users table
            if (!isset($this->headers[$this->sortBy]) && $this->sortBy == "id") {
                $this->sortBy = "ID";
            }

            if (isset($this->headers[$this->sortBy]) && !empty($this->sortBy) && !is_null($this->sortBy)) {

                $sortImage = ($this->sortDir == 'asc') ? "caret-up" : "caret-down";

                if ((extension_loaded("mbstring") && (mb_substr($this->headers[$this->sortBy]['content'], -4, 4) === "</a>")) ||
                    (substr($this->headers[$this->sortBy]['content'], -4, 4) === "</a>")
                ) {
                    $this->headers[$this->sortBy]['content'] = str_replace('</a>', MM_Utils::getIcon($sortImage, 'grey', '1.3em', '2px', '', 'margin-left:4px;') . "</a>", $this->headers[$this->sortBy]['content']);
                } else {
                    $this->headers[$this->sortBy]['content'] .= MM_Utils::getIcon($sortImage, 'grey', '1.3em', '2px', '', 'margin-left:4px;');
                }
                $this->headers[$this->sortBy]['content'] = str_replace('<a', '<a class="selected"', $this->headers[$this->sortBy]['content']);
            }

            $dg = new stdClass();
            $dg->datagrid = new stdClass();
            $dg->datagrid->attr = 'id="mm-data-grid" class="widefat"';

            if (!empty($this->width)) {
                $dg->datagrid->attr .= ' style="width:' . $this->width . '"';
            }

            $dg->datagrid->headers = $this->headers;
            $dg->datagrid->rows = $this->rows;

            $dg->sortBy = $this->sortBy;
            $dg->sortDir = $this->sortDir;
            $dg->crntPage = $this->crntPage;
            $dg->resultSize = $this->resultSize;
            $dg->totalRecords = $this->totalRecords;
            $dg->attributes = json_encode($this->attributes);
            $dg->totalPages = ceil(intval($this->totalRecords) / intval($this->resultSize));
            $dg->recordName = MM_Utils::pluralize(intval($this->totalRecords), $this->recordName);

            $dg->showPagingControls = $this->showPagingControls;

            if (empty($customTemplate)) {
                $html = MM_Template::generate(MM_MODULES . "/datagrid.php", $dg);
            } else {
                $html = MM_Template::generate(MM_MODULES . "/" . $customTemplate, $dg);
            }
        }

        return $html;
    }
}
