<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_DiagnosticsView extends MM_RBACAdminView
{
    public static $MM_JSACTION_SET_DIAGNOSTICS_MODE = "setDiagnosticsMode";
    public static $MM_JSACTION_CLEAR_LOG = "clearLog";
    public static $MM_JSACTION_STORE_FILTER_STATE = "storeFilterState";
    public static $MM_JSACTION_RESET_FORM = "resetForm";
    public static $MM_JSACTION_FILTER = "filter";

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_RESET_FORM:
                    return $this->generateFilterCriteriaForm($post);
                    break;
                case self::$MM_JSACTION_SET_DIAGNOSTICS_MODE:
                    return $this->setDiagnosticsMode($post);
                    break;
                case self::$MM_JSACTION_CLEAR_LOG:
                    return $this->clearLog();
                    break;
                case self::$MM_JSACTION_STORE_FILTER_STATE:
                    return $this->storeFilterState($post);
                    break;
                case self::$MM_JSACTION_FILTER:
                    return $this->generateDataGrid($post);
                    break;
                default:
                    return new MM_Response($response, MM_Response::$SUCCESS, true);
            }
        } else {
            return $response;
        }
    }


    public function getViewData($sortBy = null, $sortDir = null)
    {
        return parent::getData(MM_TABLE_DIAGNOSTIC_LOG, null, $sortBy, $sortDir);
    }


    protected function setDiagnosticsMode($post)
    {
        if (isset($post["newMode"]) && !empty($post["newMode"])) {
            MM_DiagnosticLog::setMode($post["newMode"]);
            return new MM_Response("Diagnostic mode changed", MM_Response::$SUCCESS, true);
        }

        return new MM_Response("Diagnostic mode setting was unchanged", MM_Response::$SUCCESS, true);
    }


    protected function clearLog()
    {
        if (MM_DiagnosticLog::clearLog()) {
            return new MM_Response("Diagnostic Log cleared successfully", MM_Response::$SUCCESS, true);
        } else {
            return new MM_Response("Error clearing Diagnostic Log", MM_Response::$ERROR, true);
        }
    }


    protected function storeFilterState($post)
    {
        if (isset($post["mm_show_filters"]) && isset($post["mm_admin_id"])) {
            $optionName = MM_OptionUtils::$OPTION_KEY_SHOW_DIAGNOSTICS_LOG_FILTERS . "-" . $post["mm_admin_id"];
            MM_OptionUtils::setOption($optionName, $post["mm_show_filters"]);
        }
        return new MM_Response();
    }


    public function generateFilterCriteriaForm($post = null)
    {
        return MM_TEMPLATE::generate(MM_MODULES . "/diagnostics.form.php", $post);
    }


    public function filter($post, MM_DataGrid $dg)
    {
        global $wpdb;

        $whereConditions = array();
        $whereSql = "";
        $orderBySql = "";

        if (!empty($post["mm_event_types"]) && is_array($post['mm_event_types'])) {
            $eventTypes = array();
            foreach ($post['mm_event_types'] as $v) {
                $eventTypes[] = $wpdb->prepare("'%s'", $v);
            }
            $whereConditions[] = "type IN (" . implode(",", $eventTypes) . ")";
        }

        if (!empty($post['mm_from_date'])) {
            $fromDate = date("Y-m-d H:i:s", strtotime($post['mm_from_date']));
            $whereConditions[] = "event_date >= '{$fromDate}'";
        }

        if (!empty($post['mm_to_date'])) {
            $toDate = date("Y-m-d H:i:s", strtotime($post['mm_to_date']));
            $whereConditions[] = "event_date <= '{$toDate}'";
        }

        if (!empty($post['mm_line_number'])) {
            $whereConditions[] = $wpdb->prepare("line = %d", $post['mm_line_number']);
        }

        $stringFilters = array(
            "ip_address" => "mm_ip_address",
            "session"    => "mm_diagnostic_session_id",
            "location"   => "mm_event_location",
            "event_data" => "mm_event_data"
        );
        foreach ($stringFilters as $k => $v) {
            if (!empty($post[$v])) {
                $whereConditions[] = $wpdb->prepare("{$k} LIKE '%%%s%%'", $post[$v]);
            }
        }

        if (isset($dg->sortBy) && !is_null($dg->sortBy) && !empty($dg->sortBy)) {
            $orderBySql = " ORDER BY {$dg->sortBy} {$dg->sortDir} ";
        }

        if (count($whereConditions)) {
            $whereSql = " WHERE " . array_shift($whereConditions);
            if (count($whereConditions)) {
                $whereSql .= " AND " . implode(" AND ", $whereConditions);
            }
        }

        $sqlResultCount = "SELECT COUNT(*) AS total FROM " . MM_TABLE_DIAGNOSTIC_LOG;
        $sqlResultCount .= $whereSql;
        $countRow = $wpdb->get_row($sqlResultCount);
        if ($countRow->total <= 0) {
            return array();
        }

        $sql = "select *, '{$countRow->total}' as total FROM " . MM_TABLE_DIAGNOSTIC_LOG;
        $sql .= $whereSql;
        $sql .= $orderBySql;

        $sql .= $dg->getLimitSql();
        $rows = $wpdb->get_results($sql, OBJECT_K);
        return array_values($rows);
    }


    public function generateDataGrid($post = null)
    {
        return MM_TEMPLATE::generate(MM_MODULES . "/diagnostics.datagrid.php", $post);
    }
}
