<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_DuplicateSubscriptionsView extends MM_RBACAdminView
{
    public static $MM_JSACTION_SEARCH = "search";
    public static $MM_JSACTION_CANCEL_SUBSCRIPTION = "cancelSubscription";

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_SEARCH:
                    return $this->generateDataGrid($post);
                case self::$MM_JSACTION_CANCEL_SUBSCRIPTION:
                    return $this->cancelDuplicateSubscription($post);
                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }

    public function search($post, MM_DataGrid $dg)
    {
        global $wpdb;

        $oit = MM_TABLE_ORDER_ITEMS;
        $ot = MM_TABLE_ORDERS;
        $ut = $wpdb->users;
        $udt = MM_TABLE_USER_DATA;

        $sql = "SELECT oi.id, mu.wp_user_id as user_id, CONCAT(mu.first_name,' ',mu.last_name) AS member_name, u.user_email, " .
            "oi.description AS product, o.order_number, o.date_added,";

        $sqlFrom =  "FROM {$oit} oi LEFT JOIN {$ot} o ON (oi.order_id = o.id) INNER JOIN " .
            "(SELECT COUNT(fo.user_id) AS freq ,fo.user_id, foi.item_id FROM {$oit} foi LEFT JOIN {$ot} fo " .
            "ON (foi.order_id = fo.id) WHERE foi.is_recurring=1 AND foi.item_type=%s AND foi.status=%s " .
            "GROUP BY fo.user_id,foi.item_id having freq > 1) AS ft ON ((oi.item_id = ft.item_id) AND " .
            "(o.user_id = ft.user_id)) LEFT JOIN {$ut} u ON (u.id = o.user_id)  LEFT JOIN {$udt} mu ON " .
            "(mu.wp_user_id = o.user_id) WHERE oi.is_recurring=1 " .
            "AND oi.item_type=%s AND oi.status=%s ORDER BY o.user_id desc";

        $sqlFrom = $wpdb->prepare($sqlFrom, MM_OrderItem::$STATUS_RECURRING, MM_OrderItem::$ORDER_ITEM_TYPE_PRODUCT, MM_OrderItem::$STATUS_RECURRING, MM_OrderItem::$ORDER_ITEM_TYPE_PRODUCT);

        $sqlResultCount = "select count(*) as total ";
        $sqlResultCount .= $sqlFrom;
        $countRow = $wpdb->get_row($sqlResultCount);

        if ($countRow->total > 0) {
            $sql .= " {$countRow->total} as total {$sqlFrom}";
        } else {
            return array();
        }

        $sql .= ",{$dg->sortBy} {$dg->sortDir} ";
        $sql .= $dg->getLimitSql();

        $rows = $wpdb->get_results($sql, OBJECT_K);
        return array_values($rows);
    }


    public function generateDataGrid($post = null)
    {
        return MM_TEMPLATE::generate(MM_MODULES . "/duplicate_subscriptions.datagrid.php", $post);
    }


    public function cancelDuplicateSubscription($post)
    {
        $orderItem = new MM_OrderItem($post['orderItemId']);
        if ($orderItem->isValid()) {
            $order = new MM_Order($orderItem->getOrderId());
            if ($order->isValid()) {
                $paymentService = $order->getPaymentMethod();
                $cancelResult = $paymentService->cancelSubscription($orderItem);
                if (MM_PaymentServiceResponse::isSuccess($cancelResult)) {
                    return new MM_Response();
                } else {
                    return new MM_Response($cancelResult->getMessage(), MM_Response::$ERROR);
                }
            } else {
                return new MM_Response("Unable to cancel subscription: Invalid Order", MM_Response::$ERROR);
            }
        } else {
            return new MM_Response("Unable to cancel subscription: Subscription is invalid", MM_Response::$ERROR);
        }
    }
}
