<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_EmployeesView extends MM_RBACAdminView
{
    public static $MM_JSACTION_SET_DEFAULT = "setDefault";

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_SAVE:
                    return $this->saveEmployee($post);

                case self::$MM_JSACTION_REMOVE:
                    return $this->removeEmployee($post);

                case self::$MM_JSACTION_SET_DEFAULT:
                    return $this->setAsDefault($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }

    public function getViewData($sortBy = null, $sortDir = null)
    {
        return parent::getData(MM_TABLE_EMPLOYEE_ACCOUNTS, null, $sortBy, $sortDir);
    }

    private function saveEmployee($post)
    {
        $employee = new MM_Employee();

        if (isset($post["id"]) && intval($post["id"]) > 0) {
            $employee->setId($post["id"]);
            $employee->getData();
        }

        $employee->setDisplayName($post["mm_display_name"]);
        $employee->setFirstName($post["mm_first_name"]);
        $employee->setLastName($post["mm_last_name"]);

        if (isset($post["mm_password"])) {
            $employee->setPassword($post["mm_password"]);
        }

        $employee->setRoleId($post["mm_role_id"]);
        $employee->setPhone($post["mm_phone"]);
        $employee->setEmail($post["mm_email"]);
        $employee->setAllowExport($post["mm_allow_export_val"]);
        $employee->setIsDefault($post["mm_is_default"]);

        if (MM_MemberMouseService::hasPermission(MM_MemberMouseService::$FEATURE_EMPLOYEE_ACCOUNTS) && !empty($post['mm_memberships'])) {
            $employee->setAccessRestrictions(MM_Employee::$ACCESS_TYPE_MEMBERSHIP, $post["mm_memberships"]);
        } else {
            $employee->setAccessRestrictions(MM_Employee::$ACCESS_TYPE_MEMBERSHIP, array());
        }

        $result = $employee->commitData();

        if (MM_Response::isError($result)) {
            return $result;
        }

        return new MM_Response();
    }

    private function removeEmployee($post)
    {
        global $wpdb;

        if (isset($post["id"]) && intval($post["id"]) > 0) {
            $removeLinked = (isset($post['remove_linked']) && ($post['remove_linked'] == "true")) ? true : false;
            $employee = new MM_Employee($post["id"]);
            $result = $employee->delete($removeLinked);

            if ($result) {
                return new MM_Response();
            } else {
                return new MM_Response(__("This employee has existing associations and can't be removed.", "membermouse"), MM_Response::$ERROR);
            }
        }

        return new MM_Response("Unable to delete employee. No ID specified.", MM_Response::$ERROR);
    }

    private function setAsDefault($post)
    {
        global $wpdb;

        if (isset($post["id"]) && intval($post["id"]) > 0) {
            // clear current default account
            $sql = "update " . MM_TABLE_EMPLOYEE_ACCOUNTS . " set is_default='0' where is_default='1'";
            $wpdb->query($sql);

            // set new default account
            $sql = "update " . MM_TABLE_EMPLOYEE_ACCOUNTS . " set is_default='1' where id='%d' limit 1";
            $results = $wpdb->query($wpdb->prepare($sql, $post["id"]));

            if ($results) {
                return new MM_Response();
            }
        }

        return new MM_Response("Unable to set employee as default. No ID specified.", MM_Response::$ERROR);
    }
}
