<?php
class MM_GiftView extends MM_RBACProductManagerView
{
    public static $MM_JSACTION_STORE_SEARCH_OPTIONS = "storeSearchOptions";

    public static $SEARCH_OPTION_GIFT_STATUS = "mm-option-gift-status";

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_STORE_SEARCH_OPTIONS:
                    return $this->storeSearchOptions($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }

    public static function hasGifts()
    {
        global $wpdb;

        $sql = "SELECT count(*) as total FROM " . MM_TABLE_COUPONS . " WHERE is_gift = '1';";

        $row = $wpdb->get_row($sql);

        if (is_null($row)) {
            return false;
        }

        return ($row->total > 0);
    }


    public function getViewData($dg)
    {
        global $wpdb;
        global $current_user;

        if (isset($current_user->ID)) {
            $giftStatus = MM_OptionUtils::getOption(self::$SEARCH_OPTION_GIFT_STATUS . "-" . $current_user->ID);
        }

        //tables used in the master query. Placing the constants into variables allows them to be embedded into the string, aiding in readability
        $couponsTable = MM_TABLE_COUPONS;
        $orderItemsTable = MM_TABLE_ORDER_ITEMS;
        $couponUsageTable = MM_TABLE_COUPON_USAGE;
        $productsTable = MM_TABLE_PRODUCTS;

        $fromWhereSQL = "FROM {$couponsTable} c LEFT JOIN {$orderItemsTable} oi ON (c.gift_order_item_id = oi.id) " .
            "LEFT JOIN {$couponUsageTable} cu on (c.id = cu.coupon_id) left join {$productsTable} p on (oi.item_id = p.id) " .
            "WHERE p.id IS NOT NULL ";

        switch ($giftStatus) {
            case "redeemed":
                $fromWhereSQL .= "AND cu.user_id IS NOT NULL AND cu.date_added IS NOT NULL AND c.is_gift=1";
                break;

            case "pending":
                $fromWhereSQL .= "AND cu.user_id IS NULL AND cu.date_added IS NULL AND c.is_gift=1";
                break;
        }

        $countQuery = "select count(*) as total {$fromWhereSQL} ";
        $countResult = $wpdb->get_row($countQuery);

        $masterQuery = "SELECT {$countResult->total} as 'total',c.date_added as date_gifted, c.gift_order_item_id as order_item_id, p.id as product_id, p.name, cu.date_added as usage_date, " .
            "cu.user_id as usage_user_id {$fromWhereSQL}";

        if (!is_null($dg)) {
            if (!is_null($dg->sortBy) && !empty($dg->sortBy)) //handle the sorting
            {
                $masterQuery .= " ORDER BY {$dg->sortBy} {$dg->sortDir}";
            }

            $masterQuery .= " " . $dg->getLimitSql(); //for pagination
        }

        $results = $wpdb->get_results($masterQuery);
        return $results;
    }

    private function storeSearchOptions($post)
    {
        if (isset($post["mm_admin_id"])) {
            if (isset($post["mm_gift_status"])) {
                $optionName = self::$SEARCH_OPTION_GIFT_STATUS . "-" . $post["mm_admin_id"];
                MM_OptionUtils::setOption($optionName, $post["mm_gift_status"]);
            }

            return new MM_Response();
        } else {
            return new MM_Response(__("Unable to store gift search options. Couldn't find current admin user ID.", "membermouse"), MM_Response::$ERROR);
        }
    }
}
