<?php

/**
 * 
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_JobsQueueLogView extends MM_RBACGeneralEmployeeView
{
    public static $MM_JSACTION_SEARCH = "search";
    public static $MM_JSACTION_STORE_JOB_STATE_SEARCH_OPTIONS = "storeJobStateSearchOptions";
    public static $SEARCH_OPTION_SHOW_FAILED_JOBS = "mm-option-show-failed-jobs";

    use MM_EnhancedDatagridView;

    protected $defaultSortCol = "id DESC";


    public function __construct()
    {
        parent::__construct();
    }


    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_SEARCH:
                    return $this->search($post);

                case self::$MM_JSACTION_STORE_JOB_STATE_SEARCH_OPTIONS:
                    return $this->storeJobStateSearchOptions($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }


    protected function storeJobStateSearchOptions($post)
    {
        if (isset($post["mm_admin_id"])) {
            if (isset($post["mm_show_failed_jobs"])) {
                $optionName = self::$SEARCH_OPTION_SHOW_FAILED_JOBS . "-" . $post["mm_admin_id"];
                MM_OptionUtils::setOption($optionName, (bool)($post["mm_show_failed_jobs"]));
            }

            return new MM_Response();
        } else {
            return new MM_Response("Unable to store job queue search options. Couldn't find current admin user ID.", MM_Response::$ERROR);
        }
    }


    public function preprocessSearch($queryMeta)
    {
        global $current_user;

        $queryMeta->query->supplemental = [];
        if (isset($current_user->ID)) {
            $showFailed = (MM_OptionUtils::getOption(self::$SEARCH_OPTION_SHOW_FAILED_JOBS . "-" . $current_user->ID) == "1") ? true : false;

            if ($showFailed) {
                $queryMeta->query->supplemental["showFailed"] = true;
            }
        }
    }


    /**
     * Query the database and return the unprocessed results in object format
     */
    protected function getRawData($criteria = array())
    {
        global $wpdb;

        $dataset = $this->processQueryMeta($criteria);
        $whereString = $dataset->whereString;
        $orderString = $dataset->orderString;
        $limitString = $dataset->limitString;

        $showFailed = $criteria->query->supplemental["showFailed"] ?? false;
        if ($showFailed == true) {
            $jobsTable = \membermouse\resque\Resque::failed_jobs_table_name();
        } else {
            $jobsTable = \membermouse\resque\Resque::completed_jobs_table_name();
        }

        $sqlResultCount = "SELECT COUNT(*) AS total FROM {$jobsTable} {$whereString}";
        $dataset->total = $wpdb->get_var($sqlResultCount);

        $sql = "SELECT id,runtime,firstrun,lastrun,priority,tries,reason,status,created_at " .
            "FROM {$jobsTable} " .
            "{$whereString}{$orderString}{$limitString}";
        $sql = trim($sql);
        $rows = $wpdb->get_results($sql);

        $pageCounter = ($dataset->pageNum == -1) ? 1 : $dataset->pageNum; //TODO: implement -1 as a marker for unpaginated
        $dataset->data[$pageCounter] = [];
        $prefetchSize = ($dataset->prefetchSize < 0) ? 0 : $dataset->prefetchSize;
        $pfCount = -1;

        foreach (array_chunk($rows, $dataset->pageSize) as $k => $page) {
            if ($pfCount >= $prefetchSize) {
                break;
            }
            $pageCounter = $dataset->pageNum + $k; //by default array_chunk will use numeric keys starting wih 0
            $dataset->data[$pageCounter] = $page;
            $pfCount++;
        }

        return $dataset;
    }
}
