<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_LoginFormView extends MM_View
{
    public static $MM_NONCE_ACTION_LOGIN_FORM = "mm-login";
    public static $MM_NONCE_NAME_LOGIN_FORM   = "mm-security";

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_LOGIN:
                    return $this->login($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }

    public function login($post)
    {
        try {
            $_POST['rememberme'] = ($_POST['rememberme'] == "true");

            $useWPNonceValidation = (MM_OptionUtils::getOption(MM_OptionUtils::$OPTION_KEY_USE_WP_NONCE_VALIDATION) == "1") ? true : false;
            if ($useWPNonceValidation) {
                if (!isset($_POST[MM_LoginFormView::$MM_NONCE_NAME_LOGIN_FORM])) {
                    throw new Exception("Invalid request parameters.");
                }

                $nonceResult = wp_verify_nonce($_POST[MM_LoginFormView::$MM_NONCE_NAME_LOGIN_FORM], MM_LoginFormView::$MM_NONCE_ACTION_LOGIN_FORM);
                if ($nonceResult === false) {
                    throw new Exception("Invalid nonce.");
                }
            }

            $user = wp_signon();
            $redirect_to = apply_filters('login_redirect', '', '', $user);

            if (is_wp_error($user)) {
                $ErrorMessageInfo = MM_Messages::get(MM_Session::$KEY_ERRORS);
                throw new Exception($ErrorMessageInfo[count($ErrorMessageInfo) - 1]);
            }

            $MM_Response = new MM_Response();
            $MM_Response->data['redirect_to'] = $redirect_to;

            return $MM_Response;
        } catch (Exception $e) {
            $MM_Response = new MM_Response($e->getMessage(), MM_Response::$ERROR);

            if (MM_Session::value('redirect_to')) {
                $MM_Response->data['redirect_to'] = MM_Session::value('redirect_to');
                MM_Session::clear('redirect_to');
            }

            return $MM_Response;
        }
    }


    /**
     * Override of parent to allow users without roles to login
     * {@inheritDoc}
     * @see MM_View::canCallMethod()
     */

    public function canCallMethod($request)
    {
        $allowedActions = array(self::$MM_JSACTION_LOGIN);
        if (($request['method'] != "performAction") || (!isset($request['mm_action'])) || (!in_array($request['mm_action'], $allowedActions))) {
            return false;
        } else {
            return true;
        }
    }
}
