<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */

class MM_MigrateDBView extends MM_RBACAdminView
{
    public static $MM_JSACTION_RENAME_TABLES = "renameTables";
    public static $MM_JSACTION_CHANGE_COLLATION = "changeCollation";

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_RENAME_TABLES:
                    return $this->renameTables();
                    break;
                case self::$MM_JSACTION_CHANGE_COLLATION:
                    return $this->changeCollation($post);
                    break;
                default:
                    return new MM_Response($response, MM_Response::$SUCCESS, true);
            }
        } else {
            return $response;
        }
    }


    /**
     * Rename all of the required and optional MemberMouse tables to use the wordpress database prefix
     * 
     * @return MM_Response indicates whether the table conversion succeeded or failed
     */
    public function renameTables()
    {
        global $wpdb;

        set_time_limit(300); //5 mins

        if (mmShouldUseLegacyDB()) {
            //unprefixed table names have been detected, attempt to rename them

            $tableExists = function ($tableName) use ($wpdb) {
                $query = $wpdb->prepare('SHOW TABLES LIKE %s', $wpdb->esc_like($tableName));
                return ($wpdb->get_var($query) == $tableName);
            };

            $requiredTables = MM_Install::getRequired2xTableNames();
            $res = $wpdb->query("START TRANSACTION;");
            foreach ($requiredTables as $tableName) {
                $res = $wpdb->query("ALTER TABLE {$tableName} RENAME {$wpdb->prefix}{$tableName};");
                if ($res === false) {
                    $error = $wpdb->last_error;
                    $error = empty($error) ? "" : ": {$error}";
                    $wpdb->query("ROLLBACK;");
                    return new MM_Response("Error renaming table {$tableName}{$error}", MM_Response::$ERROR);
                }
            }

            $fullTableList = $requiredTables;

            //now rename tables that conditionally exist
            $conditionalTables = MM_Install::getConditional2xTableNames();
            foreach ($conditionalTables as $tableName) {
                if ($tableExists($tableName)) {
                    $fullTableList[] = $tableName;
                    $res = $wpdb->query("ALTER TABLE {$tableName} RENAME {$wpdb->prefix}{$tableName};");
                    if ($res === false) {
                        $error = $wpdb->last_error;
                        $error = empty($error) ? "" : ": {$error}";
                        $wpdb->query("ROLLBACK;");
                        return new MM_Response("Error renaming table {$tableName}{$error}", MM_Response::$ERROR);
                    }
                }
            }
            $wpdb->query("COMMIT;");
            update_option("mm-option-use-legacy-db", false);
            remove_action('shutdown', array('MM_Session', 'sessionWrite')); //temporarily disable the session to prevent errors
            return new MM_Response($fullTableList);
        }
        return new MM_Response(__("Upgrade has already been completed.", "membermouse"), MM_Response::$ERROR);
    }


    /**
     * Change the collation (maybe) of a database table to utf8mb4
     * 
     * @param array $post The request
     * @return MM_Response indicates whether the table conversion succeeded or failed
     */
    public function changeCollation($post)
    {
        global $wpdb;

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        set_time_limit(300); //5 mins

        if ($post['tableName'] ?? false) {
            $tableName = $post['tableName'];

            $fullTableList = array_merge(MM_Install::getRequired2xTableNames(), MM_Install::getConditional2xTableNames());
            if (in_array($tableName, $fullTableList)) {
                //do the conversion
                if ($tableName == MM_TABLE_EVENT_LOG) {
                    $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}{$tableName}"); //clear the activity log to speed up the conversion
                }
                maybe_convert_table_to_utf8mb4("{$wpdb->prefix}{$tableName}");
            }
            return new MM_Response();
        }
        return new MM_Response("No tablename was sent for conversion", MM_Response::$ERROR);
    }
}
