<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */

abstract class MM_RBACBaseView extends MM_LoggedInUserView
{
    /**
     * Determines if the caller has permission to perform the requested action
     *
     * @param string $request
     * @return boolean
     */
    public function canCallMethod($request)
    {
        global $user, $current_user;

        $allowedMethods = ['performAction', 'enhancedPerformAction'];
        if (!in_array($request['method'], $allowedMethods) || (!isset($request['mm_action']))) {
            return false;
        }

        $requestedAction =  $request['mm_action'];

        if (parent::canCallMethod($requestedAction)) {
            $actionToTest = (isset($this->permissions[$requestedAction])) ? $requestedAction : "*";
            $userId = 0;
            if (isset($user) && isset($user->ID) && ($user->ID != "0")) {
                $userId = $user->ID;
            } else if (isset($current_user) && isset($current_user->ID) && ($current_user->ID != "0")) {
                $userId = $current_user->ID;
            }
            $employee = MM_Employee::findByUserId($userId);
            $role = $employee->isValid() ? $employee->getRoleId() : MM_Role::$ROLE_IGNORE;
            if (($role !== MM_Role::$ROLE_ADMINISTRATOR) && (MM_Utils::isSiteAdmin())) {
                //treat non-employee site admins, or site admins with a lower mm role, as if they have the mm administrator role
                $role = MM_Role::$ROLE_ADMINISTRATOR;
            } else {
                // Allow WP users with capabilities of publishing to usurp an admin role when applied to access rights.
                // This will come into play when you modify post/pages that they already have access to modify/create.
                if ((current_user_can('publish_posts') || current_user_can('publish_pages')) && ($this instanceof MM_AccessRightsView)) {
                    $role = MM_Role::$ROLE_ADMINISTRATOR;
                }
            }
            return (isset($this->permissions[$actionToTest]) && is_array($this->permissions[$actionToTest]) && in_array($role, $this->permissions[$actionToTest]));
        }
        return false;
    }
}
