<?php
class MM_RemindersView extends MM_RBACProductManagerView
{

    public static $MM_JSACTION_SEND_TEST_REMINDER = "sendTestReminder";
    public static $MM_JSACTION_SEARCH = "search";

    use MM_EnhancedDatagridView;
    use MM_NonMemberObjectView;

    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_SAVE:
                    return $this->saveReminder($post);

                case self::$MM_JSACTION_REMOVE:
                    return $this->removeReminder($post);

                case self::$MM_JSACTION_SEND_TEST_REMINDER:
                    return $this->sendTestReminder($post);

                case self::$MM_JSACTION_SEARCH:
                    return $this->search($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }


    protected function removeReminder($post)
    {
        if (isset($post["id"]) && intval($post["id"]) > 0) {
            $reminderId = sanitize_text_field($post["id"]);
            $reminder = new MM_Reminder($reminderId, false);
            $response = $reminder->delete();
            return $response;
        }

        return new MM_Response("Unable to delete reminder. No id specified.", MM_Response::$ERROR);
    }

    protected function saveReminder($post)
    {
        $reminder = new MM_Reminder();
        if (isset($post["mm_reminder_id"]) && is_numeric($post["mm_reminder_id"]) && intval($post["mm_reminder_id"]) > 0) {
            $reminder->setId($post["mm_reminder_id"]);
        }

        //set action type
        $actionType = strtolower(trim($post["mm_action_type"]));
        if (!in_array($actionType, [MM_Reminder::$MM_ACTION_SEND_EMAIL, MM_Reminder::$MM_ACTION_NOTIFY_ZAPIER, MM_Reminder::$MM_ACTION_CALL_SCRIPT])) {
            return new MM_Response("Invalid action type", MM_Response::$ERROR);
        }
        $reminder->setActionType($actionType);

        //set status
        $status = intval($post["mm_reminder_status"]);
        if (!in_array($status, [0, 1])) {
            return new MM_Response("Invalid reminder status", MM_Response::$ERROR);
        }
        $reminder->setStatus($status);

        //set trigger time
        if (!isset($post["unit_value"]) || !is_numeric($post["unit_value"])) {
            return new MM_Response("Invalid unit value", MM_Response::$ERROR);
        }
        $unitValue = intval($post["unit_value"]);
        if ($unitValue < 0) {
            return new MM_Response("Invalid unit value", MM_Response::$ERROR);
        }
        $reminder->setUnitValue($unitValue);

        //set unit type
        $unitOfMeasure = strtolower(trim($post["unit_of_measure"]));
        if (!in_array($unitOfMeasure, ['days', 'weeks', 'months', 'years'])) {
            return new MM_Response("Invalid unit of measure", MM_Response::$ERROR);
        }
        $reminder->setUnitOfMeasure($unitOfMeasure);

        //configure reminder action
        if ($actionType == MM_Reminder::$MM_ACTION_SEND_EMAIL) {
            $emailToId = $post["mm_send_email_to"];
            $emailFromId = $post["mm_send_email_from"];
            $emailCC = trim($post["mm_send_email_cc"]);
            $emailSubject = $post["mm_send_email_subject"];
            $emailBody = $post["mm_send_email_body"];

            // check email if not set to Current Member
            if ($emailToId != -1) {
                // check email is valid via employee id
                $employee = new MM_Employee($emailToId);
                if ($employee->isValid()) {
                    $email = $employee->getEmail();
                    if (!MM_Utils::hasValidEmail($email)) {
                        return new MM_Response("Empty or invalid email string " . $email, MM_Response::$ERROR);
                    }
                } else {
                    return new MM_Response("Invalid employee account " . $emailToId, MM_Response::$ERROR);
                }
            }

            // don't forget to check the CC (if included)
            if (strlen($emailCC) > 0 && !MM_Utils::hasValidEmail($emailCC)) {
                return new MM_Response("Empty or invalid CC email string " . $emailCC, MM_Response::$ERROR);
            }

            $reminder->setActionValue(MM_Reminder::prepareSendEmailValue($emailToId, $emailFromId, $emailCC, $emailSubject, $emailBody));
        } else if ($actionType == MM_Reminder::$MM_ACTION_NOTIFY_ZAPIER) {
            $mailbox = $post["mm_notify_zapier_mailbox"];
            $emailFromId = $post["mm_notify_zapier_from"];
            $additionalInfo = $post["mm_notify_zapier_additional_info"];

            if (!MM_Utils::hasValidEmail($mailbox)) {
                return new MM_Response("Empty or invalid email string.", MM_Response::$ERROR);
            }

            $reminder->setActionValue(MM_Reminder::prepareNotifyZapierValue($mailbox, $emailFromId, $additionalInfo));
        } else if ($actionType == MM_Reminder::$MM_ACTION_CALL_SCRIPT) {
            $scriptUrl = $post["mm_script_url"];

            if (!filter_var($scriptUrl, FILTER_VALIDATE_URL) || !preg_match("/^https?:\/\//", $scriptUrl)) {
                return new MM_Response("Empty or invalid URL string.", MM_Response::$ERROR);
            }
            $reminder->setActionValue(MM_Action::prepareCallScriptValue($scriptUrl));
        }

        // set (or clear) products
        $hasProductFilter = false;
        if (
            isset($post["mm_has_product_filter"]) && is_numeric($post["mm_has_product_filter"]) &&
            in_array($post["mm_has_product_filter"], ['0', '1'], true)
        ) {
            $hasProductFilter = ($post["mm_has_product_filter"] == "1");
        }

        if ($hasProductFilter) {
            $products = $post["mm_product_select"] ?? [];
            $reminder->setProductFilter($products);
        }

        return $reminder->commitData();
    }


    public function preprocessSearch($queryMeta) {}


    /**
     * Query the database and return the unprocessed results in object format
     */
    protected function getRawData($criteria = array())
    {
        global $wpdb;

        $cs = "<mm_col_sep>";
        $gs = "<mm_group_sep>";

        $dataset = $this->processQueryMeta($criteria, "r");
        $whereString = $dataset->whereString;
        $orderString = $dataset->orderString;
        $limitString = $dataset->limitString;

        $sqlResultCount = "select count(*) as total from " . MM_TABLE_REMINDERS . " r {$whereString}";
        $dataset->total = $wpdb->get_var($sqlResultCount);

        $sql = "SELECT r.*, (SELECT GROUP_CONCAT(CONCAT_WS('{$cs}',p.id,p.name) SEPARATOR '{$gs}') AS product_info " .
            "FROM " . MM_TABLE_REMINDERS_PRODUCTS . " rp INNER JOIN " . MM_TABLE_PRODUCTS . " p ON (rp.product_id = p.id) WHERE rp.reminder_id = r.id) AS products " .
            "FROM " . MM_TABLE_REMINDERS . " r " .
            "{$whereString}{$orderString}{$limitString}";
        $sql = trim($sql);
        $rows = $wpdb->get_results($sql);

        //post-process to decode product info
        foreach ($rows as $row) {
            $productArray = [];
            if (!empty($row->products)) {
                $splitProducts = explode($gs, $row->products);
                foreach ($splitProducts as $aSplitProduct) {
                    $splitProductCols = explode($cs, $aSplitProduct);
                    if (count($splitProductCols) == 2) {
                        $productArray[] = (object)["id" => $splitProductCols[0], "name" => $splitProductCols[1]]; //order needs to match the params to group_concat in the select
                    }
                }
            }
            $row->products = $productArray;
        }

        $pageCounter = ($dataset->pageNum == -1) ? 1 : $dataset->pageNum;
        $dataset->data[$pageCounter] = [];
        $prefetchSize = ($dataset->prefetchSize < 0) ? 0 : $dataset->prefetchSize;
        $pfCount = -1;

        foreach (array_chunk($rows, $dataset->pageSize) as $k => $page) {
            if ($pfCount >= $prefetchSize) {
                break;
            }
            $pageCounter = $dataset->pageNum + $k; //by default array_chunk will use numeric keys starting wih 0
            $dataset->data[$pageCounter] = $page;
            $pfCount++;
        }

        return $dataset;
    }


    protected function sendTestReminder($post)
    {
        if (!isset($post["reminder_id"])) {
            return new MM_Response("Reminder ID is required to send a test reminder", MM_Response::$ERROR);
        }

        $reminder = new MM_Reminder($post["reminder_id"]); //TODO: is this sanitized?
        if ($reminder->isValid()) {
            $testScheduledReminder = new MM_ScheduledReminder();
            $testScheduledReminder->setUserId(MM_User::$SAMPLE_USER_ID);
            $testScheduledReminder->setOrderId(MM_Order::$SAMPLE_ORDER_ID);
            $testScheduledReminder->setOrderItemId(MM_OrderItem::$SAMPLE_ORDER_ITEM_ID);
            $testScheduledReminder->setReminderId($reminder->getId());
            MM_Event::safeDoAction(MM_Event::$BILLING_REMINDER, MM_Event::packageReminderData($testScheduledReminder));
            return new MM_Response();
        } else {
            return new MM_Response("Invalid reminder; Could not execute test reminder.", MM_Response::$ERROR);
        }
    }
}
