<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 * 
 * This class responds to all of the ajax calls from 
 * 
 */
class MM_ReportingView extends MM_RBACAnalystView
{

    public static $MM_JSACTION_POLL_REPORT_DATA = "pollReportData";
    public static $MM_JSACTION_CHANGE_VISUAL_ELEMENT_DIRECTIVE = "changeVisualElementDirective";
    public static $MM_JSACTION_CLEAR_DATA_CACHE = "clearDataCache";


    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        ob_start();
        switch ($post[self::$MM_JSACTION]) {
            case self::$MM_JSACTION_POLL_REPORT_DATA:
                $response = $this->pollReportData($post);
                break;
            case self::$MM_JSACTION_CLEAR_DATA_CACHE:
                $response = $this->clearDataCache($post);
                break;
            default:
                $response = parent::performAction($post);
                $response = ($response instanceof MM_ReportResponse) ? $response : (new MM_ReportResponse());
                break;
        }
        if (!empty($post['elementName'])) {
            $response->addData("elementName", $post['elementName']);
        }
        ob_end_clean();
        return $response;
    }


    public function pollReportData($post)
    {
        $validateResponse = $this->validateReportAndElements($post);
        if (!MM_Response::isSuccess($validateResponse)) {
            return $validateResponse;
        }
        $report = $validateResponse->message;
        $elements = $post['elements'];
        $returnElements = array();
        $response = new MM_Response();
        foreach ($elements as $element) {
            $elementWidth  = (!empty($element['elementWidth'])) ? $element['elementWidth'] : "";
            $elementHeight = (!empty($element['elementHeight'])) ? $element['elementHeight'] : "";
            $params        = (!empty($element['params'])) ? $element['params'] : array();
            $directives    = (!empty($element['directives'])) ? $element['directives'] : array();
            $returnElements[$element['id']] = $report->renderVisualElement($element['id'], $params, $elementWidth, $elementHeight, $directives);
        }
        $response->addData("elements", $returnElements);
        return $response;
    }


    /**
     * Verifies that the ajax post data contained in $post contains the name of a valid report, and references a visual element
     * An instance of the report needs to be created to validate it, and as an optimization, a reference to this instance is
     * returned in the response on success
     * 
     * @param array $post The POSTed data
     * @return MM_Response Success if there is a valid report and element referenced in $post, error otherwise
     */
    protected function validateReportAndElements($post)
    {
        if (empty($post['reportName'])) {
            return new MM_Response("Error: report name not specified", MM_Response::$ERROR);
        }

        if (class_exists($post['reportName'])) {
            $tmp = new $post['reportName']();
        } else {
            $tmp = "";
        }

        if (!$tmp instanceof MM_ReportBase) {
            return new MM_Response("Error: the report specified is not a valid report", MM_Response::$ERROR);
        }

        if (empty($post['elements']) || !is_array($post['elements'])) {
            return new MM_Response("Error: visual elements not specified", MM_Response::$ERROR);
        }

        return new MM_Response($tmp);
    }


    public function clearDataCache($post)
    {
        MM_ReportDataRetriever::clearDataCache();
        return new MM_ReportResponse();
    }
}
