<?php

/**
 * 
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_StickyioShippingMethodsView extends MM_RBACProductManagerView
{
    public static $MM_JSACTION_GET_STICKYIO_SHIPPING_METHODS = "getStickyioShippingMethods";
    public static $MM_JSACTION_GET_STICKYIO_SHIPPING_DESCRIPTION = "getLLShippingDescription";


    public function __construct()
    {
        parent::__construct();
    }

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_SAVE:
                    return $this->saveStickyioShippingMethod($post);

                case self::$MM_JSACTION_REMOVE:
                    return $this->removeStickyioShippingMethod($post);

                case self::$MM_JSACTION_GET_STICKYIO_SHIPPING_METHODS:
                    return $this->getStickyioShippingMethods($post);

                case self::$MM_JSACTION_GET_STICKYIO_SHIPPING_DESCRIPTION:
                    return $this->getStickyioShippingDescription($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }

    public function getViewData($post)
    {
        return parent::getData(MM_TABLE_STICKYIO_SHIPPING_METHODS, null, $post);
    }

    private function saveStickyioShippingMethod($post)
    {
        $stickyioShippingMethod = new MM_StickyioShippingMethod();

        if (isset($post["id"]) && intval($post["id"]) > 0) {
            $stickyioShippingMethod = new MM_StickyioShippingMethod($post["id"]);

            if (!$stickyioShippingMethod->isValid()) {
                return new MM_Response("Cannot update Sticky.io shipping method mapping. Invalid ID '{$post['id']}' passed.", MM_Response::$ERROR);
            }
        }

        // get Sticky.io shipping method
        $stickyioService = MM_PaymentServiceFactory::getPaymentService(MM_PaymentService::$STICKYIO_SERVICE_TOKEN);
        $result = $stickyioService->getShippingMethod($post["stickyio_shipping_method_id"]);

        if (MM_Response::isError($result)) {
            return new MM_Response("Cannot save Sticky.io shipping method mapping. Invalid Sticky.io shipping method ID '{$post['stickyio_shipping_method_id']}' passed.", MM_Response::$ERROR);
        }

        $llShippingInfo = $result->message;

        // get MemberMouse shipping method
        $split = explode("-", $post["mm_option_key"]);
        $token = $split[0];

        //get the shipping method
        $shippingMethod = MM_ShippingMethod::getShippingMethodByToken($token);
        $shippingOption = null;

        if ($shippingMethod) {
            $optionRetrievalResponse = $shippingMethod->getShippingOptionByKey(new MM_Order(), $post["mm_option_key"]);
            $shippingOption = $optionRetrievalResponse->message;

            if ($shippingOption) {
                $mmShippingMethodName = "{$shippingOption->getName()} ({$shippingOption->getRate(true)})";
            }
        }

        if (is_null($shippingOption)) {
            return new MM_Response("Cannot save Sticky.io shipping method mapping. Invalid MemberMouse shipping method '{$post["mm_option_key"]}' passed.", MM_Response::$ERROR);
        }

        // validate mapping
        $rateMismatch = "";

        if (floatval($llShippingInfo["initial_amount"]) != $shippingOption->getRate()) {
            $rateMismatch = "RATE MISMATCH\n";
            $rateMismatch .= "Sticky.io Rate: " . _mmf($llShippingInfo["initial_amount"], "USD") . "\n";
            $rateMismatch .= "MemberMouse Rate: " . $shippingOption->getRate(true) . "\n\n";
        }

        if (!empty($rateMismatch)) {
            $errorMsg = __("Invalid shipping method mapping:\nSticky.io shipping method settings must match the MemberMouse shipping method settings.", "membermouse") . "\n\n";

            $errorMsg .= $rateMismatch;

            return new MM_Response($errorMsg, MM_Response::$ERROR);
        }

        $stickyioShippingMethod->setMMOptionKey($post["mm_option_key"]);
        $stickyioShippingMethod->setStickyioShippingMethodId($post["stickyio_shipping_method_id"]);
        $stickyioShippingMethod->setStickyioShippingMethodName($post["stickyio_shipping_method_name"]);

        return $stickyioShippingMethod->commitData();
    }

    private function removeStickyioShippingMethod($post)
    {
        global $wpdb;

        if (isset($post["id"]) && intval($post["id"]) > 0) {
            $stickyioShippingMethod = new MM_StickyioShippingMethod($post["id"], false);
            $result = $stickyioShippingMethod->delete();

            if ($result) {
                return new MM_Response();
            } else {
                return new MM_Response(__("Error removing Sticky.io shipping method mapping.", "membermouse"), MM_Response::$ERROR);
            }
        }

        return new MM_Response("Unable to delete Sticky.io shipping method mapping. No ID specified.", MM_Response::$ERROR);
    }

    private function getStickyioShippingMethods($post)
    {
        if (isset($post["campaign_id"])) {
            $stickyioService = MM_PaymentServiceFactory::getPaymentService(MM_PaymentService::$STICKYIO_SERVICE_TOKEN);
            return MM_HtmlUtils::generateSelectionsList($stickyioService->getShippingMethodsByCampaign($post["campaign_id"]));
        } else {
            return new MM_Response("Error retrieving shipping methods. No campaign ID specified.", MM_Response::$ERROR);
        }
    }

    private function getStickyioShippingDescription($post)
    {
        if (isset($post["shipping_id"])) {
            $stickyioService = MM_PaymentServiceFactory::getPaymentService(MM_PaymentService::$STICKYIO_SERVICE_TOKEN);
            return $stickyioService->getShippingDescription($post["shipping_id"]);
        } else {
            return new MM_Response("Error retrieving shipping information. No shipping method ID specified.", MM_Response::$ERROR);
        }
    }
}
