<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_UpcomingPaymentsView extends MM_RBACGeneralEmployeeView
{
    public static $MM_JSACTION_SEARCH = "search";
    public static $MM_JSACTION_RESET_FORM = "resetForm";
    public static $MM_JSACTION_STORE_SEARCH_STATE = "storeSearchState";
    public static $MM_JSACTION_SHOW_EDIT_PAYMENT_DATE_DIALOG = "showEditPaymentDateDialog";
    public static $MM_JSACTION_EDIT_PAYMENT_DATE = "editPaymentDate";
    public static $MM_JSACTION_SKIP_PAYMENT = "skipPayment";
    public static $MM_JSACTION_CANCEL_SINGLE_SUBSCRIPTION = "cancelSingleSubscription";
    public static $MM_JSACTION_CANCEL_SUBSCRIPTION = "cancelSubscriptionOnly";
    public static $MM_JSACTION_REBILL_SUBSCRIPTION_NOW = "rebillSubscriptionNow";

    use MM_EnhancedDatagridView;

    protected $defaultSortCol = "se.scheduled_date ASC";

    public function performAction($post)
    {
        $response = parent::performAction($post);

        if (!($response instanceof MM_Response)) {
            switch ($post[self::$MM_JSACTION]) {
                case self::$MM_JSACTION_RESET_FORM:
                    return $this->generateSearchForm($post);

                case self::$MM_JSACTION_STORE_SEARCH_STATE:
                    return $this->storeSearchState($post);

                case self::$MM_JSACTION_SEARCH:
                    return $this->search($post);

                case self::$MM_JSACTION_SHOW_EDIT_PAYMENT_DATE_DIALOG:
                    return $this->showEditPaymentDateDialog($post);

                case self::$MM_JSACTION_EDIT_PAYMENT_DATE:
                    return $this->editPaymentDate($post);

                case self::$MM_JSACTION_SKIP_PAYMENT:
                    return $this->skipPayment($post);

                case self::$MM_JSACTION_CANCEL_SUBSCRIPTION:
                    return $this->cancelSubscriptionOnly($post);

                case self::$MM_JSACTION_REBILL_SUBSCRIPTION_NOW:
                    return $this->rebillSubscriptionNow($post);

                case self::$MM_JSACTION_CANCEL_SINGLE_SUBSCRIPTION:
                    return $this->cancelSingleSubscription($post);

                default:
                    return new MM_Response($response);
            }
        } else {
            return $response;
        }
    }


    public function generateSearchForm($post = null)
    {
        return MM_TEMPLATE::generate(MM_MODULES . "/upcoming_payments.form.php", $post);
    }


    protected function showEditPaymentDateDialog($post)
    {
        $info = $this->createValueObjectFromRequest($post);
        return new MM_Response(MM_TEMPLATE::generate(MM_MODULES . "/upcoming_payments_edit.dialog.php", $info));
    }


    public function preprocessSearch($queryMeta)
    {
        //add prefixes for fields not in the primary table
        $sortInstructions = $queryMeta->sortCol ?? "";
        $sortComponents = explode(" ", strtolower($sortInstructions));
        $sortCol = $sortComponents[0] ?? "";
        if (!empty($sortCol)) {
            if ($sortCol == "description") {
                $queryMeta->sortCol = "oi.description";
            }
        }

        if (isset($queryMeta->query->conditions)) {
            foreach ($queryMeta->query->conditions as $aCondition) {
                if (isset($aCondition->name)) {
                    switch ($aCondition->name) {
                        case "se.scheduled_date":
                            $aCondition->value = MM_utils::dateToUTC($aCondition->value, "Y-m-d H:i:s");
                            break;
                        default:
                            break;
                    }
                }
            }
        }
    }

    /**
     * Query the database and return the unprocessed results in object format
     */
    protected function getRawData($criteria = array())
    {
        global $wpdb;

        $scheduledPaymentsTable = MM_TABLE_SCHEDULED_PAYMENTS;
        $scheduledEventsTable = MM_TABLE_SCHEDULED_EVENTS;
        $orderItemsTable = MM_TABLE_ORDER_ITEMS;
        $ordersTable = MM_TABLE_ORDERS;
        $userDataTable = MM_TABLE_USER_DATA;
        $paymentServicesTable = MM_TABLE_PAYMENT_SERVICES;
        $paymentEventType = MM_ScheduledEvent::$PAYMENT_SERVICE_EVENT;

        $dataset = $this->processQueryMeta($criteria);
        $whereString = $dataset->whereString;
        $orderString = $dataset->orderString;
        $limitString = $dataset->limitString;

        $sql = "SELECT sp.id, se.scheduled_date, o.order_number, oi.recurring_amount, oi.currency, oi.id as order_item_id, mud.last_name, mud.first_name, " .
            "u.user_email, o.user_id, oi.description, p.name as payment_service, sp.billing_status, se.status ";

        $sqlFrom =  "FROM {$scheduledPaymentsTable} sp inner join {$scheduledEventsTable} se on ((sp.id = se.id) AND (se.event_type = {$paymentEventType}))" .
            "INNER JOIN {$orderItemsTable} oi ON (sp.order_item_id = oi.id) " .
            "INNER JOIN {$ordersTable} o ON (oi.order_id = o.id) " .
            "INNER JOIN {$userDataTable} mud ON (o.user_id = mud.wp_user_id) " .
            "INNER JOIN {$wpdb->users} u ON (mud.wp_user_id = u.ID) " .
            "INNER JOIN {$paymentServicesTable} p ON (o.payment_id = p.id) ";

        //constrain to unprocessed schedules
        $whereString = empty($whereString) ? "WHERE " : "{$whereString} AND ";
        $whereString .= "(se.status != " . MM_ScheduledEvent::$EVENT_PROCESSED . ") ";

        //filter by products
        if (isset($dataset->query->products) && is_array($dataset->query->products)) {
            foreach ($dataset->query->products as $k => $v) {
                if (!is_numeric($v)) {
                    unset($dataset->query->products[$k]); //integers only, discard all non-integers
                }
            }

            if (count($dataset->query->products)) {
                $itemIdString = implode(",", $dataset->query->products);
                $whereString = empty($whereString) ? "WHERE " : "{$whereString} AND ";
                $whereString .= $wpdb->prepare("((oi.item_type = %s) AND (oi.item_id IN ({$itemIdString})))", MM_OrderItem::$ORDER_ITEM_TYPE_PRODUCT);
            }
        }

        //filter by coupons
        if (isset($dataset->query->coupons) && is_array($dataset->query->coupons)) {
            foreach ($dataset->query->coupons as $k => $v) {
                if (!is_numeric($v)) {
                    unset($dataset->query->coupons[$k]); //integers only, discard all non-integers
                }
            }

            if (count($dataset->query->coupons)) {
                //coupon filtering is a special case, because coupons are special order items in the same order as the item the transaction is tied to...
                //to filter correctly, we will conditionally join again the order items table, using the coupon type
                $couponIdString = implode(",", $dataset->query->coupons);
                $sqlFrom .= "INNER JOIN {$orderItemsTable} coupon_items ON ((coupon_items.item_type='" . MM_OrderItem::$ORDER_ITEM_TYPE_COUPON . "') " .
                    "AND (t.order_id = coupon_items.order_id)) ";
                $whereString = empty($whereString) ? "WHERE " : "{$whereString} AND ";
                $whereString .= $wpdb->prepare("(coupon_items.item_id IN ({$couponIdString}))", MM_OrderItem::$ORDER_ITEM_TYPE_PRODUCT);
            }
        }

        $sqlResultCount = "select count(*) as total {$sqlFrom} {$whereString}";
        $dataset->total = $wpdb->get_var($sqlResultCount);

        $sqlQuery = "{$sql} {$sqlFrom} {$whereString}{$orderString}{$limitString}";
        $sqlQuery = trim($sqlQuery);
        $rows = $wpdb->get_results($sqlQuery);

        foreach ($rows as $row) {
            $row->scheduled_date = MM_Utils::dateToLocal($row->scheduled_date, "Y-m-d H:i");
            $row->formatted_amount = _mmf($row->recurring_amount, $row->currency);
        }

        $pageCounter = ($dataset->pageNum == -1) ? 1 : $dataset->pageNum; //TODO: implement -1 as a marker for unpaginated
        $dataset->data[$pageCounter] = [];
        $prefetchSize = ($dataset->prefetchSize < 0) ? 0 : $dataset->prefetchSize;
        $pfCount = -1;

        foreach (array_chunk($rows, $dataset->pageSize) as $k => $page) {
            if ($pfCount >= $prefetchSize) {
                break;
            }
            $pageCounter = $dataset->pageNum + $k; //by default array_chunk will use numeric keys starting wih 0
            $dataset->data[$pageCounter] = $page;
            $pfCount++;
        }

        return $dataset;
    }


    protected function storeSearchState($post)
    {
        if (isset($post["mm_show_search"])) {
            if (isset($post["mm_admin_id"])) {
                $optionName = MM_OptionUtils::$OPTION_KEY_SHOW_UPCOMING_PAYMENTS_SEARCH . "-" . $post["mm_admin_id"];
                MM_OptionUtils::setOption($optionName, $post["mm_show_search"]);
                return new MM_Response();
            } else {
                return new MM_Response("Unable to store search state. Couldn't find current user ID.", MM_Response::$ERROR);
            }
        } else {
            return new MM_Response("Unable to store search state. Missing current state value.", MM_Response::$ERROR);
        }
    }


    protected function editPaymentDate($post)
    {
        if (isset($post['scheduledPaymentId']) && isset($post['mm_next_rebill_date'])) {
            $scheduledPaymentEvent = new MM_ScheduledPaymentEvent(intval($post["scheduledPaymentId"]));
            if ($scheduledPaymentEvent->isValid()) {
                $sv = new MM_SubscriptionsView(); //The subscriptions tab in member details already has logic to modify payment dates, reuse that
                $response = $sv->saveSubscription(["order_item_id" => $scheduledPaymentEvent->getOrderItemId(), 'mm_next_rebill_date' => $post['mm_next_rebill_date']]);
                if (MM_Response::isSuccess($response)) {
                    $response->message = __("Payment date updated successfully", "membermouse");
                }
                return $response;
            }
        }

        return new MM_Response(__("Subscription payment not found", "membermouse"), MM_Response::$ERROR);
    }


    protected function cancelSingleSubscription($post)
    {
        global $wpdb;

        if (isset($post['scheduledPaymentId'])) {
            $spTable = MM_TABLE_SCHEDULED_PAYMENTS;
            $sql = $wpdb->prepare("SELECT order_item_id FROM {$spTable} WHERE id = %d", $post['scheduledPaymentId']);
            $orderItemId = $wpdb->get_var($sql);
            if (!empty($orderItemId)) {
                $post['mm_order_item_id'] = $orderItemId;
            }
        }
        return $this->cancelSubscriptionOnly($post);
    }


    protected function cancelSubscriptionOnly($post)
    {
        $subscriptionsView = new MM_SubscriptionsView();
        return $subscriptionsView->cancelSubscriptionOnly($post);
    }


    public function skipPayment($post)
    {
        if (isset($post["scheduledPaymentId"])) {
            $scheduledPaymentEvent = new MM_ScheduledPaymentEvent(intval($post["scheduledPaymentId"]));
            if ($scheduledPaymentEvent->isValid()) {
                $orderItem = new MM_OrderItem($scheduledPaymentEvent->getOrderItemId());
                $currentPaymentDate = $scheduledPaymentEvent->getScheduledDate();
                $nextPaymentDate = MM_ScheduledPaymentService::calculateNextPaymentDate($orderItem, $currentPaymentDate);
                $nextPaymentDateTS = strtotime($nextPaymentDate);
                if ($nextPaymentDateTS > strtotime($currentPaymentDate)) {
                    $scheduledPaymentEvent->setScheduledDate($nextPaymentDate);
                    $scheduledPaymentEvent->commitData(false);
                    $formattedDate = MM_Utils::dateToLocal($nextPaymentDateTS);
                    return new MM_Response(__("Payment adjusted to ", "membermouse") . $formattedDate);
                }
            }
        }
        return new MM_Response(__("An error was encountered when attempting to adjust the payment date", "membermouse"), MM_Response::$ERROR);
    }


    protected function rebillSubscriptionNow($post)
    {
        if (isset($post["scheduledPaymentId"])) {
            $scheduledPaymentEvent = new MM_ScheduledPaymentEvent(intval($post["scheduledPaymentId"]));
            if ($scheduledPaymentEvent->isValid()) {
                $scheduledPaymentEvent->setBatchId("manual");
                $paymentResponse = $scheduledPaymentEvent->execute();
                return new MM_Response($paymentResponse->message);
            } else {
                return new MM_Response("Unable to rebill subscription: Subscription id was invalid", MM_Response::$ERROR);
            }
        } else {
            return new MM_Response("Unable to rebill subscription: Subscription id was missing", MM_Response::$ERROR);
        }
    }
}
