<?php

/**
 *
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */

function isLocalInstall($specificServer = "localhost")
{
    if (isset($_SERVER["SERVER_NAME"]) && strlen($_SERVER["SERVER_NAME"]) > 0) {
        if (preg_match("/(" . $specificServer . ")/", $_SERVER["SERVER_NAME"])) {
            return true;
        }
    }

    return true;
}


function mmShouldUseLegacyDB()
{
    global $wpdb;

    $tableExists = function ($tableName) use ($wpdb) {
        $query = $wpdb->prepare('SHOW TABLES LIKE %s', $wpdb->esc_like($tableName));
        return ($wpdb->get_var($query) == $tableName);
    };

    if (!$tableExists($wpdb->prefix . "mm_user_data") && $tableExists("mm_user_data")) {
        return true;
    }
    return false;
}

$useLegacyDB = get_option("mm-option-use-legacy-db", null);
if ($useLegacyDB == null) {
    $useLegacyDB = mmShouldUseLegacyDB();
    update_option("mm-option-use-legacy-db", $useLegacyDB);
}

global $wpdb;
define("MM_PREFIX", $useLegacyDB ? "" : $wpdb->prefix);
define("MM_LANGUAGE_DOMAIN", "membermouse");

$centralServer = "https://hub.membermouse.com/index.php?q=/";
$centralServerUrl = "https://hub.membermouse.com";
$centralServerPrettyUrl = "https://hub.membermouse.com";

$reservedGetParams = array(
    's' => 1,
    'p' => 1,
    'page_id' => 1,
    'name' => 1,
);

if (!defined("MM_CENTRAL_SERVER_URL")) {
    define("MM_CENTRAL_SERVER_URL", $centralServerUrl);
}
if (!defined("MM_PRETTY_CENTRAL_SERVER_URL")) {
    define("MM_PRETTY_CENTRAL_SERVER_URL", $centralServerPrettyUrl);
}
if (!defined("MM_CENTRAL_SERVER")) {
    define("MM_CENTRAL_SERVER", $centralServer);
}
define("MM_PLUGIN_ABSPATH", dirname(dirname(__FILE__)));

$pluginDirArray = explode(DIRECTORY_SEPARATOR, dirname(dirname(__FILE__)));
define("MM_PLUGIN_NAME", array_pop($pluginDirArray));
define("MM_LIB_DIR", MM_PLUGIN_ABSPATH . DIRECTORY_SEPARATOR . "lib");
define("MM_LANGUAGES_DIR", MM_PLUGIN_ABSPATH . DIRECTORY_SEPARATOR . "languages");

define("MM_RESOURCES_URL", plugins_url(MM_PLUGIN_NAME . "/resources/"));
define("MM_IMAGES_URL", MM_RESOURCES_URL . "images/");

define("MM_NO_DATA", "&mdash;");
define("MM_GET_KEY", "345346539284890489234");

/**
 * jQuery UI theme MemberMouse uses. Might want to think about
 * making this an option at some point in the future
 **/
define("MM_JQUERY_UI_THEME", "smoothness");

/** Member Mouse Session Lifespan (In seconds) **/
define("MM_SESSION_LIFESPAN", 86400); // 24 hours

/** Set this to true if the current version of the plugin is a beta version **/
define("MM_IS_BETA", false);

define("MM_TYPE_MEMBERSHIP_LEVEL", "member_type");
define("MM_TYPE_BUNDLE", "access_tag");
define("MM_TYPE_POST", "post");
define("MM_TYPE_PRODUCT", "product");
define("MM_TYPE_CUSTOM_FIELD", "custom_field");
define("MM_TYPE_EMPLOYEE_ACCOUNT", "employee_account");

/** PURCHASE LINK STYLES **/
define("MM_LINK_STYLE_EXPLICIT", "explicit-links");
define("MM_LINK_STYLE_REFERENCE", "reference-links");

/** DATABASE MYSQL DRIVERS **/
define("MM_MYSQL_DRIVER", "mysql");
define("MM_MYSQLI_DRIVER", "mysqli");

/** DATABASE TABLE NAMES **/
define("MM_TABLE_QUEUE_TASKS", MM_PREFIX . "mm_queue_tasks");
define("MM_TABLE_SESSIONS", MM_PREFIX . "mm_sessions");
define("MM_TABLE_COUPONS", MM_PREFIX . "mm_coupons");
define("MM_TABLE_USER_DEFINED_PAGES", MM_PREFIX . "mm_user_defined_pages");
define("MM_TABLE_COUPON_RESTRICTIONS", MM_PREFIX . "mm_coupon_restrictions");
define("MM_TABLE_COUPON_USAGE", MM_PREFIX . "mm_coupon_usage");
define("MM_TABLE_BUNDLES", MM_PREFIX . "mm_bundles");
define("MM_TABLE_APPLIED_BUNDLES", MM_PREFIX . "mm_applied_bundles");
define("MM_TABLE_BUNDLE_PRODUCTS", MM_PREFIX . "mm_bundle_products");
define("MM_TABLE_BUNDLE_CATEGORIES", MM_PREFIX . "mm_bundle_categories");
define("MM_TABLE_MEMBERSHIP_LEVELS", MM_PREFIX . "mm_membership_levels");
define("MM_TABLE_MEMBERSHIP_LEVEL_PRODUCTS", MM_PREFIX . "mm_membership_level_products");
define("MM_TABLE_MEMBERSHIP_LEVEL_CATEGORIES", MM_PREFIX . "mm_membership_level_categories");
define("MM_TABLE_COMMISSION_PROFILES", MM_PREFIX . "mm_commission_profiles");
define("MM_TABLE_RETENTION_REPORTS", MM_PREFIX . "mm_retention_reports");
define("MM_TABLE_CORE_PAGES", MM_PREFIX . "mm_core_pages");
define("MM_TABLE_ORDERS", MM_PREFIX . "mm_orders");
define("MM_TABLE_CORE_PAGE_TYPES", MM_PREFIX . "mm_core_page_types");
define("MM_TABLE_POSTS_ACCESS", MM_PREFIX . "mm_posts_access");
define("MM_TABLE_SMARTTAGS", MM_PREFIX . "mm_smarttags");
define("MM_TABLE_SMARTTAG_GROUPS", MM_PREFIX . "mm_smarttag_groups");
define("MM_TABLE_PRODUCTS", MM_PREFIX . "mm_products");
define("MM_TABLE_EMPLOYEE_ACCOUNTS", MM_PREFIX . "mm_employee_accounts");
define("MM_TABLE_EVENT_LOG", MM_PREFIX . "mm_log_events");
define("MM_TABLE_CAPTCHA_LOG", MM_PREFIX . "mm_log_captcha");
define("MM_TABLE_API_KEYS", MM_PREFIX . "mm_api_keys");
define("MM_TABLE_ACTIONS", MM_PREFIX . "mm_actions");
define("MM_TABLE_CUSTOM_FIELDS", MM_PREFIX . "mm_custom_fields");
define("MM_TABLE_CUSTOM_FIELD_OPTIONS", MM_PREFIX . "mm_custom_field_options");
define("MM_TABLE_CUSTOM_FIELD_DATA", MM_PREFIX . "mm_custom_field_data");
define("MM_TABLE_VERSION_RELEASES", MM_PREFIX . "mm_version_releases");
define("MM_TABLE_EMAIL_SERVICE_PROVIDERS", MM_PREFIX . "mm_email_service_providers");
define("MM_TABLE_EMAIL_PROVIDER_MAPPINGS", MM_PREFIX . "mm_email_provider_mappings");
define("MM_TABLE_EMAIL_PROVIDER_BUNDLE_MAPPINGS", MM_PREFIX . "mm_email_provider_bundle_mappings");
define("MM_TABLE_AFFILIATE_PROVIDERS", MM_PREFIX . "mm_affiliate_providers");
define("MM_TABLE_AFFILIATE_PROVIDER_MAPPINGS", MM_PREFIX . "mm_affiliate_provider_mappings");
define("MM_TABLE_AFFILIATE_REBILL_COMMISSIONS", MM_PREFIX . "mm_affiliate_rebill_commissions");
define("MM_TABLE_AFFILIATE_PARTNER_PAYOUTS", MM_PREFIX . "mm_affiliate_partner_payouts");
define("MM_TABLE_TRANSACTION_KEY", MM_PREFIX . "mm_transaction_key");
define("MM_TABLE_LOGIN_TOKEN", MM_PREFIX . "mm_login_token");
define("MM_TABLE_PAYMENT_SERVICES", MM_PREFIX . "mm_payment_services");
define("MM_TABLE_COUNTRIES", MM_PREFIX . "mm_countries");
define("MM_TABLE_COUNTRY_SUBDIVISIONS", MM_PREFIX . "mm_country_subdivisions");
define("MM_TABLE_ORDER_ITEMS", MM_PREFIX . "mm_order_items");
define("MM_TABLE_ORDER_ITEM_ACCESS", MM_PREFIX . "mm_order_item_access");
define("MM_TABLE_ORDER_COUPONS", MM_PREFIX . "mm_order_coupons");
define("MM_TABLE_SHIPPING_METHODS", MM_PREFIX . "mm_shipping_methods");
define("MM_TABLE_USER_DATA", MM_PREFIX . "mm_user_data");
define("MM_TABLE_SOCIAL_LOGIN_PROVIDERS", MM_PREFIX . "mm_social_login_providers");
define("MM_TABLE_SOCIAL_LOGIN_LINKED_PROFILES", MM_PREFIX . "mm_social_login_linked_profiles");
define("MM_TABLE_REMINDERS", MM_PREFIX . "mm_reminders");
define("MM_TABLE_REMINDERS_PRODUCTS", MM_PREFIX . "mm_reminders_products");

/** Scheduler **/
define("MM_TABLE_SCHEDULED_EVENTS", MM_PREFIX . "mm_scheduled_events");
define("MM_TABLE_SCHEDULED_PAYMENTS", MM_PREFIX . "mm_scheduled_payments");
define("MM_TABLE_SCHEDULED_REMINDERS", MM_PREFIX . "mm_scheduled_reminders");
define("MM_TABLE_SCHEDULED_PAYMENT_JOB_LINKS", MM_PREFIX . "mm_scheduled_payment_job_links");
define("MM_TABLE_RESQUE_JOB_LINKS", MM_PREFIX . "mm_resque_job_links");
if (!defined("MM_SCHEDULING_SERVER_URL")) {
    define("MM_SCHEDULING_SERVER_URL", "{$centralServerUrl}/scheduler.php");
}

/** PAYMENT SERVICE TABLES **/
define("MM_TABLE_CARD_ON_FILE", MM_PREFIX . "mm_card_on_file");
define("MM_TABLE_TRANSACTION_LOG", MM_PREFIX . "mm_transaction_log");
define("MM_TABLE_AUTHNET_AIM_TRANSACTIONS", MM_PREFIX . "mm_authorizenet_aim_transactions");
define("MM_TABLE_AUTHNET_ARB_SUBSCRIPTIONS", MM_PREFIX . "mm_authorizenet_arb_subscriptions");
define("MM_TABLE_AUTHNET_ARB_SUBSCRIPTION_HISTORY", MM_PREFIX . "mm_authorizenet_arb_subscription_history");
define("MM_TABLE_AUTHNET_PENDING_OVERDUE_SUBSCRIPTIONS", MM_PREFIX . "mm_authorizenet_pending_overdue_subscriptions");
define("MM_TABLE_PAYPAL_IPN_LOG", MM_PREFIX . "mm_paypal_ipn_log");
define("MM_TABLE_PAYPAL_SUBSCR_LINKS", MM_PREFIX . "mm_paypal_subscr_links");
define("MM_TABLE_COINBASE_IPN_LOG", MM_PREFIX . "mm_coinbase_ipn_log");
define("MM_TABLE_COINBASE_SUBSCR_LINKS", MM_PREFIX . "mm_coinbase_subscr_links");
define("MM_TABLE_COINBASE_BUTTON_LINKS", MM_PREFIX . "mm_coinbase_button_links");
define("MM_TABLE_COINBASEMINIMAL_IPN_LOG", MM_PREFIX . "mm_coinbaseminimal_ipn_log");
define("MM_TABLE_COINBASEMINIMAL_BUTTON_LINKS", MM_PREFIX . "mm_coinbaseminimal_button_links");
define("MM_TABLE_COINBASEMINIMAL_SCHEDULED_EVENTS", MM_PREFIX . "mm_coinbaseminimal_scheduled_events");
define("MM_TABLE_COINBASEMINIMAL_TRANSACTIONS", MM_PREFIX . "mm_coinbaseminimal_transactions");
define("MM_TABLE_STRIPE_CUSTOMER_LINKS", MM_PREFIX . "mm_stripe_customer_links");
define("MM_TABLE_STRIPE_CHARGES", MM_PREFIX . "mm_stripe_charges");
define("MM_TABLE_STRIPE_MEMBERSHIPS", MM_PREFIX . "mm_stripe_memberships");
define("MM_TABLE_STRIPE_SUBSCRIPTION_PAYMENTS", MM_PREFIX . "mm_stripe_subscription_payments");
define("MM_TABLE_STRIPE_WEBHOOKS", MM_PREFIX . "mm_stripe_webhooks");
define("MM_TABLE_STRIPE_PRODUCTS", MM_PREFIX . "mm_stripe_products");
define("MM_TABLE_STRIPE_COUPONS", MM_PREFIX . "mm_stripe_coupons");
define("MM_TABLE_CLICKBANK_PRODUCT_LINKS", MM_PREFIX . "mm_clickbank_product_links");
define("MM_TABLE_CLICKBANK_IPN_LOG", MM_PREFIX . "mm_clickbank_ipn_log");
define("MM_TABLE_CLICKBANK_ORDER_ITEM_LINKS", MM_PREFIX . "mm_clickbank_order_item_links");
define("MM_TABLE_BRAINTREE_CUSTOMER_LINKS", MM_PREFIX . "mm_braintree_customer_links");
define("MM_TABLE_BRAINTREE_CHARGES", MM_PREFIX . "mm_braintree_charges");
define("MM_TABLE_AUTHNET_CIM_CUSTOMER_LINKS", MM_PREFIX . "mm_authorizenet_cim_customer_links");
define("MM_TABLE_AUTHNET_CIM_CHARGES", MM_PREFIX . "mm_authorizenet_cim_charges");
define("MM_TABLE_LIMELIGHT_PRODUCTS", MM_PREFIX . "mm_limelight_products");
define("MM_TABLE_LIMELIGHT_SHIPPING_METHODS", MM_PREFIX . "mm_limelight_shipping_methods");
define("MM_TABLE_LIMELIGHT_SUBSCRIPTION_LINKS", MM_PREFIX . "mm_limelight_subscription_links");
define("MM_TABLE_LIMELIGHT_CHARGES", MM_PREFIX . "mm_limelight_charges");
define("MM_TABLE_LIMELIGHT_IPN_LOG", MM_PREFIX . "mm_limelight_ipn_log");

define("MM_TABLE_STICKYIO_PRODUCTS", MM_PREFIX . "mm_stickyio_products");
define("MM_TABLE_STICKYIO_SHIPPING_METHODS", MM_PREFIX . "mm_stickyio_shipping_methods");
define("MM_TABLE_STICKYIO_SUBSCRIPTION_LINKS", MM_PREFIX . "mm_stickyio_subscription_links");
define("MM_TABLE_STICKYIO_CHARGES", MM_PREFIX . "mm_stickyio_charges");
define("MM_TABLE_STICKYIO_IPN_LOG", MM_PREFIX . "mm_stickyio_ipn_log");

define("MM_TABLE_TEST_CARDONFILE", MM_PREFIX . "mm_test_cardonfile");


/** SHIPPING METHOD TABLES **/
define("MM_TABLE_FLATRATE_SHIPPING_OPTIONS", MM_PREFIX . "mm_flatrate_shipping_options");

/** Reporting Data Tables **/
define("MM_TABLE_REPORT_DATA_CACHE", MM_PREFIX . "mm_report_data_cache");

define("MM_TABLE_DIAGNOSTIC_LOG", MM_PREFIX . "mm_diagnostic_log");

/** MODULE NAMES **/
define("MM_MODULE_DASHBOARD", "mmdashboard");
define("MM_MODULE_VERSION_HISTORY", "version_history");
define("MM_MODULE_REPAIR_INSTALL", "repair_install");
define("MM_MODULE_MIGRATE_DB", "migrate_db");
define("MM_MODULE_DIAGNOSTICS", "diagnostics");
define("MM_MODULE_LICENSE", "license");
define("MM_MODULE_PRODUCT_SETTINGS", "product_settings");
define("MM_MODULE_CHECKOUT_SETTINGS", "checkout_settings");
define("MM_MODULE_MEMBERSHIP_LEVELS", "membership_levels");
define("MM_MODULE_BUNDLES", "bundles");
define("MM_MODULE_AFFILIATE_SETTINGS", "affiliate_settings");
define("MM_MODULE_AFFILIATE_INTEGRATION", "affiliate_integration");
define("MM_MODULE_COMMISSION_PROFILES", "commission_profiles");
define("MM_MODULE_AFFILIATE_TRACKING", "affiliate_tracking");
define("MM_MODULE_DEVELOPER_TOOLS", "developer_tools");
define("MM_MODULE_CANCELLATION_METHOD", "cancellation_method");
define("MM_MODULE_WEBFORMS", "webforms");
define("MM_MODULE_API", "api");
define("MM_MODULE_PUSH_NOTIFICATIONS", "push_notifications");
define("MM_MODULE_WORDPRESS_HOOKS", "wordpress_hooks");
define("MM_MODULE_CUSTOM_FIELDS", "custom_field");
define("MM_MODULE_EMPLOYEES", "employees");
define("MM_MODULE_EMAIL_INTEGRATION", "email_integration");
define("MM_MODULE_PRODUCTS", "products");
define("MM_MODULE_PURCHASE_LINKS", "purchaselinks");
define("MM_MODULE_SHIPPING", "shipping");
define("MM_MODULE_CAPTCHA", "captcha");
define("MM_MODULE_COUNTRIES", "countries");
define("MM_MODULE_CHECKOUT_OTHER_SETTINGS", "checkout_other_settings");
define("MM_MODULE_PAYMENT_METHODS", "payment_methods");
define("MM_MODULE_TEST_DATA", "test_data");
define("MM_MODULE_REMINDERS", "reminders");
define("MM_MODULE_DRIP_CONTENT_SCHEDULE", "drip_content_schedule");
define("MM_MODULE_META", "page_meta");
define("MM_MODULE_MANAGE_MEMBERS", "manage_members");
define("MM_MODULE_BROWSE_MEMBERS", "members");
define("MM_MODULE_IMPORT_WIZARD", "import_wizard");
define("MM_MODULE_USER_DEFINED_PAGES", "user_defined_pages");
define("MM_MODULE_MEMBER_DETAILS", "member_details");
define("MM_MODULE_MEMBER_DETAILS_GENERAL", "details_general");
define("MM_MODULE_COURSES", "courses");
define("MM_MODULE_MEMBER_DETAILS_CUSTOM_FIELDS", "details_custom_fields");
define("MM_MODULE_MEMBER_DETAILS_ACCESS_RIGHTS", "details_access_rights");
define("MM_MODULE_MEMBER_DETAILS_USER_DEFINED", "details_user_defined");
define("MM_MODULE_MEMBER_DETAILS_TRANSACTION_HISTORY", "details_transaction_history");
define("MM_MODULE_MEMBER_DETAILS_SUBSCRIPTIONS", "details_subscriptions");
define("MM_MODULE_MEMBER_DETAILS_GIFT_HISTORY", "details_gift_history");
define("MM_MODULE_MEMBER_DETAILS_ACTIVITY_LOG", "details_activity_log");
define("MM_MODULE_MANAGE_TRANSACTIONS", "manage_transactions");
define("MM_MODULE_UPCOMING_PAYMENTS", "upcoming_payments");
define("MM_MODULE_GENERAL_SETTINGS", "general_settings");
define("MM_MODULE_OTHER_SETTINGS", "other_settings");
define("MM_MODULE_SAFE_MODE", "safe_mode");
define("MM_MODULE_REPAIR_CORE_PAGES", "repair_core_pages");
define("MM_MODULE_SUPPORT", "support");
define("MM_MODULE_SMARTTAG_LIBRARY", "smarttag.library");
define("MM_MODULE_SMARTTAG_LOOKUP", "smarttag.idlookup");
define("MM_EMAIL_TEMPLATES", "email_templates");
define("MM_MODULE_COUPONS", "coupons");
define("MM_MODULE_GIFTS", "gifts");
define("MM_MODULE_COUPON_LOG", "coupon_log");
define("MM_MODULE_PAYMENT_SETTINGS", "payment_settings");
define("MM_MODULE_EMAIL_SETTINGS", "email_settings");
define("MM_MODULE_CAPTCHA_IPN_LOG", "captcha_ipn_log");
define("MM_MODULE_ACTIVITY_LOG", "activity_log");
define("MM_MODULE_PAYPAL_IPN_LOG", "paypal_ipn_log");
define("MM_MODULE_CLICKBANK_IPN_LOG", "clickbank_ipn_log");
define("MM_MODULE_LIMELIGHT_LOG", "limelight_log");
define("MM_MODULE_STICKYIO_LOG", "stickyio_log");
define("MM_MODULE_JOBS_QUEUE_LOG", "jobs_queue_log");
define("MM_MODULE_LOGS", "logs");
define("MM_MODULE_FREE_MEMBER_FORM", "free_member_webform");
define("MM_MODULE_LOGIN_FORM", "login_webform");
define("MM_MODULE_PHP_INTERFACE", "php_interface");
define("MM_MODULE_DUPLICATE_SUBSCRIPTION_TOOL", "duplicate_subscriptions");
define("MM_MODULE_EXTENSIONS", "extensions");

define("MM_MODULE_STICKYIO_PRODUCTS", "stickyio_products");
define("MM_MODULE_STICKYIO_SHIPPING_METHODS", "stickyio_shipping_methods");

define("MM_MODULE_LIMELIGHT_PRODUCTS", "limelight_products");
define("MM_MODULE_LIMELIGHTV2_PRODUCTS", "limelightv2_products");
define("MM_MODULE_LIMELIGHT_SHIPPING_METHODS", "limelight_shipping_methods");
define("MM_MODULE_LIMELIGHTV2_SHIPPING_METHODS", "limelightv2_shipping_methods");

//reporting
define("MM_MODULE_REPORTING", "reporting");
define("MM_MODULE_GET_REPORTING", "getreporting");
define("MM_MODULE_NEW_MEMBERS_REPORT", "new_members_report");
define("MM_MODULE_NEW_MEMBERS_COMPARISON_REPORT", "new_members_comparison_report");
define("MM_MODULE_TOTAL_VALUE_REPORT", "total_value_report");
define("MM_MODULE_CUSTOMER_VALUE_REPORT", "customer_value_report");
define("MM_MODULE_SIMPLE_AVERAGE_REPORT", "simple_average_report");
define("MM_MODULE_SALES_BY_PRODUCT_REPORT", "sales_by_product_report");
define("MM_MODULE_SALES_BY_PAYMENT_SERVICE_REPORT", "sales_by_payment_service_report");
define("MM_MODULE_SALES_BY_MEMBERSHIP_REPORT", "sales_by_membership_report");
define("MM_MODULE_AFFILIATE_REPORT", "affiliate_report");

define("MM_EXPORT_FILE_MEMBERS_IMPORT_TEMPLATE", "import_members_template");

// stripe connect & auth service
define("MM_MODULE_ACCOUNT_LOGIN", "membermouse_account_login");
if (!defined('MM_AUTH_SERVICE_DOMAIN')) {
    define('MM_AUTH_SERVICE_DOMAIN', 'auth.caseproof.com');
}
define('MM_AUTH_SERVICE_URL', 'https://' . MM_AUTH_SERVICE_DOMAIN);

if (!defined('MM_STRIPE_SERVICE_DOMAIN')) {
    define('MM_STRIPE_SERVICE_DOMAIN', 'stripe.membermouse.com');
}
define('MM_STRIPE_SERVICE_URL', 'https://' . MM_STRIPE_SERVICE_DOMAIN);
define('MMOUSE_SCRIPT_URL', site_url('/index.php?plugin=mmouse'));

define('MM_CURL_USER_AGENT', 'MemberMouse/3.0 (' . site_url() . ')');
