<?php
use JShrink\Minifier;
require_once("lib/minifier.php");

class MM_ReleaseSetup
{
	public static function encodeMMService()
	{
		$contents = file_get_contents(ABSPATH."wp-content/plugins/membermouse/com/membermouse/service/class.membermouseservice.php");
		if($contents)
		{
			$contents = preg_replace("/^(\<\?php)/", "", $contents);
			$contents = preg_replace("/(\?\>)$/", "", $contents);
			$encodedClass = base64_encode($contents);
			echo $encodedClass;
			exit;
		}
	}
	
	private static function saveFile($filename, $contents)
	{
		$fh = fopen($filename, 'w') or die("can't open file");
		fwrite($fh, $contents);
		fclose($fh);
	}
	
	public static function createZip($path, $version) 
	{
		// increase script timeout value
		ini_set('max_execution_time', 300);
		
		if (!file_exists($path)) //shouldnt happen, it goes into the major-versions dir!!
		{
			mkdir($path);
		}
		
		if (!class_exists("ZipArchive"))
		{
			die("ZipArchive class not found, this functionality requires pecl zip extension and php >= 5.2.0");
		}
		
		$zip = new ZipArchive();
		if ($zip->open("{$path}/{$version}.zip", ZIPARCHIVE::CREATE | ZIPARCHIVE::OVERWRITE) !== TRUE) {
			die ("Could not open archive");
		}
		
		chdir(ABSPATH."wp-content/plugins");
		$fileCtr = 0;
		$fileList = array();
		$iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator("membermouse/",FilesystemIterator::KEY_AS_PATHNAME | FilesystemIterator::CURRENT_AS_FILEINFO | FilesystemIterator::SKIP_DOTS));
		foreach ($iterator as $key=>$value) 
		{
			$realpath = realpath($key);
			if (!MM_ReleaseSetup::isExcluded($realpath))
			{
				$fileList[$key] = $realpath;
			}
		}
		
		self::combineAndMinifyJS($fileList);
		
		foreach ($fileList as $fileInZip=>$realZipFilepath)
		{
		    $lcFile = strtolower($fileInZip);
		    if (($lcFile != "changelog.md") && ($lcFile != "readme.md")) //exclude documentation from captive libs to save space
		    {
    			if (!$zip->addFile($realZipFilepath, $fileInZip))
    			{
    				die ("ERROR: Could not add file: $fileInZip");
    			}
    			else
    			{
    				$fileCtr++;
    			}
		    }
		}
		
		$zip->close();
		echo "{$fileCtr} files added to zip file";
	}
	
	
	protected static function combineAndMinifyJS(&$fileList)
	{
		$minifyTargets = array(
				"membermouse/resources/js/common/mm-common-core.js" =>array(
				        'class.js',
				        'mm-cache.js',
				        'mm-main.js',
				        'class.ajax.js',
				        'mm-dialog.js',
						'mm-core.js',
				        'mm-iframe.js',
						'class.form.js',
						'mm-smarttag_library.js',
						'mm-payment_utils.js'
				)
		);
		
		
		foreach ($minifyTargets as $targetFilename=>$targetComponentArray)
		{
			$matchIndexes = array();
			foreach ($targetComponentArray as $targetComponentFilename)
			{
				foreach ($fileList as $k=>$eachFile)
				{
					if (strpos($eachFile,$targetComponentFilename) !== false)
					{
						//match, remove from array
						$matchIndexes[] = $k;
						break;
					}
				}
			}
			if (count($matchIndexes) == count($targetComponentArray))
			{
				//all files matched, combine and minify and remove source
				$concatenatedSource = "";
				foreach ($matchIndexes as $matchedIndex)
				{
					$concatenatedSource.=file_get_contents($fileList[$matchedIndex]);
					unset($fileList[$matchedIndex]);
				}
				//$concatenatedSource now contains the source of all the files, run it through the minifier
				$minifiedSource = JShrink\Minifier::minify($concatenatedSource,array('flaggedComments' => false));
				if (!empty($minifiedSource))
				{
					file_put_contents($targetFilename, $minifiedSource);
					$fileList[$targetFilename] = $targetFilename;
				}
			}
			
		}
	}
	
	
	protected static function isExcluded($filePath)
	{
		$exclusions = array(
			".*svn.*",
			".*git.*",
			".*/membermouse/\..*",
			".*/membermouse/com/membermouse/cache/.*\.cache",
			".*/membermouse/logs/.*",
			".*/membermouse/tests/.*",
			".*/membermouse/lib/class\.setup\.php",
			".*/membermouse/release\.php",
			".*/membermouse/lib/minifier\.php",
			".*\.DS_Store"
		);
		
		foreach ($exclusions as $exclusion)
		{			
			if (preg_match("=$exclusion=","=$filePath="))
			{
				return true;
			}
		}
		return false;
	}
}