<?php

namespace membermouse\resque;

use \Exception;
use \MM_PaymentServiceFactory;
use \MM_PaymentServiceResponse;
use \MM_ScheduledEvent;
use \MM_ScheduledPaymentEvent;
use \MM_DiagnosticLog;

if (!defined('ABSPATH')) {
    die('You are not allowed to call this page directly.');
}

class ScheduledPaymentJob extends Job
{
    public function perform()
    {
        if (!isset($this->scheduled_event_id) || empty($this->scheduled_event_id)) {
            throw new \Exception(__('"scheduled_event_id" cannot be blank', 'membermouse'));
        }

        $paymentEvent = new \MM_ScheduledPaymentEvent($this->scheduled_event_id);

        if (($paymentEvent->getStatus() == \MM_ScheduledEvent::$EVENT_PROCESSING) || ($paymentEvent->getStatus() == \MM_ScheduledEvent::$EVENT_PROCESSED)) {
            //this event is either already processed or currently being processed, so return true to remove this job from the queue
            return true;
        }

        try {
            $paymentEvent->updateStatus(\MM_ScheduledEvent::$EVENT_PROCESSING);
            $paymentService = MM_PaymentServiceFactory::getPaymentServiceById($paymentEvent->getPaymentServiceId());
            if (is_null($paymentService)) {
                throw new \Exception("Improper event configuration: Payment service with id {$paymentEvent->getPaymentServiceId()} not found");
            }

            $paymentResponse = $paymentService->processScheduledPaymentEvent($paymentEvent);
            switch ($paymentResponse->status) {
                case \MM_PaymentServiceResponse::$SUCCESS:
                    $paymentEvent->updateStatus(\MM_ScheduledEvent::$EVENT_PROCESSED); //retain batch id on success for diagnostic purposes
                    return true;
                case \MM_PaymentServiceResponse::$FAILED:
                case \MM_PaymentServiceResponse::$ERROR:
                case \MM_PaymentServiceResponse::$PENDING:
                    //clear batch id and reset status on failure so schedules are available for subsequent sweeps
                    $paymentEvent->clearStatusAndBatchId(true, true);
                    return true;
                default:
                    //status is error or unknown, treat as failure
                    throw new \Exception($paymentResponse->message ?? __("An unknown error was encountered", "membermouse"));
            }
        } catch (\Exception $e) {
            $paymentEvent->clearStatusAndBatchId(true, true);
            throw new \Exception($e->getMessage());
        }
    }
}
