<?php
$appliedBundle = MM_AppliedBundle::getAppliedBundle($p->memberId, $p->bundleId);

if ($appliedBundle->isValid()) {
    $canChangeDaysCalc = true;

    if ($appliedBundle->getStatus() != MM_Status::$ACTIVE && $appliedBundle->getStatus() != MM_Status::$EXPIRED && $appliedBundle->getStatus() != MM_Status::$PENDING_CANCELLATION) {
        $canChangeDaysCalc = false;
    }

    $customDateSelected = "";
    $fixedSelected = "";
    $joinDateSelected = "";
    $customDateValue = "";
    $fixedValue = "";
    switch ($appliedBundle->getDaysCalcMethod()) {
        case MM_DaysCalculationTypes::$CUSTOM_DATE:
            $calcMethod = MM_DaysCalculationTypes::$CUSTOM_DATE;
            $customDateValue = MM_Utils::dateToLocal($appliedBundle->getDaysCalcValue(), "m/d/Y");
            $customDateSelected = "checked";
            break;

        case MM_DaysCalculationTypes::$FIXED:
            $calcMethod = MM_DaysCalculationTypes::$FIXED;
            $fixedValue = $appliedBundle->getDaysCalcValue();
            $fixedSelected = "checked";
            break;

        default:
            $calcMethod = MM_DaysCalculationTypes::$JOIN_DATE;
            $joinDateSelected = "checked";
            break;
    }

    if ($appliedBundle->doesExpire()) {
        $expirationDate = MM_Utils::dateToLocal($appliedBundle->getExpirationDate(), "m/d/Y");
    }
?>
    <script type='text/javascript'>
        jQuery(document).ready(function() {
            jQuery("#mm-custom-date").datepicker({
                dateFormat: "mm/dd/yy"
            });
        });
    </script>
    <div id='mm-calc-method-div'>
        <input type='hidden' id='member_id' value='<?php echo $p->memberId ?>' />
        <input type='hidden' id='bundle_id' value='<?php echo $p->bundleId; ?>' />
        <?php
        $calcMethodDesc = __("This determines how MemberMouse will calculate the number of days a member has had a bundle. This is used primarily in determining where a member is in a drip content schedule and therefore what content they get access to. By default, the calculation is done based on the date the bundle was first applied to the member's account, but you can choose to have the calculation done based on a custom date or fix the number of days to a specific number.", "membermouse");
        ?>
        <p>
            <?php
            _e("'Days with Bundle' Calculation Method", "membermouse");
            echo MM_Utils::getInfoIcon($calcMethodDesc);
            ?>
        </p>
        <?php if (!$canChangeDaysCalc) { ?>
            <div style="margin-bottom:5px;">
                <?php
                echo MM_Utils::getIcon('warning', 'yellow', '1.3em', '1px');
                _e("You can modify the number of days this bundle is fixed at, but to change the calculation method you must change the bundle's status to Active.", "membermouse");
                ?>
            </div>
        <?php } ?>
        <div style="margin-bottom:5px;"></div>
        <div style="margin-bottom:5px;">
            <input type='radio' <?php echo ((!$canChangeDaysCalc) ? "disabled='disabled'" : ""); ?> onchange="mmjs.changeCalcMethodHandler('<?php echo MM_DaysCalculationTypes::$JOIN_DATE; ?>');" id='mm-calc-method-reg-date' <?php echo $joinDateSelected; ?> name='mm-calc-method' /> <?php _e("By join date", "membermouse"); ?>
        </div>
        <div style="margin-bottom:5px;">
            <input type='radio' <?php echo ((!$canChangeDaysCalc) ? "disabled='disabled'" : ""); ?> onchange="mmjs.changeCalcMethodHandler('<?php echo MM_DaysCalculationTypes::$CUSTOM_DATE; ?>');" id='mm-calc-method-custom-date' <?php echo $customDateSelected; ?> name='mm-calc-method' /> <?php _e("By custom date", "membermouse"); ?>

            <input <?php echo ((!$canChangeDaysCalc) ? "disabled='disabled'" : ""); ?> id="mm-custom-date" type="text" style="width: 152px" value="<?php echo $customDateValue; ?>" />
            <a onClick="jQuery('#mm-custom-date').focus();"><?php echo MM_Utils::getCalendarIcon(); ?></a>
        </div>
        <div style="margin-bottom:5px;">
            <input type='radio' onchange="mmjs.changeCalcMethodHandler('<?php echo MM_DaysCalculationTypes::$FIXED; ?>');" id='mm-calc-method-fixed' <?php echo $fixedSelected; ?> name='mm-calc-method' /> <?php _e("Fixed at", "membermouse"); ?> <input id="mm-fixed" type="text" value="<?php echo $fixedValue; ?>" style="width: 52px" /> <?php _e("days", "membermouse"); ?> <br />
        </div>

        <input type='hidden' id='mm-calc-method' value="<?php echo $calcMethod; ?>" />

        <?php if ($appliedBundle->doesExpire() && !$appliedBundle->isPendingCancellation()) { ?>
            <div style="width: 360px; margin-top: 8px; margin-bottom: 8px;" class="mm-divider"></div>
            <p><?php _e("Bundle Expires", "membermouse"); ?></p>
            <p>
                <input id="mm-expiration-date" type="text" style="width: 152px" value="<?php echo $expirationDate; ?>" />
                <a onClick="jQuery('#mm-expiration-date').focus();"><?php echo MM_Utils::getCalendarIcon(); ?></a>
            </p>
        <?php } ?>

        <?php
        if ($appliedBundle->isPendingCancellation()) {
            $cancellationDate = MM_Utils::dateToLocal($appliedBundle->getCancellationDate(false), "m/d/Y");
        ?>
            <div style="width: 360px; margin-top: 8px; margin-bottom: 8px;" class="mm-divider"></div>
            <?php
            if ($appliedBundle->getPendingStatus() == MM_Status::$CANCELED) {
                $pendingBundleTitle = __("Bundle Pending Cancellation", "membermouse");
                $canForceText = __("You can force cancel the bundle by setting the cancellation date to any date in the past.", "membermouse");
                $bundleDisposition = __("Bundle will be cancelled on", "membermouse");
            } else {
                $pendingBundleTitle = __("Bundle Pending Pause", "membermouse");
                $canForceText = __("You can force pause the bundle by setting the cancellation date to any date in the past.", "membermouse");
                $bundleDisposition = __("Bundle will be paused on", "membermouse");
            }
            ?>
            <p><?php echo $pendingBundleTitle; ?></p>

            <p>
                <?php echo MM_Utils::getInfoIcon(); ?>
                <em><?php echo $canForceText; ?></em>
                <br /><br />
                <?php echo $bundleDisposition; ?>
                <input id="mm-cancellation-date" name="mm-cancellation-date" type="text" style="width: 152px" value="<?php echo $cancellationDate; ?>" />
                <a onClick="jQuery('#mm-cancellation-date').focus();"><?php echo MM_Utils::getCalendarIcon(); ?></a>
            </p>
        <?php } ?>
    </div>

    <div class="mm-dialog-footer-container">
        <div class="mm-dialog-button-container">
            <a href="javascript:mmjs.saveBundleConfiguration(<?php echo $p->bundleId; ?>);" class="mm-ui-button blue"><?php _e("Save", "membermouse"); ?></a>
            <a href="javascript:mmjs.closeDialog();" class="mm-ui-button"><?php _e("Cancel", "membermouse"); ?></a>
        </div>
    </div>
    <script type='text/javascript'>
        jQuery(document).ready(function() {
            jQuery("#mm-expiration-date").datepicker({
                dateFormat: "mm/dd/yy"
            });
            jQuery("#mm-cancellation-date").datepicker({
                dateFormat: "mm/dd/yy"
            });
        });
    </script>
<?php } else {
    _e("Invalid Member ID or bundle ID", "membermouse");
} ?>