<?php
$reminder = new MM_Reminder($p->id);
$actionValue = $reminder->getActionValue();
$eventAttributes = $reminder->getEventAttributes();
$hasProductFilter = $reminder->hasProductFilter();
$status = $reminder->getStatus();
//the helper function to manage product multiselects expects a list of selected products with the product id as the array key.
//to avoid querying the products table unnecessarily, the reminder only stores product ids, so the ids need to be transformed into 
//an array with the expected format. 
$keyedProductIds = array_combine(array_values($reminder->getProductIds()), $reminder->getProductIds());

// unit options
$unitOfMeasureOptions = [
    "days"   => __("Days", "membermouse"),
    "weeks"  => __("Weeks", "membermouse"),
    "months" => __("Months", "membermouse"),
    "years"  => __("Years", "membermouse"),
];

// set defaults
$params = array("scriptUrl", "emailToId", "emailFromId", "emailCC", "emailBody", "emailSubject", "zapierMailbox", "zapierAdditionalInfo");
foreach ($params as $key) {
    if (!isset($actionValue[$key])) {
        if ($key == "emailToId") {
            $actionValue[$key] = MM_Action::$CURRENT_MEMBER_PLACEHOLDER;
        } else {
            $actionValue[$key] = "";
        }
    }
}

$templateTitle = sprintf(
    __('Upcoming Payment for Order (#%s)', 'membermouse'),
    "[MM_Order_Data name='id']",
);
$templateGreeting = sprintf(
    __('Hi %s,', 'membermouse'),
    "[MM_Member_Data name='firstName']",
);
$templateShippingMsg = sprintf(
    __('Your order will be shipped via %s', 'membermouse'),
    "[MM_Order_Data name='shippingMethod']",
);
$templateContactMsg = sprintf(
    __('If you have any questions concerning your order, feel free to contact us at %s.', 'membermouse'),
    "[MM_Employee_Data name='email']",
);
?>

<script>
    function insertBillingReminderTemplate() {
        var str = "<?php echo $templateTitle; ?>";
        jQuery("#mm-send-email-subject").val(str);

        var str = "<?php echo $templateGreeting; ?>\n\n";

        str += "<?php _e("Here are the details for your upcoming subscription payment:", "membermouse"); ?>" + "\n";
        str += "<?php _e("Order ID", "membermouse"); ?>: [MM_Order_Data name='id']\n";
        str += "<?php _e("Product Name", "membermouse"); ?>: [MM_Order_Data name='productName']\n";
        str += "<?php _e("Subtotal", "membermouse"); ?>: [MM_Order_Data name='subtotal' doFormat='true']\n";
        str += "<?php _e("Discount", "membermouse"); ?>: [MM_Order_Data name='discount' doFormat='true']\n";
        str += "<?php _e("Shipping", "membermouse"); ?>: [MM_Order_Data name='shipping' doFormat='true']\n";
        str += "<?php _e("Order Total", "membermouse"); ?>: [MM_Order_Data name='total' doFormat='true']\n\n";

        str += "<?php _e("Billing Address", "membermouse"); ?>" + ":\n";
        str += "[MM_Order_Data name='billingAddress']\n";
        str += "[MM_Order_Data name='billingCity'], [MM_Order_Data name='billingState'] [MM_Order_Data name='billingZipCode']\n";
        str += "[MM_Order_Data name='billingCountry']\n\n";

        str += "<?php _e("Shipping Address", "membermouse"); ?>" + ":\n";
        str += "[MM_Order_Data name='shippingAddress']\n";
        str += "[MM_Order_Data name='shippingCity'], [MM_Order_Data name='shippingState'] [MM_Order_Data name='shippingZipCode']\n";
        str += "[MM_Order_Data name='shippingCountry']\n\n";

        str += "<?php echo $templateShippingMsg; ?>" + "\n\n";
        str += "<?php echo $templateContactMsg; ?>";

        insertTemplate(str);
    }

    function insertTemplate(str) {
        jQuery("#mm-send-email-body").val(str);
    }
</script>
<style>
    .mm-reminder-label-column {
        white-space: nowrap;
        width: 30%;
    }

    .mm-reminder-value-column {
        width: 70%;
    }
</style>
<div id="mm-form-container">
    <input type='hidden' id='mm-reminder-id' value='<?php echo $reminder->getId(); ?>' />
    <table style="width:100%" cellpadding="2">
        <tr>
            <td class="mm-reminder-label-column"><?php _e("Status", "membermouse"); ?></td>
            <td class="mm-reminder-value-column">
                <div id="mm-status-container">
                    <input type="radio" name="status" value="1" <?php echo (($status == "1") ? "checked" : ""); ?> /> <?php _e("Active", "membermouse"); ?> &nbsp;
                    <input type="radio" name="status" value="0" <?php echo (($status == "0") ? "checked" : ""); ?> /> <?php _e("Inactive", "membermouse"); ?>
                    <input type="hidden" id="mm_reminder_status" value="<?php echo $status; ?>" />
                </div>
            </td>
        </tr>
        <tr>
            <td colspan="2">
                <div style="width: 540px; margin-top: 5px; margin-bottom:4px;" class="mm-divider"></div>
            </td>
        </tr>
        <tr>
            <td class="mm-reminder-label-column"><?php _e("When to send", "membermouse"); ?></td>
            <td class="mm-reminder-value-column">
                <input type="text" name="unit_value" id="unit_value" value="<?php echo $reminder->getUnitValue(); ?>" size="5">
                <select name="unit_of_measure" id="unit_of_measure">
                    <?php echo MM_HtmlUtils::generateSelectionsList($unitOfMeasureOptions, $reminder->getUnitOfMeasure()); ?>
                </select>&nbsp;
                <?php _e("before the next payment", "membermouse"); ?>
            </td>
        </tr>
        <tr>
            <td colspan="2">
                <div style="width: 540px; margin-top: 5px; margin-bottom:4px;" class="mm-divider"></div>
            </td>
        </tr>
        <tr>
            <td class="mm-reminder-label-column"><?php _e("Applies to", "membermouse"); ?></td>
            <td class="mm-reminder-value-column">
                <input type="radio" name="applies_to" class="reminder-applies-to" value="all" onclick="mmjs.appliesToChangeHandler()" <?php echo ($hasProductFilter ? "" : "checked"); ?> /> <?php _e("All Products", "membermouse"); ?> &nbsp;
                <input type="radio" name="applies_to" class="reminder-applies-to" value="select_products" onclick="mmjs.appliesToChangeHandler()" <?php echo (!$hasProductFilter ? "" : "checked"); ?> /> <?php _e("Selected Products", "membermouse"); ?><br>
                <input type="hidden" id="mm-has-product-filter" value="<?php echo ($hasProductFilter ? "1" : "0"); ?>" />
            </td>
        </tr>
        <tr>
            <td class="mm-reminder-label-column">&nbsp;</td>
            <td class="mm-reminder-value-column">
                <select id="mm-product-select" name="products[]" multiple style="width: 100%; <?php echo ($hasProductFilter ? "" : "display:none;"); ?>">
                    <?php echo MM_HtmlUtils::getProducts($keyedProductIds); ?>
                </select>
            </td>
        </tr>
        <tr>
            <td colspan="2">
                <div style="width: 540px; margin-top: 5px; margin-bottom:4px;" class="mm-divider"></div>
            </td>
        </tr>
        <tr>
            <td class="mm-reminder-label-column"><?php _e("Action", "membermouse"); ?></td>
            <td class="mm-reminder-value-column">
                <select id="mm-action-type" onchange="mmjs.actionChangeHandler();">
                    <?php echo MM_HtmlUtils::getActionsList($reminder->getActionType()); ?>
                </select>&nbsp;
            </td>
        </tr>
        <tr id="mm-reminder-custom-script-options" style="display:none;">
            <td class="mm-reminder-label-column">&nbsp;</td>
            <td class="mm-reminder-value-column">
                <p style="font-size:11px;">
                    <?php _e("Enter the URL of your custom script below", "membermouse"); ?>:
                </p>
                <input type='text' id='mm-script-url' value='<?php echo $actionValue["scriptUrl"]; ?>' style="width:400px; font-family:courier !important; font-size:11px;" />
                <p style="font-size:11px;">
                    <?php _e("When the event selected above occurs, MemberMouse will call the script passing an event type and any relevant data.", "membermouse"); ?>
                    <?php _e("Download the sample script below to see how to access the data passed.", "membermouse"); ?>
                </p>
                <p>
                    <a href="https://membermouse.com/assets/files/product_purchased_notification_script.php" class="mm-ui-button">
                        <?php echo MM_Utils::getIcon('shopping-cart', '', '1.3em', '1px'); ?> <?php _e("Billing Reminder Notification Script", "membermouse"); ?>
                    </a>
                    <span style="margin-left:10px;"></span>
                </p>
            </td>
        </tr>
        <tr id="mm-reminder-notify-zapier-options" style="display:none;">
            <td class="mm-reminder-label-column">&nbsp;</td>
            <td class="mm-reminder-value-column">
                <div style="line-height:22px; margin-bottom:5px;">
                    <?php _e("Mailbox", "membermouse"); ?> <a href="https://parser.zapier.com/" target="_blank" style="font-size:10px;"><?php _e("create mailbox", "membermouse"); ?></a><br />
                    <input id="mm-notify-zapier-mailbox" type="text" style="width:295px; font-family:courier; font-size:11px;" value="<?php echo $actionValue["zapierMailbox"]; ?>" />
                </div>

                <div style="line-height:22px; margin-bottom:10px;">
                    <?php _e("From", "membermouse"); ?> <br />
                    <select id="mm-notify-zapier-from" class="medium-text">
                        <?php echo MM_HtmlUtils::getEmployees($actionValue["emailFromId"]); ?>
                    </select>
                </div>

                <div class="mm-info-box yellow" style="margin-bottom:10px;">
                    <?php echo MM_Utils::getIcon('info-circle', '#8a6d3b', '1.3em', '2px', '', ''); ?>
                    <?php _e("MemberMouse will automatically send all data related to this event to Zapier. If you'd like to append additional information to the data already being sent, enter it in the field below.", "membermouse"); ?>
                </div>

                <div style="line-height:22px;">
                    <?php _e("Additional Information", "membermouse"); ?> <?php echo MM_SmartTagLibraryView::smartTagLibraryButtons("mm-send-email-body"); ?>
                    <span style="font-size:11px; color:#666666;">
                        <?php
                        $idRequired = __("you must provide an ID", "membermouse");
                        $limitedEvents = __("only with payment and affiliate events", "membermouse");
                        $validSmartTags = __("Only the following SmartTags can be used here:", "membermouse") . "\n";
                        $validSmartTags .= "[MM_Access_Decision] (" . $idRequired . ")\n";
                        $validSmartTags .= "[MM_Content_Data] (" . $idRequired . ")\n";
                        $validSmartTags .= "[MM_Content_Link] (" . $idRequired . ")\n";
                        $validSmartTags .= "[MM_CorePage_Link]\n";
                        $validSmartTags .= "[MM_CustomField_Data]\n";
                        $validSmartTags .= "[MM_Employee_Data]\n";
                        $validSmartTags .= "[MM_Member_Data]\n";
                        $validSmartTags .= "[MM_Member_Decision]\n";
                        $validSmartTags .= "[MM_Member_Link]\n";
                        $validSmartTags .= "[MM_Order_Data] (" . $limitedEvents . ")\n";
                        $validSmartTags .= "[MM_Order_Decision] (" . $limitedEvents . ")\n";
                        $validSmartTags .= "[MM_Purchase_Link]";
                        ?>
                        <br />
                        <em><?php _e("Note: Only certain SmartTags can be used here", "membermouse"); ?></em><?php echo MM_Utils::getInfoIcon($validSmartTags); ?>
                    </span>
                </div>

                <div style="margin-top:5px">
                    <textarea id="mm-notify-zapier-additional-info" style="width:450px; font-family:courier; font-size:11px;" rows="6"><?php echo htmlentities($actionValue["zapierAdditionalInfo"], ENT_QUOTES, 'UTF-8', true); ?></textarea>
                </div>
            </td>
        </tr>
        <tr id="mm-reminder-email-options" style="display:none;">
            <td class="mm-reminder-label-column">&nbsp;</td>
            <td class="mm-reminder-value-column">
                <div>
                    <?php _e("To", "membermouse"); ?>
                    <select id="mm-send-email-to" class="medium-text" style="margin-left:19px;">
                        <option value="<?php echo MM_Action::$CURRENT_MEMBER_PLACEHOLDER; ?>" <?php echo ($actionValue["emailToId"] == MM_Action::$CURRENT_MEMBER_PLACEHOLDER) ? "selected" : ""; ?>>Current Member</option>
                        <?php echo MM_HtmlUtils::getEmployees($actionValue["emailToId"]); ?>
                    </select>
                </div>

                <div>
                    <?php _e("From", "membermouse"); ?>
                    <select id="mm-send-email-from" class="medium-text">
                        <?php echo MM_HtmlUtils::getEmployees($actionValue["emailFromId"]); ?>
                    </select>
                </div>

                <div>
                    <?php _e("CC", "membermouse"); ?><?php echo MM_Utils::getInfoIcon("Enter emails addresses to CC separated by commas"); ?>
                    <input id="mm-send-email-cc" type="text" style="width:295px; font-family:courier; font-size:11px;" value="<?php echo $actionValue["emailCC"]; ?>" />
                </div>

                <div style="margin-top:5px">
                    <?php _e("Subject", "membermouse"); ?>*
                    <input id="mm-send-email-subject" type="text" style="width:386px; font-family:courier; font-size:11px;" value="<?php echo htmlentities($actionValue["emailSubject"], ENT_COMPAT | ENT_HTML401, "UTF-8"); ?>" />
                </div>

                <div style="margin-top:5px">
                    <?php _e("Body", "membermouse"); ?>* <?php echo MM_SmartTagLibraryView::smartTagLibraryButtons("mm-send-email-body"); ?>
                    <span style="font-size:11px; color:#666666;">
                        <?php _e("Insert Template", "membermouse"); ?>: <a href="javascript:insertBillingReminderTemplate();" style="color:#21759B;"><?php _e("Billing Reminder", "membermouse"); ?></a>,
                        <?php
                        $idRequired = __("you must provide an ID", "membermouse");
                        $limitedEvents = __("only with payment and affiliate events", "membermouse");
                        $validSmartTags = __("Only the following SmartTags can be used here:", "membermouse") . "\n";
                        $validSmartTags .= "[MM_Access_Decision] (" . $idRequired . "\n";
                        $validSmartTags .= "[MM_Content_Data] (" . $idRequired . ")\n";
                        $validSmartTags .= "[MM_Content_Link] (" . $idRequired . ")\n";
                        $validSmartTags .= "[MM_CorePage_Link]\n";
                        $validSmartTags .= "[MM_CustomField_Data]\n";
                        $validSmartTags .= "[MM_Employee_Data]\n";
                        $validSmartTags .= "[MM_Member_Data]\n";
                        $validSmartTags .= "[MM_Member_Decision]\n";
                        $validSmartTags .= "[MM_Member_Link]\n";
                        $validSmartTags .= "[MM_Order_Data] (" . $limitedEvents . ")\n";
                        $validSmartTags .= "[MM_Order_Decision] (" . $limitedEvents . ")\n";
                        $validSmartTags .= "[MM_Purchase_Link]";
                        ?>
                        <br />
                        <em><?php _e("Note: Only certain SmartTags can be used here", "membermouse"); ?></em><?php echo MM_Utils::getInfoIcon($validSmartTags); ?>
                    </span>
                </div>

                <div style="margin-top:5px">
                    <textarea id="mm-send-email-body" style="width:450px; font-family:courier; font-size:11px;" rows="6"><?php echo htmlentities($actionValue["emailBody"], ENT_QUOTES, 'UTF-8', true); ?></textarea>
                </div>
            </td>
        </tr>
    </table>
</div>

<script>
    mmjs.actionChangeHandler();
    jQuery(document).ready(function() {
        jQuery('input[name="status"]').change(function() {
            jQuery('#mm_reminder_status').val(jQuery(this).val());
        });
    });
</script>

<div class="mm-dialog-footer-container">
    <div class="mm-dialog-button-container">
        <a href="javascript:mmjs.save();" class="mm-ui-button blue"><?php _e("Save Reminder", "membermouse"); ?></a>
        <a href="javascript:mmjs.closeDialog();" class="mm-ui-button"><?php _e("Cancel", "membermouse"); ?></a>
    </div>
</div>