<?php

/**
 * 
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */

$view = new MM_UpcomingPaymentsView();
$view->setupEnhancedDatagrid();

//translations
$mmehdts = $view->getDefaultGridTranslations();

//images
$mmehdgi = $view->getDefaultGridImages("upcoming payment");
$mmehdgi += ['skip' => MM_Utils::getIcon('forward', 'light-blue', '1.4em', '1px', 'Skip this payment', 'margin-left:5px;')];

// Lets customize the edit icon based on the user's permission.
unset($mmehdgi['edit']);
if (MM_MemberMouseService::hasPermission(MM_MemberMouseService::$FEATURE_ADV_SUBSCRIPTION_MGMT) == MM_MemberMouseService::$ACTIVE) {
    $mmehdgi += ['edit' => MM_Utils::getIcon('pencil', 'yellow', '1.4em', '1px', 'Edit Subscription', 'margin-left:5px;')];
} else {
    $upgradeMsg = esc_html__("Additional subscription management features are only available on the Pro plan and above. Visit membermouse.com to upgrade your plan.", "membermouse");
    $mmehdgi += ['noedit' => MM_Utils::getIcon('pencil', 'grey', '1.4em', '1px', $upgradeMsg, 'margin-left:5px;')];
}
//set up a billing status mapping to pass to the javascript. This allows the javascript to pickup any future changes in the serverside mappings
$billingStatusMap = (object)[
    MM_ScheduledPaymentEvent::$EVENT_PENDING => __("Pending", "membermouse"),
    MM_ScheduledPaymentEvent::$FIRST_REBILL_FAILED => __("1st Failure", "membermouse"),
    MM_ScheduledPaymentEvent::$SECOND_REBILL_FAILED => __("2nd Failure", "membermouse"),
    MM_ScheduledPaymentEvent::$THIRD_REBILL_FAILED => __("3rd Failure", "membermouse"),
    MM_ScheduledPaymentEvent::$PERMANENTLY_FAILED => __("Failed", "membermouse"),
    MM_ScheduledPaymentEvent::$LEGACY_EVENT => __("Pending", "membermouse")
];

//get the initial data from the view, get a unique id for the grid
$dataset = json_encode($view->search());
$gridId = uniqid();

$showSearch = false;

//store advanced search form preference

global $current_user;

if (isset($current_user) && isset($current_user->ID)) {
    echo "<input type='hidden' id='mm-admin-id' value='{$current_user->ID}' />";

    // determine if this user's preference is to have the advanced search open
    $showSearchOptionName = MM_OptionUtils::$OPTION_KEY_SHOW_UPCOMING_PAYMENTS_SEARCH . "-" . $current_user->ID;
    $showSearchOptionValue = MM_OptionUtils::getOption($showSearchOptionName);

    if ($showSearchOptionValue == "1") {
        $showSearch = true;
    }
}

// Get active offsite services for display of exclusion messaging

$excludedServices = array();
$activePaymentServices = MM_PaymentServiceFactory::getAvailablePaymentServices();
foreach ($activePaymentServices as $paymentService) {
    if (!($paymentService instanceof MM_ScheduledPaymentService)) {
        $excludedServices[] = $paymentService->getName();
    }
}

?>
<div class="mm-wrap">
    <div id="mm-upcoming-payments-general-warning" style="width:750px; margin-top:20px;" class="mm-info-box yellow">
        <p>
            <i class="fa fa-exclamation-triangle" aria-hidden="true" style="color:#d00000"></i>
            <?php printf(__('The Upcoming Payments tool allows you to view, modify, and delete scheduled payments that will be billed by MemberMouse. You can find more information in our support article %1$sManaging Upcoming Payments%2$s.', 'membermouse'), '<a href="https://membermouse.com/docs/managing-upcoming-payments" target="_blank">', '</a>'); ?>
        </p>
        <p>
            <?php _e("Please exercise caution when using this feature. The Upcoming Payments tool is capable of modifying billing dates, skipping payments, and deleting subscriptions, individually or in bulk. Many changes cannot be undone, so be sure to double-check any modifications before confirming. This tool is intended for maintenance scenarios, and most daily subscription management tasks should be performed in Manage Members.", "membermouse"); ?>
        </p>
    </div>
    <?php if (count($excludedServices) > 0) { ?>
        <div style="width:750px;" class="mm-info-box yellow">
            <p>
                <i class="fa fa-exclamation-circle" aria-hidden="true" style="color:#1e3050"></i>
                <?php _e("One or more of your active payment services utilizes subscriptions that are not administered by MemberMouse. These subscriptions will not be displayed in Upcoming Payments. To modify subscriptions created in the following payment services, please log into the service and utilize its dashboard:", "membermouse"); ?>
            </p>
            <ul>
                <?php
                foreach ($excludedServices as $excludedService) {
                    echo '<span style="margin-left:20px;"><i class="fa fa-caret-right"></i> ' . $excludedService . '</span><br/>';
                }
                ?>
            </ul>
        </div>
    <?php } ?>
    <div style="margin-top:20px;" class="mm-button-container">
        <a id="mm-show-search-btn" onclick="mmjs.showSearch()" class="mm-ui-button blue" <?php echo ($showSearch) ? "style=\"display:none;\"" : ""; ?>><?php echo MM_Utils::getIcon('search-plus'); ?> <?php _e("Advanced Search", "membermouse"); ?></a>
        <a id="mm-hide-search-btn" onclick="mmjs.hideSearch()" class="mm-ui-button" <?php echo ($showSearch) ? "" : "style=\"display:none;\""; ?>><?php echo MM_Utils::getIcon('search-minus'); ?> <?php _e("Advanced Search", "membermouse"); ?></a>
    </div>

    <div style="width: 98%; margin-top: 10px; margin-bottom: 0px;" class="mm-divider"></div>

    <div id="mm-advanced-search" <?php echo ($showSearch) ? "" : "style=\"display:none;\""; ?>>
        <div id="mm-advanced-search-container" style="width:98%">
            <?php echo $view->generateSearchForm($_POST); ?>
        </div>
        <div style="width: 98%; margin-top: 0px; margin-bottom: 10px;" class="mm-divider"></div>
    </div>

    <div id='mm_manage_transactions_csv'></div>
    <div id="gridHolder" data-grid-id="<?php echo $gridId; ?>" style="width:98%">
        <!-- Container for datagrid -->
    </div>

    <?php if ($dataset != null) { ?>
        <script type="application/json" id="<?php echo "upcoming-payments-{$gridId}-data"; ?>">
            <?php echo $dataset; ?>
        </script>
    <?php } ?>
</div>
<div id='mm-edit-payment-date-dialog'></div>
<script type="text/javascript">
    jQuery(document).ready(function() {
        let mmehdgi = <?php echo json_encode($mmehdgi); ?>; //set up images
        let mmehdts = <?php echo json_encode($mmehdts); ?>; //set up translations
        mmjs.orderLink = "<?php echo MM_ModuleUtils::getUrl(MM_MODULE_MANAGE_MEMBERS, MM_MODULE_MEMBER_DETAILS_TRANSACTION_HISTORY) . "&user_id="; ?>";
        mmjs.memberLink = "<?php echo MM_ModuleUtils::getUrl(MM_MODULE_MANAGE_MEMBERS, MM_MODULE_MEMBER_DETAILS_GENERAL) . "&user_id="; ?>";
        let bsm = <?php echo json_encode($billingStatusMap); ?>; //set up billing status mapping
        mmjs.renderGrid(mmehdgi, mmehdts, bsm);
        mmjs.bindEventListeners();
    });
</script>