/*!
 * MemberMouse(TM) (http://www.membermouse.com)
 * (c) MemberMouse, LLC. All rights reserved.
 */
class MM_MigrateDBViewJS {

    constructor(moduleName) {
        //framework settings
        this.module = moduleName;
        this.method = "performAction";
        this.action = "module-handle";

        //page settings
        this.migrationInProgress = false;
        this.progressDialogId = "mm-migration-progress-dialog";
        this.progressControlId = "mm-migration-progress";
        this.progressStatusMsgId = "mm-migration-progress-text";

        //init 
        if (document.readyState !== 'loading') {
            this.registerEventHandlers();
        }
        else {
            document.addEventListener('DOMContentLoaded', this.registerEventHandlers.bind(this));
        }
    }


    showProgressDialog() {
        let dialogId = "#" + this.progressDialogId;
        this.setProgressDialogStatusMessage("");
        jQuery.blockUI({ css: { width: '700px' }, message: jQuery(dialogId) });
    }


    closeProgressDialog() {
        jQuery.unblockUI({ onUnblock: function (data) { } });
    }


    setProgressDialogStatusMessage(statusMsg) {
        let displayMsg = (statusMsg.length > 0) ? wp.i18n.__("Status:", "membermouse") + " " + statusMsg : statusMsg;
        document.getElementById(this.progressStatusMsgId).innerHTML = displayMsg;
    }


    updateProgress(current, total) {
        let progress = Math.round((current / (total)) * 100);
        document.getElementById(this.progressControlId).value = progress;
    }


    asyncRequest(params) {
        let request = Object.assign({
            "action": this.action,
            "module": this.module,
            "method": this.method
        }, params);

        return new Promise((resolve, reject) => {
            try {
                jQuery.ajax({
                    type: "POST",
                    url: wpadmin_url + 'admin-ajax.php',
                    data: request,
                    dataType: 'json',
                    dataFilter: mmJSONDatafilter
                }).then(function (data) {
                    resolve(data);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }


    async doMigration() {
        if (this.migrationInProgress == false) {
            this.migrationInProgress = true;
            let migrationStatus = wp.i18n.__("upgrade completed successfully", "membermouse");

            //show modal progress bar
            document.getElementById(this.progressControlId).value = 0;
            this.showProgressDialog();

            try {
                let currentProgress = 0;
                this.updateProgress(0, 1);
                let response = await this.asyncRequest({ "mm_action": "renameTables" });
                if (!(response.type && response.message)) {
                    throw new Error('Incorrect response received from server');
                }
                if (response.type != 'success') {
                    throw new Error(response.message);
                }
                //response.type is success, the tables were successfully renamed. The server should have sent back a list of tables
                //if the option is checked to change table collation, loop through the returned tables and convert
                let migrateCollation = document.getElementById("migrate-collation").checked;
                let totalTasks = (migrateCollation && response.message && Array.isArray(response.message)) ? response.message.length + 1 : 1;
                this.updateProgress(++currentProgress, totalTasks);

                if (migrateCollation) {
                    if (!Array.isArray(response.message)) {
                        throw new Error('Database tables were renamed successfully, but there was an error upgrading the collation');
                    }

                    for (let tableName of response.message) {
                        this.setProgressDialogStatusMessage(wp.i18n.__("changing collation for table", "membermouse") + " " + tableName);
                        let convResponse = await this.asyncRequest({ "mm_action": "changeCollation", "tableName": tableName });
                        if (!convResponse.type) {
                            throw new Error('Incorrect response received from server');
                        }
                        if (convResponse.type != 'success') {
                            throw new Error(convResponse.message);
                        }
                        this.updateProgress(++currentProgress, totalTasks);
                    }
                }
                this.migrationComplete();
            }
            catch (e) {
                this.cancelMigration(e.message);
            }
        }
    }


    returnToMemberMouse() {
        this.closeProgressDialog();
        window.location.replace(MemberMouseGlobal.adminUrl);
    }


    migrationComplete() {
        this.migrationInProgress = false;
        this.setProgressDialogStatusMessage(wp.i18n.__("Operation successful", "membermouse"));
        document.getElementById("mm-migration-ok").style.display = "";
        document.getElementById("mm-migration-ok").value = wp.i18n.__("Ok", "membermouse");

    }


    cancelMigration(reason) {
        this.migrationInProgress = false;
        this.setProgressDialogStatusMessage(reason);
        document.getElementById("mm-migration-ok").style.display = "";
        document.getElementById("mm-migration-ok").value = wp.i18n.__("Close", "membermouse");
    }


    registerEventHandlers() {
        document.getElementById("db-migration-form").addEventListener('submit', (e) => e.preventDefault(), false);
        document.getElementById("do-migration").addEventListener('click', this.doMigration.bind(this), false);
        document.getElementById("mm-migration-ok").addEventListener('click', (e) => this.returnToMemberMouse(), false);
    }
}

const mmjs = new MM_MigrateDBViewJS("MM_MigrateDBView", "");